--[[ 
GatherSageCT
Updated for Cataclysm by Csalago

Adds skill-level and other information to gathering item tooltips.
 

PATCH NOTES

v4.3c - minor fixes for undocumented changes to low level mining/smelting

v4.3b - TOC change for patch 4.2

v4.3 - TOC change for patch 4.1

v4.2 - updated Prospecting

v4.1
--Fixed a few Herbalism typos
--updated Milling

v4.0
-- First Cataclysm update
-- update to Ace3
-- updated Mining and Herbalism skills so far



]]--
-- Version Info
local MAJOR_VERSION = "4.3"


GatherSage = LibStub("AceAddon-3.0"):NewAddon("GatherSageCT", "AceConsole-3.0", "AceEvent-3.0");
GatherSage.title = "GatherSage";
GatherSage.version = MAJOR_VERSION;

function GatherSage:OnInitialize()
	self:HookTooltipMethods()
end

function GatherSage.SetItem(tooltip)
	if not tooltip.GetItem then return end
	local _, link = tooltip:GetItem()
	GatherSage.ProcessWithLink(tooltip, link)
end

function GatherSage:HookTooltipMethods()
	for _, tooltip in ipairs(self.TooltipList) do
		if tooltip then
			for _, methodName in ipairs(self.MethodList) do
				hooksecurefunc(tooltip, methodName, GatherSage.SetItem)
			end
		end
	end
end

function GatherSage.ProcessWithLink(tooltip, link)
	if (not link) then return; end
end

function GatherSage.ProcessWithName(tooltip, itemname)
	if (not itemname) then return; end
	local ttLine1 = _G[tooltip:GetName().."TextLeft1"]:GetText()
	local ttLine2 = _G[tooltip:GetName().."TextLeft2"]:GetText()
	--DEFAULT_CHAT_FRAME:AddMessage(ttLine1);
	--DEFAULT_CHAT_FRAME:AddMessage(ttLine2);
	--DEFAULT_CHAT_FRAME:AddMessage(itemname);
	if (not ttLine2) or ((string.find(ttLine2,gsMINING)) or (string.find(ttLine2,gsHERBING)))  then 
		if ((gsMine[itemname]) or (gsOre[itemname])) then
			GatherSage.MiningTooltip(tooltip, itemname);
		elseif (gsHerb[itemname]) then
			GatherSage.HerbingTooltip(tooltip, itemname);
		end
	elseif (ttLine2) and (string.find(ttLine2,gsPROSPECTABLE)) then
		GatherSage.ProspectTooltip(tooltip, itemname);
	elseif (gsOre[itemname]) then
		GatherSage.ProspectTooltip(tooltip, itemname);
	elseif (ttLine2) and (string.find(ttLine2,gsMILLABLE)) then
		GatherSage.MillingTooltip(tooltip, itemname);
	elseif (tooltip:NumLines() > 2) then
		-- scan entire tooltip for "Skinnable" in case other addons have moved the text
		for i=2, tooltip:NumLines() do
			ttScan = _G[tooltip:GetName().."TextLeft"..i]:GetText()
			if (ttScan) then
				if (string.find(ttScan,gsSKINNABLE)) then
					GatherSage.SkinningTooltip(tooltip, itemname);
				end
			end
		end
	end
end


-- Get UnitLevel functions for Skinning skill calculation

function GatherSage.GetTargetUnitMethod()
	local targetType
	if (UnitExists("mouseover")) then
		targetType = "mouseover"
	elseif (UnitExists("playertarget")) then
		targetType = "playertarget"
	end
	return targetType
end

function GatherSage.GetUnitLevel()
	local targetType = GatherSage.GetTargetUnitMethod()
	if not targetType then return end
	
	local mobLevel = UnitLevel(targetType)

	-- if mob level is too high for us to know, then artifically set it
	if (mobLevel == -1) then
		mobLevel = UnitLevel("player") + 11
	end
	
	return mobLevel
end




-- Separate Tooltip Functions per Profession

function GatherSage.MillingTooltip(frame, itemname)
    if (gsCommonPigments[itemname]) then
        local levelreq;
	local mGroup = (gsCommonPigments[itemname])
		if (mGroup == gsAshen) then
			if(gsMill[itemname]["baseskill"] == 0) then
				levelreq = "?";
			else
				levelreq = gsMill[itemname]["baseskill"];
			end
		else
			if(gsMill[mGroup]["baseskill"] == 0) then
				levelreq = "?";
			else
				levelreq = gsMill[mGroup]["baseskill"];
			end
        end
        local myskilllevel = GatherSage_GetLevel("milling");
	if myskilllevel > 0 then
		local reqmsg = " ("..levelreq..")";
		frame:SetHeight(frame:GetHeight() + 14);
		if(levelreq == "?") then
			frame:AddLine(gspRequires..levelreq..gspSkillToMill,1,0.5,0,1);
		elseif(levelreq <= myskilllevel) then
			frame:AddLine(gspRequires..levelreq..gspSkillToMill,0,1,0,1);
		else
			frame:AddLine(gspRequires..levelreq..gspSkillToMill,1,0,0,1);
		end
		frame:AddLine(gsGrey..gspCommon..gsTail..gsWhite..gsCommonPigments[itemname]..gsTail);
		if (gsRarePigments[itemname]) then
			frame:AddLine(gsGrey..gspUncommon..gsTail..gsGreen..gsRarePigments[itemname]..gsTail);
		end
	end
    end
    frame:Show()
end


function GatherSage.ProspectTooltip(frame, itemname)
    if (gsOre[itemname]) then
        local levelreq;
        if(gsOre[itemname]["baseskill"] == 0) then
            levelreq = "?";
        else
            levelreq = gsOre[itemname]["baseskill"];
        end
        local myskilllevel = GatherSage_GetLevel("mining");
		if myskilllevel > 0 then
			local reqmsg = " ("..levelreq..")";
			frame:SetHeight(frame:GetHeight() + 14);
			if(levelreq == "?") then
				frame:AddLine(gspRequires..levelreq..gspSkillToSmelt,1,0.5,0,1);
			elseif(levelreq <= myskilllevel) then
				frame:AddLine(gspRequires..levelreq..gspSkillToSmelt,0,1,0,1);
			else
				frame:AddLine(gspRequires..levelreq..gspSkillToSmelt,1,0,0,1);
			end
		end
    end
    if (gsProspect[itemname]) then
        local levelreq;
        if(gsProspect[itemname]["baseskill"] == 0) then
            levelreq = "?";
        else
            levelreq = gsProspect[itemname]["baseskill"];
        end
        local myskilllevel = GatherSage_GetLevel("prospecting");
		if myskilllevel >0 then
			local reqmsg = " ("..levelreq..")";
			frame:SetHeight(frame:GetHeight() + 14);
			if(levelreq == "?") then
				frame:AddLine(gspRequires..levelreq..gspSkillToProspect ,1,0.5,0,1);
			elseif(levelreq <= myskilllevel) then
				frame:AddLine(gspRequires..levelreq..gspSkillToProspect ,0,1,0,1);
			else
				frame:AddLine(gspRequires..levelreq..gspSkillToProspect ,1,0,0,1);
			end
			if (gsOreHasGem[itemname]) then
				for key, gemtype in pairs(gsOreHasGem[itemname]) do
					local myorehasgem = gsLight..gspChanceOf..gsTail..gemtype;
					frame:AddLine(myorehasgem);
					frame:SetHeight(frame:GetHeight() + 14);
				end
			end
		end
    end
    frame:Show()
end


function GatherSage.SkinningTooltip(frame, itemname)
	local levelreq;
	local moblvl = GatherSage.GetUnitLevel();
	local myskilllevel = GatherSage_GetLevel("skinning");
	if myskilllevel > 0 then
		if moblvl < 21 then
			levelreq = ((moblvl - 10) * 10);
			if levelreq < 1 then levelreq = 1 end
		else
			levelreq = (moblvl * 5);
		end
		local reqmsg = " ("..levelreq..")";
		frame:SetHeight(frame:GetHeight() + 14);
		if(levelreq == "?") then
			frame:AddLine(gspRequires..levelreq..gspSkinning,1,0.5,0,1);
		elseif(levelreq <= myskilllevel) then
			frame:AddLine(gspRequires..levelreq..gspSkinning,0,1,0,1);
		else
			frame:AddLine(gspRequires..levelreq..gspSkinning,1,0,0,1);
		end
	end
	frame:Show()
end


function GatherSage.MiningTooltip(frame, itemname)
    if (gsMine[itemname]) then
        local levelreq;
        if(gsMine[itemname]["baseskill"] == 0) then
            levelreq = "?";
        else
            levelreq = gsMine[itemname]["baseskill"];
        end
        local myskilllevel = GatherSage_GetLevel("mining");
       if myskilllevel > 0 then
        local reqmsg = " ("..levelreq..")";
	frame:SetHeight(frame:GetHeight() + 14);
        if(levelreq == "?") then
		frame:AddLine(gspRequires..levelreq..gspSkill,1,0.5,0,1);
        elseif(levelreq <= myskilllevel) then
		frame:AddLine(gspRequires..levelreq..gspSkill,0,1,0,1);
	else
		frame:AddLine(gspRequires..levelreq..gspSkill,1,0,0,1);
	end

	if (gsMineHasOre[itemname]) then
		local myminehasore = gsLight.."Chance of: "..gsTail..gsWhite..gsMineHasOre[itemname]..gsTail;
		frame:AddLine(myminehasore);
		frame:SetHeight(frame:GetHeight() + 14);
		local setwidno = ceil((strlen(myminehasore)-11)*7.5);
		if (frame:GetWidth() < setwidno) then frame:SetWidth(setwidno); end
	end
	if (gsMineHasStone[itemname]) then
		if (type(gsMineHasStone[itemname]) == "table") then
			for key, stonetype in pairs(gsMineHasStone[itemname]) do
				local myminehasstone = gsLight.."Chance of: "..gsTail..gsWhite..stonetype..gsTail;
				frame:AddLine(myminehasstone);
				frame:SetHeight(frame:GetHeight() + 14);
				local setwidno = ceil((strlen(myminehasstone)-11)*7.5);
				if (frame:GetWidth() < setwidno) then frame:SetWidth(setwidno); end
			end
		else
			local myminehasstone = gsLight.."Chance of: "..gsTail..gsWhite..gsMineHasStone[itemname]..gsTail;
			frame:AddLine(myminehasstone);
			frame:SetHeight(frame:GetHeight() + 14);
			local setwidno = ceil((strlen(myminehasstone)-11)*7.5);
			if (frame:GetWidth() < setwidno) then frame:SetWidth(setwidno); end
		end
	end
	if (gsMineHasGem[itemname]) then
		for key, gemtype in pairs(gsMineHasGem[itemname]) do
			local myminehasgem = gsLight..gspChanceOf..gsTail..gemtype;
			frame:AddLine(myminehasgem);
			frame:SetHeight(frame:GetHeight() + 14);
		end
	end
       end
    elseif (gsOre[itemname]) then
        local levelreq;
        if(gsOre[itemname]["baseskill"] == 0) then
            levelreq = "?";
        else
            levelreq = gsOre[itemname]["baseskill"];
        end
        local myskilllevel = GatherSage_GetLevel("mining");
       if myskilllevel > 0 then
        local reqmsg = " ("..levelreq..")";
	frame:SetHeight(frame:GetHeight() + 14);
        if(levelreq == "?") then
		frame:AddLine(gspRequires..levelreq..gspSkillToSmelt,1,0.5,0,1);
        elseif(levelreq <= myskilllevel) then
		frame:AddLine(gspRequires..levelreq..gspSkillToSmelt,0,1,0,1);
	else
		frame:AddLine(gspRequires..levelreq..gspSkillToSmelt,1,0,0,1);
	end
       end
    end
    frame:Show()
end


function GatherSage.HerbingTooltip(frame, itemname)
    local mouseFocus = GetMouseFocus();
    local mouseFocusName = mouseFocus:GetName();
    if not mouseFocusName then mouseFocusName = ""; end

    if (gsHerb[itemname]) and ((mouseFocusName =="Minimap") or (mouseFocusName =="WorldFrame")) then  -- only process if Tooltip is over Minimap or WorldFrame, Ignores herbs already picked.
        local levelreq;
        if(gsHerb[itemname]["baseskill"] == 0) then
            levelreq = "?";
        else
            levelreq = gsHerb[itemname]["baseskill"];
        end
        local myskilllevel = GatherSage_GetLevel("herbing");
      if myskilllevel >0 then
        local reqmsg = " ("..levelreq..")";
	frame:SetHeight(frame:GetHeight() + 14);
        if(levelreq == "?") then
		frame:AddLine(gspRequires..levelreq..gspSkill,1,0.5,0,1);
        elseif(levelreq <= myskilllevel) then
		frame:AddLine(gspRequires..levelreq..gspSkill,0,1,0,1);
	else
		frame:AddLine(gspRequires..levelreq..gspSkill,1,0,0,1);
	end
	if (gsHerbHasHerb[itemname]) then
		if (type(gsHerbHasHerb[itemname]) == "table") then
			for key, herbtype in pairs(gsHerbHasHerb[itemname]) do
				local myherbhasherb = gsLight..gspChanceOf..gsTail..gsWhite..herbtype..gsTail;
				frame:AddLine(myherbhasherb);
				frame:SetHeight(frame:GetHeight() + 14);
				local setwidno = ceil(strlen(myherbhasherb)*7.5);
				if (frame:GetWidth() < setwidno) then frame:SetWidth(setwidno); end
			end
		else
			local myherbhasherb = gsLight..gspChanceOf..gsTail..gsWhite..gsHerbHasHerb[itemname]..gsTail;
			frame:AddLine(myherbhasherb);
			frame:SetHeight(frame:GetHeight() + 14);
			local setwidno = ceil(strlen(myherbhasherb)*7.5);
			if (frame:GetWidth() < setwidno) then frame:SetWidth(setwidno); end
		end
	end
      end
    end
    frame:Show()
end



function GatherSage_Tooltip_OnShow(self)
    local parentFrame = self:GetParent();
    local parentFrameName = parentFrame:GetName();
    local itemName = _G[parentFrameName.."TextLeft1"]:GetText();
    GatherSage.ProcessWithName(parentFrame, itemName);
end

function GatherSage_GetLevel(skilltype)
	if (skilltype=="mining" or skilltype=="herbing" or skilltype=="skinning" or skilltype=="prospecting" or skilltype=="milling") then
		local prof1,prof2 = GetProfessions();
		--first pass
		if prof1 ~= nil then
			local skillname1, _, skillrank1 = GetProfessionInfo(prof1);
			--DEFAULT_CHAT_FRAME:AddMessage("Skill1: " ..prof1.." ".. skillname1 .. " - " .. skillrank1 .. " - " .. skilltype);  --DEBUG MESSAGE FOR SKILL NAME
			if (skillname1 == gsMINING and skilltype=="mining") then
				return skillrank1;
			elseif (skillname1 == gsHERBING and skilltype=="herbing") then
				return skillrank1;
			elseif (skillname1 == gsSKINNING and skilltype=="skinning") then
				return skillrank1;
			elseif (skillname1 == gsPROSPECT and skilltype=="prospecting") then
				return skillrank1;
			elseif (skillname1 == gsMILLING and skilltype=="milling") then
				return skillrank1;
			end   
		end
		--second pass
		if prof2 ~= nil then
			local skillname2, _, skillrank2 = GetProfessionInfo(prof2);
			--DEFAULT_CHAT_FRAME:AddMessage("Skill2: " ..prof2.." ".. skillname2 .. " - " .. skillrank2 .. " - " .. skilltype);  --DEBUG MESSAGE FOR SKILL NAME
			if (skillname2 == gsMINING and skilltype=="mining") then
				return skillrank2;
			elseif (skillname2 == gsHERBING and skilltype=="herbing") then
				return skillrank2;
			elseif (skillname2 == gsSKINNING and skilltype=="skinning") then
				return skillrank2;
			elseif (skillname2 == gsPROSPECT and skilltype=="prospecting") then
				return skillrank2;
			elseif (skillname2 == gsMILLING and skilltype=="milling") then
				return skillrank2;
			end 
		end
		return 0
	else
		return 0;
	end
end


-------------------------------
-- Hook Tooltip SetX Methods --
-------------------------------
GatherSage.TooltipList = {
	ItemRefTooltip,
	GameTooltip,
	ShoppingTooltip1,
	ShoppingTooltip2,
	--EquipCompare support
	ComparisonTooltip1,
	ComparisonTooltip2,
	--EQCompare support
	EQCompareTooltip1,
	EQCompareTooltip2,
	-- socketing interface
	--ItemSocketingDescription, -- commented out cause can't get it to work
	-- LinkWrangler support
	IRR_ItemRefTooltip1,
	IRR_ItemCompTooltip1,
	IRR_ItemCompTool11,
	IRR_ItemRefTooltip2,
	IRR_ItemCompTooltip2,
	IRR_ItemCompTool12,
	IRR_ItemRefTooltip3,
	IRR_ItemCompTooltip3,
	IRR_ItemCompTool13,
	IRR_ItemRefTooltip4,
	IRR_ItemCompTooltip4,
	IRR_ItemCompTool14,
	IRR_ItemRefTooltip5,
	IRR_ItemCompTooltip5,
	IRR_ItemCompTool15,
	-- MultiTips support
	ItemRefTooltip2,
	ItemRefTooltip3,
	ItemRefTooltip4,
	ItemRefTooltip5,
	-- Gatherer support ?
	GatherNoteTemplate,
}

GatherSage.MethodList = {
	"SetHyperlink",
	"SetBagItem",
	"SetInventoryItem",
	-- auction
	"SetAuctionItem",
	"SetAuctionSellItem",
	-- loot
	"SetLootItem",
	"SetLootRollItem",
	-- crafting
	"SetTradeSkillItem",
	"SetTrainerService",
	-- mail
	"SetInboxItem",
	"SetSendMailItem",
	-- buffs
	"SetUnitBuff",
	-- quest log
	"SetQuestItem",
	"SetQuestLogItem",
	-- trade
	"SetTradePlayerItem",
	"SetTradeTargetItem",
	-- vendor tooltip
	"SetMerchantItem",
	"SetBuybackItem",
	-- socketing interface
	--"SetSocketedItem", -- main display
	"SetSocketGem",
	"SetExistingSocketGem",
}
