﻿local PassLoot = LibStub("AceAddon-3.0"):GetAddon("PassLoot");
local L = LibStub("AceLocale-3.0"):GetLocale("PassLoot_AtlasLoot");
local module = PassLoot:NewModule(L["ModuleName"], "AceEvent-3.0");

-- Temporary functions until AtlasLoot implements their own API, so I don't have to update this if they change their database.
local AtlasLoot, WishList;
AtlasLoot = LibStub("AceAddon-3.0"):GetAddon("AtlasLoot", true);
if ( AtlasLoot ) then
  WishList = AtlasLoot:GetModule("WishList", true);
end
local function AtlasLoot_GetWishLists()
  local Table = {};
  if ( WishList and WishList.allWishLists and WishList.allWishLists[WishList.realm] ) then
    for PlayerKey, PlayerValue in pairs(WishList.allWishLists[WishList.realm]) do
      Table[PlayerKey] = {};
      for WishListKey, WishListValue in ipairs(PlayerValue) do
        Table[PlayerKey][WishListKey] = { WishListValue.info.name };
      end
    end
  end
  return Table;
end

local function AtlasLoot_CheckWishlistItem(itemID, playerName, wishlist)
  if ( WishList and WishList.allWishLists and WishList.allWishLists[WishList.realm] ) then
    for PlayerKey, PlayerValue in pairs(WishList.allWishLists[WishList.realm]) do
      if ( not playerName or playerName == PlayerKey ) then
        for WishListKey, WishListValue in ipairs(PlayerValue) do
          if ( not wishlist or wishlist == WishListValue.info.name ) then
            for ItemKey, ItemValue in pairs(WishListValue[1]) do
              if ( itemID == ItemValue[2] ) then
                return true;
              end
            end
          end
        end
      end
    end
  end
end

module.ConfigOptions_RuleDefaults = {
  -- { VariableName, Default },
  { "AtlasLootWishList",
    -- {
      -- [1] = WhichPlayer,  -- -1 = all players, -2 = current player
      -- [2] = WhichList,  -- -1 = all lists
      -- [3] = Exception,
    -- }
  },
};
module.NewFilterValue_Player = -1;
module.NewFilterValue_List = -1;

StaticPopupDialogs.PassLoot_AtlasLootWishList = {
  ["text"] = L["AtlasLoot is not loaded, rules with AtlasLoot filters will not be matched."],
	["button1"] = OKAY,
  ["timeout"] = 60,
  ["whileDead"] = 1,
  ["hideOnEscape"] = 1,
};

function module:OnEnable()
  self:RegisterDefaultVariables(self.ConfigOptions_RuleDefaults);
  self:AddWidget(self.Widget);
  self:CheckDBVersion(3, "UpgradeDatabase");
  self:RegisterMessage("PassLoot_OnProfileChanged");
end

function module:OnDisable()
  self:UnregisterDefaultVariables();
  self:RemoveWidgets();
  self:UnregisterMessage("PassLoot_OnProfileChanged");
end

function module:PassLoot_OnProfileChanged()
  -- if ( not AtlasLoot_GetWishLists or not AtlasLoot_CheckWishlistItem ) then
  if ( not AtlasLoot or not WishList ) then
    local Data
    for Key, Value in pairs(PassLoot.db.profile.Rules) do
      Data = self.Widget:GetData(Key);
      if ( #Data > 0 ) then
        local Dialog = StaticPopup_Show("PassLoot_AtlasLootWishList");
        break;
      end
    end
  end
end

function module:UpgradeDatabase(FromVersion, Rule)
  if ( FromVersion == 1 ) then
    local Table = {
      { "AtlasLootWishList", {} },
    };
    if ( Rule.AtlasLootWishList ) then
      Table[1][2][1] = { self.NewFilterValue_Player, self.NewFilterValue_List, false };
    end
    return Table;
  end
  if ( FromVersion == 2 ) then
    local Table = {
      { "AtlasLootWishList", nil },
    };
    if ( type(Rule.AtlasLootWishList) == "table" ) then
      if ( #Rule.AtlasLootWishList == 0 ) then
        return Table;
      end
    end
  end
  return;
end

function module:CreateWidget()
  local Frame = CreateFrame("Frame");
  Frame:SetWidth(240);
  Frame:SetHeight(43);
  
  Frame.Title = Frame:CreateFontString(nil, "BACKGROUND", "GameFontNormal");
  Frame.Title:SetParent(Frame);
  Frame.Title:SetPoint("TOP", Frame, "TOP");
  Frame.Title:SetText(L["AtlasLoot not loaded!"]);
  Frame.Title:Hide();

  Frame.DropDown = CreateFrame("Frame", "PassLoot_Frames_Widgets_AtlasLootWishList", Frame, "UIDropDownMenuTemplate");
  Frame.DropDown:EnableMouse(true);
  Frame.DropDown:SetHitRectInsets(15, 15, 0 ,0);
  _G[Frame.DropDown:GetName().."Text"]:SetJustifyH("CENTER");
  if ( select(4, GetBuildInfo()) < 30000 ) then
    UIDropDownMenu_SetWidth(220, Frame.DropDown);
  else
    UIDropDownMenu_SetWidth(Frame.DropDown, 220);
  end
  Frame.DropDown:SetScript("OnEnter", function() self:ShowTooltip(L["AtlasLoot WishList"], L["Selected rule will only match items in the selected Wishlist."]) end);
  Frame.DropDown:SetScript("OnLeave", function() GameTooltip:Hide() end);
  local Button = _G[Frame.DropDown:GetName().."Button"];
  Button:SetScript("OnEnter", function() self:ShowTooltip(L["AtlasLoot WishList"], L["Selected rule will only match items in the selected Wishlist."]) end);
  Button:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame.DropDown.Title = Frame.DropDown:CreateFontString(Frame.DropDown:GetName().."Title", "BACKGROUND", "GameFontNormalSmall");
  Frame.DropDown.Title:SetParent(Frame.DropDown);
  Frame.DropDown.Title:SetPoint("BOTTOMLEFT", Frame.DropDown, "TOPLEFT", 20, 0);
  Frame.DropDown.Title:SetText(L["AtlasLoot WishList"]);
  if ( select(4, GetBuildInfo()) < 30000 ) then
    Frame.DropDown.initialize = function(...) self:DropDown_Init(Frame.DropDown, ...) end;
  else
    Frame.DropDown.initialize = function(...) self:DropDown_Init(...) end;
  end
  Frame.DropDown:SetPoint("TOP", Frame, "TOP", 0, -1 * Frame.DropDown.Title:GetHeight());
  Frame.DropDown:Hide();

  Frame.Height = Frame:GetHeight();
  Frame.Width = Frame:GetWidth();
  Frame.Info = {
    L["FilterListName"],
    L["FilterListDesc"],
  };
  return Frame;
end
module.Widget = module:CreateWidget();

-- Local function to get the data and make sure it's valid data
function module.Widget:GetData(RuleNum)
  local Data = module:GetConfigOption("AtlasLootWishList", RuleNum);
  local Changed = false;
  if ( Data ) then
    if ( type(Data) == "table" and #Data > 0 ) then
      for Key, Value in ipairs(Data) do
        if ( type(Value) ~= "table" or not ( type(Value[1]) ~= "string" or Value[1] ~= -1 ) or not ( type(Value[2]) ~= "string" or Value[2] ~= -1 ) ) then
          Data[Key] = {
            module.NewFilterValue_Player,
            module.NewFilterValue_List,
            false
          };
          Changed = true;
        end
      end
    else
      Data = nil;
      Changed = true;
    end
  end
  if ( Changed ) then
  -- AceLibrary('AceConsole-2.0'):PrintLiteral(Data);
    module:SetConfigOption("AtlasLootWishList", Data);
  end
  return Data or {};
end

function module.Widget:GetNumFilters(RuleNum)
  local Value = self:GetData(RuleNum);
  return #Value;
end

function module.Widget:AddNewFilter()
  local Value = self:GetData();
  local NewTable = {
    module.NewFilterValue_Player,
    module.NewFilterValue_List,
    false
  };
  table.insert(Value, NewTable);
  module:SetConfigOption("AtlasLootWishList", Value);
end

function module.Widget:RemoveFilter(Index)
  local Value = self:GetData();
  table.remove(Value, Index);
  if ( #Value == 0 ) then
    Value = nil;
  end
  module:SetConfigOption("AtlasLootWishList", Value );
end

function module.Widget:DisplayWidget(Index)
  if ( AtlasLoot_GetWishLists and AtlasLoot_CheckWishlistItem ) then
    self.Title:Hide();
    self.DropDown:Show();
    if ( Index ) then
      module.FilterIndex = Index;
    end
    local Value = self:GetData();
    local Value_Player = Value[module.FilterIndex][1];
    local Value_List = Value[module.FilterIndex][2];
    if ( select(4, GetBuildInfo()) < 30000 ) then
      UIDropDownMenu_SetText(module:GetAtlasLootText(Value_Player, Value_List), module.Widget.DropDown);
    else
      UIDropDownMenu_SetText(module.Widget.DropDown, module:GetAtlasLootText(Value_Player, Value_List));
    end
  else
    self.Title:Show();
    self.DropDown:Hide();
  end
end

function module.Widget:GetFilterText(Index)
  local Value = self:GetData();
  if ( Value[Index][1] and Value[Index][2] ) then
    if ( AtlasLoot_GetWishLists and AtlasLoot_CheckWishlistItem ) then
      return module:GetAtlasLootText(Value[Index][1], Value[Index][2]);
    else
      return L["AtlasLoot not loaded!"];
    end
  end
end

function module.Widget:IsException(RuleNum, Index)
  local Data = self:GetData(RuleNum);
  return Data[Index][3];
end

function module.Widget:SetException(RuleNum, Index, Value)
  local Data = self:GetData(RuleNum);
  Data[Index][3] = Value;
  module:SetConfigOption("AtlasLootWishList", Data);
end

function module.Widget:SetMatch(ItemLink, Tooltip)
  _, _, module.CurrentMatch = string.find(ItemLink, "|Hitem:(%d+):");
  module.CurrentMatch = tonumber(module.CurrentMatch);
  module:Debug("ItemID: "..(module.CurrentMatch or "nil"));
end

function module.Widget:GetMatch(RuleNum, Index)
  local RuleValue = self:GetData(RuleNum);
  local Player = RuleValue[Index][1];
  local List = RuleValue[Index][2];
  if ( Player == -1 ) then
    Player = nil;
  elseif ( Player == -2 ) then
    Player = UnitName("player");
  end
  if ( List == -1 ) then
    List = nil;
  end
  if ( AtlasLoot_GetWishLists and AtlasLoot_CheckWishlistItem ) then
    return AtlasLoot_CheckWishlistItem(module.CurrentMatch, Player, List);
  end
  return false;
end

local SortedList = {};
function module:DropDown_Init(Frame, Level)
  Level = Level or 1;
  local info = {};
  info.checked = false;
  info.notCheckable = true;
  if ( select(4, GetBuildInfo()) < 30000 ) then
    info.func = function(...) self:DropDown_OnClick(this, ...) end;
  else
    info.func = function(...) self:DropDown_OnClick(...) end;
  end
  info.owner = Frame;
  local List = AtlasLoot_GetWishLists();
  if ( Level == 1 ) then
    for Key, Value in pairs(SortedList) do
      SortedList[Key] = nil;
    end
    for Key, Value in pairs(List) do
      table.insert(SortedList, Key);
    end
    table.sort(SortedList);
    info.text = ALL;
    info.hasArrow = false;
    info.notClickable = false;
    info.value = {
      ["Player"] = -1,
      ["List"] = -1,
    };
    UIDropDownMenu_AddButton(info, Level);
    info.text = L["Current Player"];
    info.hasArrow = false;
    info.notClickable = false;
    info.value = {
      ["Player"] = -2,
      ["List"] = -1,
    };
    UIDropDownMenu_AddButton(info, Level);
    for Key, Value in ipairs(SortedList) do
      info.text = Value;
      info.hasArrow = true;
      info.notClickable = false;
      info.value = {
        ["Player"] = Value,
        ["List"] = -1,
      };
      UIDropDownMenu_AddButton(info, Level);
    end
  else
    for Key, Value in pairs(SortedList) do
      SortedList[Key] = nil;
    end
    for Key, Value in pairs(List[UIDROPDOWNMENU_MENU_VALUE.Player]) do
      table.insert(SortedList, Key);
    end
    table.sort(SortedList);
    info.text = ALL;
    info.hasArrow = false;
    info.notClickable = false;
    info.value = {
      ["Player"] = UIDROPDOWNMENU_MENU_VALUE.Player,
      ["List"] = -1,
    };
    UIDropDownMenu_AddButton(info, Level);
    for Key, Value in ipairs(SortedList) do
      info.text = List[UIDROPDOWNMENU_MENU_VALUE.Player][Value][1];
      info.hasArrow = false;
      info.notClickable = false;
      info.value = {
        ["Player"] = UIDROPDOWNMENU_MENU_VALUE.Player,
        ["List"] = List[UIDROPDOWNMENU_MENU_VALUE.Player][Value][1],
      };
      UIDropDownMenu_AddButton(info, Level);
    end
  end
end

function module:DropDown_OnClick(Frame)
  local Value = self.Widget:GetData();
  Value[self.FilterIndex][1] = Frame.value.Player;
  Value[self.FilterIndex][2] = Frame.value.List;
  self:SetConfigOption("AtlasLootWishList", Value);
  if ( select(4, GetBuildInfo()) < 30000 ) then
    UIDropDownMenu_SetText(self:GetAtlasLootText(Frame.value.Player, Frame.value.List), Frame.owner);
  else
    UIDropDownMenu_SetText(Frame.owner, self:GetAtlasLootText(Frame.value.Player, Frame.value.List));
  end
  DropDownList1:Hide(); -- Nested dropdown buttons don't hide their parent menus on click.
end

function module:GetAtlasLootText(Player, List)
  if ( Player == -1 ) then
    return ALL;
  elseif ( Player == -2 ) then
    return L["Current Player"];
  end
  if ( List == -1 ) then
    List = ALL;
  end
  local Text = string.gsub(L["FILTER_TEXT"], "%%player%%", Player);
  Text = string.gsub(Text, "%%list%%", List);
  return Text;
end

--[=[
AtlasLoot_GetWishLists([playerName])
Returns a Table with wishlist infos, if name not exist in wishlisttable it returns nil.
[playerName]      -> Enter a PlayerName <string>

return:
table = {
   ["playerName"] = {
      [WishListNumber] = {
         [1] = "WishlistName",
         [2] = "WishlistIcon",
      },
   },
}

examples:
AtlasLoot_GetWishLists()
AtlasLoot_GetWishLists("Lag")
]=]

--[=[
AtlasLoot_CheckWishlistItem(itemID ,[playerName ,[wishlist] ])
Returns a Table with infos about the item.
itemID          -> Enter a ItemID
[playerName]   -> Enter a PlayerName, needed if you want to check only wishlists from a particular player (if you need to check all players, enter nil)
[wishlist]      -> Checks only this wishlist (playerName can be nil)

return:
table = {
   [index] = {
      [1] = "playerName",
      [2] = "WishListName"
   }
}

examples:
AtlasLoot_CheckWishlistItem(6908)
AtlasLoot_CheckWishlistItem(6908, UnitName("player"))
AtlasLoot_CheckWishlistItem(6908, "Lag")
AtlasLoot_CheckWishlistItem(6908, nil, "TestWishList")
AtlasLoot_CheckWishlistItem(6908, "Lag", "TestWishList")
]=]
