﻿--[[

	Quest Completist - qcCollect.lua
	Written by: Alistair Maxwell

--]]

local COLOUR_DEATHKNIGHT = "|cffc41f3b"
local COLOUR_DRUID = "|cffff7d0a"
local COLOUR_HUNTER = "|cffabd473"
local COLOUR_MAGE = "|cff69ccf0"
local COLOUR_PALADIN = "|cfff58cba"
local COLOUR_PRIEST = "|cffffffff"
local COLOUR_ROGUE = "|cfffff569"
local COLOUR_SHAMAN = "|cff0070de"
local COLOUR_WARLOCK = "|cff9482c9"
local COLOUR_WARRIOR = "|cffc79c6e"

local function qcRound(num, idp)

	return tonumber(string.format("%." .. (idp or 0) .. "f", num))

end

function qcWipeWorkingDB()

	wipe(qcWorkingDB)
	wipe(qcWorkingLog)

end

function qcMergeDB(qcPersonalMerge)

	qcMergeSVData(qcPersonalMerge)
	qcSetClassBits()
	qcSetQuestCategories()

end

function qcGenerateMapData()

	local qcInitiatorID = (tonumber(strsub(UnitGUID("questnpc"),7,10),16) or 0)
	local qcInitiatorName = UnitName("npc")
	local qcMapID = GetCurrentMapAreaID()
	local qcMapLevel = GetCurrentMapDungeonLevel()
	local qcQuestID = GetQuestID()
	local X, Y = GetPlayerMapPosition("player")
	local qcQuestType = 1
	if (QuestIsDaily()) then
		qcQuestType = 3
	elseif (QuestIsWeekly()) then
		qcQuestType = 5
	end

	print("{" .. qcMapLevel .. "," .. qcQuestType .. "," .. qcInitiatorID .. ",\"" .. qcInitiatorName .. "\"," .. string.format("%.1f",X*100) .. "," .. string.format("%.1f",Y*100) .. ",{" .. qcQuestID .. "}},")

end

function qcCleanQuestDatabaseEmptyTables()

	wipe(qcWorkingDB)
	qcWorkingDB = qcCopyTable(qcQuestDatabase)

	for qcIndex, qcEntry in pairs(qcWorkingDB) do
		if (qcWorkingDB[qcIndex][12] == nil) then qcWorkingDB[qcIndex][12] = {} end
		if (qcWorkingDB[qcIndex][13] == nil) then qcWorkingDB[qcIndex][13] = {} end

		local qcInitiatorCount = 0
		for qcInitiatorIndex, qcInitiatorEntry in pairs(qcWorkingDB[qcIndex][13]) do
			qcInitiatorCount = (qcInitiatorCount + 1)
		end
		if (qcInitiatorCount == 0) then qcWorkingDB[qcIndex][13] = nil end
		if (qcWorkingDB[qcIndex][13] == nil) then
			local qcRepCount = 0
			for qcRepIndex, qcRepEntry in pairs(qcWorkingDB[qcIndex][12]) do
				qcRepCount = (qcRepCount + 1)
			end
			if (qcRepCount == 0) then qcWorkingDB[qcIndex][12] = nil end
		end
	end

end

function qcFindMappedNILQuests()

	for qcMapIndex, qcMapEntry in pairs(qcMapIcon) do
		for qcInitiatorIndex, qcInitiatorEntry in pairs(qcMapIcon[qcMapIndex]) do
			for qcInitiatorQuestIndex, qcInitiatorQuestEntry in pairs(qcMapIcon[qcMapIndex][qcInitiatorIndex][7]) do
				if (qcQuestDatabase[qcInitiatorQuestEntry] == nil) then
					print(qcInitiatorQuestEntry .. " - Given by: " .. tostring(qcMapIcon[qcMapIndex][qcInitiatorIndex][4]))
				end
			end
		end
	end

end

function qcUpdateInitiatorData()

	wipe(qcWorkingDB)
	qcWorkingDB = qcCopyTable(qcQuestDatabase)

	for qcIndex, qcEntry in pairs(qcWorkingDB) do
		if (qcWorkingDB[qcIndex][12] == nil) then
			qcWorkingDB[qcIndex][12] = {}
		end
		if (qcWorkingDB[qcIndex][13] == nil) then
			qcWorkingDB[qcIndex][13] = {}
		else
			wipe(qcWorkingDB[qcIndex][13])
		end
	end

	for qcMapIndex, qcMapEntry in pairs(qcMapIcon) do
		for qcInitiatorIndex, qcInitiatorEntry in pairs(qcMapIcon[qcMapIndex]) do
			for qcInitiatorQuestIndex, qcInitiatorQuestEntry in pairs(qcMapIcon[qcMapIndex][qcInitiatorIndex][7]) do
				if (qcWorkingDB[qcInitiatorQuestEntry] == nil) then
					print("Quest not found:", qcInitiatorQuestEntry)
				else
					SetMapByID(qcMapIndex)
					local qcFloorName = _G["DUNGEON_FLOOR_" .. strupper(GetMapInfo() or "") .. (qcMapIcon[qcMapIndex][qcInitiatorIndex][1])] or 0
					if (qcFloorName ~= 0) then print(qcFloorName) end
					table.insert(qcWorkingDB[qcInitiatorQuestEntry][13],{qcMapIcon[qcMapIndex][qcInitiatorIndex][3],qcMapIcon[qcMapIndex][qcInitiatorIndex][4],qcMapIndex,qcFloorName,qcRound(qcMapIcon[qcMapIndex][qcInitiatorIndex][5],1),qcRound(qcMapIcon[qcMapIndex][qcInitiatorIndex][6],1)})
				end
			end
		end
	end

end

function qcFindQuestsWithNoMapData()

	local qcFound
	local qcCountFound = 0
	local qcCountNotFound = 0
	local qcQuestCount = 0
	for qcQuestIndex, qcQuestEntry in pairs(qcQuestDatabase) do
		qcFound = false
		qcQuestCount = (qcQuestCount + 1)
		for qcMapIndex, qcMapEntry in pairs(qcMapIcon) do
			for qcInitiatorIndex, qcInitiatorEntry in pairs(qcMapIcon[qcMapIndex]) do
				for qcInitiatorQuestIndex, qcInitiatorQuestEntry in pairs(qcMapIcon[qcMapIndex][qcInitiatorIndex][7]) do
					if (qcInitiatorQuestEntry == qcQuestIndex) then
						qcFound = true
						--break
					end
				end
			end
		end
		if not (qcFound) then
			print(qcQuestIndex)
			-- Log the quest details.
			-- Possibly log them by Map ID or zone or something similar?
			qcCountNotFound = (qcCountNotFound + 1)
		else
			qcCountFound = (qcCountFound + 1)
		end
	end
	print("Out of " .. qcQuestCount .. " quests, map data was found for " .. qcCountFound .. " (" .. qcRound(((qcCountFound/qcQuestCount)*100),2) .. "%), and " .. qcCountNotFound .. " (" .. qcRound(((qcCountNotFound/qcQuestCount)*100),2) .. "%) had no map data assosiated with it.")

end						

function qcQueryMapDataForQuest(qcQuestID)

	if not (qcQuestID) then return nil end
	local qcFound
	for qcMapIndex, qcMapEntry in pairs(qcMapIcon) do
		for qcInitiatorIndex, qcInitiatorEntry in pairs(qcMapIcon[qcMapIndex]) do
			for qcInitiatorQuestIndex, qcInitiatorQuestEntry in pairs(qcMapIcon[qcMapIndex][qcInitiatorIndex][7]) do
				if (qcInitiatorQuestEntry == qcQuestID) then
					if not (qcFound) then
						print("Quest: " .. qcInitiatorQuestEntry .. " - " .. qcQuestDatabase[qcInitiatorQuestEntry][2])
						print("   Given by: " .. tostring(qcMapIcon[qcMapIndex][qcInitiatorIndex][4]))
						print("      Located on Map ID: " .. qcMapIndex .. " @ " .. qcMapIcon[qcMapIndex][qcInitiatorIndex][5] .. ", " .. qcMapIcon[qcMapIndex][qcInitiatorIndex][6])
					else
						print("   Also given by: " .. tostring(qcMapIcon[qcMapIndex][qcInitiatorIndex][4]))
						print("      Located on Map ID: " .. qcMapIndex .. " @ " .. qcMapIcon[qcMapIndex][qcInitiatorIndex][5] .. ", " .. qcMapIcon[qcMapIndex][qcInitiatorIndex][6])
					end
					qcFound = true
				end
			end
		end
	end

end

function qcCleanOldDatabase()	

	local qcCleanedCount = 0
	wipe(qcWorkingDB)
	qcWorkingDB = qcCopyTable(qcQuestDBv2)
	for qcMapIndex, qcMapEntry in pairs(qcMapIcon) do
		for qcInitiatorIndex, qcInitiatorEntry in pairs(qcMapIcon[qcMapIndex]) do
			for qcInitiatorQuestIndex, qcInitiatorQuestEntry in pairs(qcMapIcon[qcMapIndex][qcInitiatorIndex][7]) do
				if not (qcWorkingDB[qcInitiatorQuestEntry] == nil) then
					qcCleanedCount = (qcCleanedCount + 1)
					qcWorkingDB[qcInitiatorQuestEntry] = nil
				end
			end
		end
	end

	print(qcCleanedCount .. " quests cleaned from the old qcQuestDBv2 database.")

end

function qcFixRaceFlags()

	local qcCorrectionCount = 0
	wipe(qcWorkingDB)
	qcWorkingDB = qcCopyTable(qcQuestDatabase)
	for qcIndex, qcEntry in pairs(qcWorkingDB) do
		if (qcWorkingDB[qcIndex][7] == 1) then --[[ Alliance Only ]]--
			if (qcWorkingDB[qcIndex][8] == 4095) then
				qcCorrectionCount = (qcCorrectionCount + 1)
				qcWorkingDB[qcIndex][8] = 3149
			end
		elseif (qcWorkingDB[qcIndex][7] == 2) then --[[ Horde Only ]]--
			if (qcWorkingDB[qcIndex][8] == 4095) then
				qcCorrectionCount = (qcCorrectionCount + 1)
				qcWorkingDB[qcIndex][8] = 946
			end
		end
	end

	print(qcCorrectionCount .. " quests race flags corrected.")

end

function qcSetQuestCategories()

	local qcRecordCount = 0
	local qcInvalidCount = 0
	local qcChangeCount = 0
	table.insert(qcWorkingLog, "-----------------------")
	table.insert(qcWorkingLog, "PROCESSING ZONE ID's...")
	table.insert(qcWorkingLog, "-----------------------")

	for qcIndex, qcEntry in pairs(qcWorkingDB) do
		qcRecordCount = (qcRecordCount + 1)
		if (qcEntry[2][1] == 99999) then
			qcInvalidCount = (qcInvalidCount + 1)
			for qcCategoryIndex, qcCategoryEntry in ipairs(qcQuestCategories) do
				if (qcCategoryEntry[2] == qcEntry[1][4]) then
					qcChangeCount = (qcChangeCount + 1)
					table.insert(qcWorkingLog, "ZONE ID UPDATED FOR QUEST: " .. tostring(qcEntry[1][2]) .. " [" .. tostring(qcEntry[1][1]) .. "]")
					table.insert(qcWorkingLog, "    ZONE ID - ORIGINAL VALUE: '" .. tostring(qcEntry[2][1]) .. "' NEW VALUE: '" .. tostring(qcCategoryEntry[1]) .. "'")
					qcEntry[2][1] = qcCategoryEntry[1]
					break
				end
			end
		end
	end

	table.insert(qcWorkingLog, "NO VALID ZONE ID FOUND FOR THE FOLLOWING CATEGORIES:")
	for qcIndex, qcEntry in pairs(qcWorkingDB) do
		if (qcEntry[2][1] == 99999) then
			table.insert(qcWorkingLog, tostring(qcEntry[1][4]))
		end
	end

	print(tostring(qcRecordCount) .. " records were checked for invalid zone ID's. " .. tostring(qcInvalidCount) .. " invalid records were found. " .. tostring(qcChangeCount) .. " changes were made, leaving " .. tostring(qcInvalidCount - qcChangeCount) .. " invalid records remaining.")

end

function qcMergeSVData(qcPersonalMerge)

	if (qcPersonalMerge == nil) then print("Missing: qcPersonalMerge Bool") return nil end

	for qcIndex, qcEntry in pairs(qcQuestDBv2) do
		table.insert(qcWorkingDB, qcIndex, qcEntry)
	end

	local qcNewCount = 0
	local qcRecordCount = 0
	local qcClassUpdate = 0
	local qcRaceUpdate = 0
	local qcFactionUpdate = 0
	local qcReputationUpdate = 0

	local qcTab = "   "
	local qcSeparator = " -> "
	local qcLockedData = " LOCKED DATA"
	local qcLeadingBracket = "["
	local qcNewQuestFound = "New Quest Found: "
	local qcExistingQuestFound = "Existing Quest Found: "
	local qcCategoryUnknown = "<Category Unknown>"
	local qcPrelimData = "Only Preliminary Data Available."
	local qcNPCID = "NPC ID: "
	local qcNPCName = "NPC Name: "
	local qcNPCZone = "NPC Zone: "
	local qcNPCX = "NPC X: "
	local qcNPCY = "NPC Y: "
	local qcMapID = "Map ID: "
	local qcMapLevel = "Map Level: "
	local qcQuestName = "Quest Name: "
	local qcQuestLevel = "Quest Level: "
	local qcQuestCategory = "Quest Category: "
	local qcZoneID = "Zone ID: "

	table.insert(qcWorkingLog, "Processing Quest Records...")

	for qcIndex, qcEntry in pairs(qcSVData) do
		qcRecordCount = (qcRecordCount + 1)
		if (qcWorkingDB[qcIndex] == nil) then --[[ THIS IS A BRAND NEW QUEST ]]--
			table.insert(qcWorkingDB, qcIndex, qcEntry)
			qcWorkingDB[qcIndex][4][2] = 4095
			qcWorkingDB[qcIndex][4][3] = 1023
			table.insert(qcWorkingLog, qcTab .. qcNewQuestFound .. tostring(qcEntry[1][1]))
			qcNewCount = (qcNewCount + 1)
		else --[[ THIS QUEST ALREADY EXISTS ]]--
			table.insert(qcWorkingLog, qcTab .. qcExistingQuestFound .. tostring(qcEntry[1][1]))
			if (qcEntry[1][4] == qcCategoryUnknown) then --[[ ONLY PRELIM DATA AVAILABLE ]]--
				table.insert(qcWorkingLog, qcTab .. qcTab .. qcPrelimData)
				if (qcWorkingDB[qcIndex][1][1] == nil) then	qcWorkingDB[qcIndex][1][1] = qcIndex end
				--[[ THINGS TO UPDATE IF ONLY PRELIM DATA AVAILABLE:
						NPC ID		[3][1] PERSONAL MERGE ONLY
						NPC NAME	[3][2] PERSONAL MERGE ONLY
						NPC ZONE	[3][3] PERSONAL MERGE ONLY
						NPC X		[3][4] PERSONAL MERGE ONLY
						NPC Y		[3][5] PERSONAL MERGE ONLY
						MAP ID		[3][6] PERSONAL MERGE ONLY
						MAP LEVEL	[3][7] PERSONAL MERGE ONLY
				--]]
				if not (qcWorkingDB[qcIndex][3][1] == qcEntry[3][1]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCID .. tostring(qcWorkingDB[qcIndex][3][1]) .. qcSeparator .. tostring(qcEntry[3][1]))
							qcWorkingDB[qcIndex][3][1] = qcEntry[3][1]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCID .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][2] == qcEntry[3][2]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCName .. tostring(qcWorkingDB[qcIndex][3][2]) .. qcSeparator .. tostring(qcEntry[3][2]))
							qcWorkingDB[qcIndex][3][2] = qcEntry[3][2]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCName .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][3] == qcEntry[3][3]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCZone .. tostring(qcWorkingDB[qcIndex][3][3]) .. qcSeparator .. tostring(qcEntry[3][3]))
							qcWorkingDB[qcIndex][3][3] = qcEntry[3][3]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCZone .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][4] == qcEntry[3][4]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCX .. tostring(qcWorkingDB[qcIndex][3][4]) .. qcSeparator .. tostring(qcEntry[3][4]))
							qcWorkingDB[qcIndex][3][4] = qcEntry[3][4]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCX .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][5] == qcEntry[3][5]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCY .. tostring(qcWorkingDB[qcIndex][3][5]) .. qcSeparator .. tostring(qcEntry[3][5]))
							qcWorkingDB[qcIndex][3][5] = qcEntry[3][5]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcNPCY .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][6] == qcEntry[3][6]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcMapID .. tostring(qcWorkingDB[qcIndex][3][6]) .. qcSeparator .. tostring(qcEntry[3][6]))
							qcWorkingDB[qcIndex][3][6] = qcEntry[3][6]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcMapID .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][7] == qcEntry[3][7]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcMapLevel .. tostring(qcWorkingDB[qcIndex][3][7]) .. qcSeparator .. tostring(qcEntry[3][7]))
							qcWorkingDB[qcIndex][3][7] = qcEntry[3][7]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcTab .. qcMapLevel .. qcLockedData)
						end
					end
				end
			else --[[ COMPLETE QUEST DATA AVAILABLE ]]--
				if (qcWorkingDB[qcIndex][1][1] == nil) then	qcWorkingDB[qcIndex][1][1] = qcIndex end
				--[[ THINGS TO UPDATE WHEN FULL DATA AVAILABLE:
						QUEST NAME		[1][2]
						QUEST LEVEL		[1][3]
						QUEST CATEGORY	[1][4]
						ZONE ID			[2][1]
						NPC ID			[3][1] PERSONAL MERGE ONLY
						NPC NAME		[3][2] PERSONAL MERGE ONLY
						NPC ZONE		[3][3] PERSONAL MERGE ONLY
						NPC X			[3][4] PERSONAL MERGE ONLY
						NPC Y			[3][5] PERSONAL MERGE ONLY
						MAP ID			[3][6] PERSONAL MERGE ONLY
						MAP LEVEL		[3][7] PERSONAL MERGE ONLY
						FACTION FLAGS	[4][1]
						RACE FLAGS		[4][2]
						CLASS FLAGS		[4][3]
						REPUTATIONS		[6][#]
				--]]
				if not (qcWorkingDB[qcIndex][1][2] == qcEntry[1][2]) then
					if not (string.sub(qcEntry[1][2], 1, 1) == qcLeadingBracket) then
						table.insert(qcWorkingLog, qcTab .. qcTab .. qcQuestName .. tostring(qcWorkingDB[qcIndex][1][2]) .. qcSeparator .. tostring(qcEntry[1][2]))
						qcWorkingDB[qcIndex][1][2] = qcEntry[1][2]
					end
				end
				if not (qcWorkingDB[qcIndex][1][3] == qcEntry[1][3]) then
					table.insert(qcWorkingLog, qcTab .. qcTab .. qcQuestLevel .. tostring(qcWorkingDB[qcIndex][1][3]) .. qcSeparator .. tostring(qcEntry[1][3]))
					qcWorkingDB[qcIndex][1][3] = qcEntry[1][3]
				end
				if not (qcWorkingDB[qcIndex][1][4] == qcEntry[1][4]) then
					table.insert(qcWorkingLog, qcTab .. qcTab .. qcQuestCategory .. tostring(qcWorkingDB[qcIndex][1][4]) .. qcSeparator .. tostring(qcEntry[1][4]))
					qcWorkingDB[qcIndex][1][4] = qcEntry[1][4]
				end
				if not (qcWorkingDB[qcIndex][2][1] == qcEntry[2][1]) then
					table.insert(qcWorkingLog, qcTab .. qcTab .. qcZoneID .. tostring(qcWorkingDB[qcIndex][2][1]) .. qcSeparator .. tostring(qcEntry[2][1]))
					qcWorkingDB[qcIndex][2][1] = qcEntry[2][1]
				end
				if not (qcWorkingDB[qcIndex][3][1] == qcEntry[3][1]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCID .. tostring(qcWorkingDB[qcIndex][3][1]) .. qcSeparator .. tostring(qcEntry[3][1]))
							qcWorkingDB[qcIndex][3][1] = qcEntry[3][1]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCID .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][2] == qcEntry[3][2]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCName .. tostring(qcWorkingDB[qcIndex][3][2]) .. qcSeparator .. tostring(qcEntry[3][2]))
							qcWorkingDB[qcIndex][3][2] = qcEntry[3][2]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCName .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][3] == qcEntry[3][3]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCZone .. tostring(qcWorkingDB[qcIndex][3][3]) .. qcSeparator .. tostring(qcEntry[3][3]))
							qcWorkingDB[qcIndex][3][3] = qcEntry[3][3]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCZone .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][4] == qcEntry[3][4]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCX .. tostring(qcWorkingDB[qcIndex][3][4]) .. qcSeparator .. tostring(qcEntry[3][4]))
							qcWorkingDB[qcIndex][3][4] = qcEntry[3][4]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCX .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][5] == qcEntry[3][5]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCY .. tostring(qcWorkingDB[qcIndex][3][5]) .. qcSeparator .. tostring(qcEntry[3][5]))
							qcWorkingDB[qcIndex][3][5] = qcEntry[3][5]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcNPCY .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][6] == qcEntry[3][6]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcMapID .. tostring(qcWorkingDB[qcIndex][3][6]) .. qcSeparator .. tostring(qcEntry[3][6]))
							qcWorkingDB[qcIndex][3][6] = qcEntry[3][6]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcMapID .. qcLockedData)
						end
					end
				end
				if not (qcWorkingDB[qcIndex][3][7] == qcEntry[3][7]) then
					if (qcPersonalMerge == true) then
						if not (qcWorkingDB[qcIndex][3][9] == 1) then
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcMapLevel .. tostring(qcWorkingDB[qcIndex][3][7]) .. qcSeparator .. tostring(qcEntry[3][7]))
							qcWorkingDB[qcIndex][3][7] = qcEntry[3][7]
						else
							table.insert(qcWorkingLog, qcTab .. qcTab .. qcMapLevel .. qcLockedData)
						end
					end
				end
				--[[ FACTION FLAGS - TODO: NEEDS REVIEWED ]]--
				do
					local qcEntryFaction = qcEntry[4][1]
					local qcExistingFaction = qcWorkingDB[qcIndex][4][1]
					local qcNewFaction = 0

					local qcEntryAlliance, qcEntryHorde = 0, 0
					local qcExistingAlliance, qcExistingHorde = 0, 0
					local qcNewAlliance, qcNewHorde = 0, 0

					if (bit.band(qcEntryFaction, qcFactionBits["ALLIANCE"]) ~= 0) then qcEntryAlliance = qcFactionBits["ALLIANCE"] end
					if (bit.band(qcEntryFaction, qcFactionBits["HORDE"]) ~= 0) then qcEntryHorde = qcFactionBits["HORDE"] end

					if (bit.band(qcExistingFaction, qcFactionBits["ALLIANCE"]) ~= 0) then qcExistingAlliance = qcFactionBits["ALLIANCE"] end
					if (bit.band(qcExistingFaction, qcFactionBits["HORDE"]) ~= 0) then qcExistingHorde = qcFactionBits["HORDE"] end

					if (qcEntryAlliance > qcExistingAlliance) then
						table.insert(qcWorkingLog, "    FACTION UPDATE - ALLIANCE ADDED")
						qcFactionUpdate = (qcFactionUpdate + 1)
					end
					qcNewAlliance = math.max(qcEntryAlliance, qcExistingAlliance)
					if (qcEntryHorde > qcExistingHorde) then
						table.insert(qcWorkingLog, "    FACTION UPDATE - HORDE ADDED")
						qcFactionUpdate = (qcFactionUpdate + 1)
					end
					qcNewHorde = math.max(qcEntryHorde, qcExistingHorde)
					qcNewFaction = (qcNewAlliance + qcNewHorde)
					qcWorkingDB[qcIndex][4][1] = qcNewFaction
				end
				--[[ RACE FLAGS - TODO: NEEDS REVIEWED ]]--
				do
					local qcEntryRace = qcEntry[4][2]
					local qcExistingRace = qcWorkingDB[qcIndex][4][2]
					local qcNewRace = 0

					local qcEntryHuman, qcEntryOrc, qcEntryDwarf, qcEntryNightelf, qcEntryScourge, qcEntryTauren, qcEntryGnome, qcEntryTroll, qcEntryGoblin, qcEntryBloodelf, qcEntryDraenei, qcEntryWorgen = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
					local qcExistingHuman, qcExistingOrc, qcExistingDwarf, qcExistingNightelf, qcExistingScourge, qcExistingTauren, qcExistingGnome, qcExistingTroll, qcExistingGoblin, qcExistingBloodelf, qcExistingDraenei, qcExistingWorgen = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
					local qcNewHuman, qcNewOrc, qcNewDwarf, qcNewNightelf, qcNewScourge, qcNewTauren, qcNewGnome, qcNewTroll, qcNewGoblin, qcNewBloodelf, qcNewDraenei, qcNewWorgen = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

					if (bit.band(qcEntryRace, qcRaceBits["HUMAN"]) ~= 0) then qcEntryHuman = qcRaceBits["HUMAN"] end
					if (bit.band(qcEntryRace, qcRaceBits["ORC"]) ~= 0) then qcEntryOrc = qcRaceBits["ORC"] end
					if (bit.band(qcEntryRace, qcRaceBits["DWARF"]) ~= 0) then qcEntryDwarf = qcRaceBits["DWARF"] end
					if (bit.band(qcEntryRace, qcRaceBits["NIGHTELF"]) ~= 0) then qcEntryNightelf = qcRaceBits["NIGHTELF"] end
					if (bit.band(qcEntryRace, qcRaceBits["SCOURGE"]) ~= 0) then qcEntryScourge = qcRaceBits["SCOURGE"] end
					if (bit.band(qcEntryRace, qcRaceBits["TAUREN"]) ~= 0) then qcEntryTauren = qcRaceBits["TAUREN"] end
					if (bit.band(qcEntryRace, qcRaceBits["GNOME"]) ~= 0) then qcEntryGnome = qcRaceBits["GNOME"] end
					if (bit.band(qcEntryRace, qcRaceBits["TROLL"]) ~= 0) then qcEntryTroll = qcRaceBits["TROLL"] end
					if (bit.band(qcEntryRace, qcRaceBits["GOBLIN"]) ~= 0) then qcEntryGoblin = qcRaceBits["GOBLIN"] end
					if (bit.band(qcEntryRace, qcRaceBits["BLOODELF"]) ~= 0) then qcEntryBloodelf = qcRaceBits["BLOODELF"] end
					if (bit.band(qcEntryRace, qcRaceBits["DRAENEI"]) ~= 0) then qcEntryDraenei = qcRaceBits["DRAENEI"] end
					if (bit.band(qcEntryRace, qcRaceBits["WORGEN"]) ~= 0) then qcEntryWorgen = qcRaceBits["WORGEN"] end

					if (bit.band(qcExistingRace, qcRaceBits["HUMAN"]) ~= 0) then qcExistingHuman = qcRaceBits["HUMAN"] end
					if (bit.band(qcExistingRace, qcRaceBits["ORC"]) ~= 0) then qcExistingOrc = qcRaceBits["ORC"] end
					if (bit.band(qcExistingRace, qcRaceBits["DWARF"]) ~= 0) then qcExistingDwarf = qcRaceBits["DWARF"] end
					if (bit.band(qcExistingRace, qcRaceBits["NIGHTELF"]) ~= 0) then qcExistingNightelf = qcRaceBits["NIGHTELF"] end
					if (bit.band(qcExistingRace, qcRaceBits["SCOURGE"]) ~= 0) then qcExistingScourge = qcRaceBits["SCOURGE"] end
					if (bit.band(qcExistingRace, qcRaceBits["TAUREN"]) ~= 0) then qcExistingTauren = qcRaceBits["TAUREN"] end
					if (bit.band(qcExistingRace, qcRaceBits["GNOME"]) ~= 0) then qcExistingGnome = qcRaceBits["GNOME"] end
					if (bit.band(qcExistingRace, qcRaceBits["TROLL"]) ~= 0) then qcExistingTroll = qcRaceBits["TROLL"] end
					if (bit.band(qcExistingRace, qcRaceBits["GOBLIN"]) ~= 0) then qcExistingGoblin = qcRaceBits["GOBLIN"] end
					if (bit.band(qcExistingRace, qcRaceBits["BLOODELF"]) ~= 0) then qcExistingBloodelf = qcRaceBits["BLOODELF"] end
					if (bit.band(qcExistingRace, qcRaceBits["DRAENEI"]) ~= 0) then qcExistingDraenei = qcRaceBits["DRAENEI"] end
					if (bit.band(qcExistingRace, qcRaceBits["WORGEN"]) ~= 0) then qcExistingWorgen = qcRaceBits["WORGEN"] end

					if (qcEntryHuman > qcExistingHuman) then
						table.insert(qcWorkingLog, "    RACE UPDATE - HUMAN ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewHuman = math.max(qcEntryHuman, qcExistingHuman)
					if (qcEntryOrc > qcExistingOrc) then
						table.insert(qcWorkingLog, "    RACE UPDATE - ORC ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewOrc = math.max(qcEntryOrc, qcExistingOrc)
					if (qcEntryDwarf > qcExistingDwarf) then
						table.insert(qcWorkingLog, "    RACE UPDATE - DWARF ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewDwarf = math.max(qcEntryDwarf, qcExistingDwarf)
					if (qcEntryNightelf > qcExistingNightelf) then
						table.insert(qcWorkingLog, "    RACE UPDATE - NIGHTELF ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewNightelf = math.max(qcEntryNightelf, qcExistingNightelf)
					if (qcEntryScourge > qcExistingScourge) then
						table.insert(qcWorkingLog, "    RACE UPDATE - SCOURGE ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewScourge = math.max(qcEntryScourge, qcExistingScourge)
					if (qcEntryTauren > qcExistingTauren) then
						table.insert(qcWorkingLog, "    RACE UPDATE - TAUREN ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewTauren = math.max(qcEntryTauren, qcExistingTauren)
					if (qcEntryGnome > qcExistingGnome) then
						table.insert(qcWorkingLog, "    RACE UPDATE - GNOME ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewGnome = math.max(qcEntryGnome, qcExistingGnome)
					if (qcEntryTroll > qcExistingTroll) then
						table.insert(qcWorkingLog, "    RACE UPDATE - TROLL ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewTroll = math.max(qcEntryTroll, qcExistingTroll)
					if (qcEntryGoblin > qcExistingGoblin) then
						table.insert(qcWorkingLog, "    RACE UPDATE - GOBLIN ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewGoblin = math.max(qcEntryGoblin, qcExistingGoblin)
					if (qcEntryBloodelf > qcExistingBloodelf) then
						table.insert(qcWorkingLog, "    RACE UPDATE - BLOODELF ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewBloodelf = math.max(qcEntryBloodelf, qcExistingBloodelf)
					if (qcEntryDraenei > qcExistingDraenei) then
						table.insert(qcWorkingLog, "    RACE UPDATE - DRAENEI ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewDraenei = math.max(qcEntryDraenei, qcExistingDraenei)
					if (qcEntryWorgen > qcExistingWorgen) then
						table.insert(qcWorkingLog, "    RACE UPDATE - WORGEN ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewWorgen = math.max(qcEntryWorgen, qcExistingWorgen)
					qcNewRace = (qcNewHuman + qcNewOrc + qcNewDwarf + qcNewNightelf + qcNewScourge + qcNewTauren + qcNewGnome + qcNewTroll + qcNewGoblin + qcNewBloodelf + qcNewDraenei + qcNewWorgen)
					qcWorkingDB[qcIndex][4][2] = qcNewRace
				end
				--[[ CLASS FLAGS - TODO: NEEDS REVIEWED ]]--
				do
					local qcEntryClass = qcEntry[4][3]
					local qcExistingClass = qcWorkingDB[qcIndex][4][3]
					local qcNewClass = 0

					local qcEntryWarrior, qcEntryPaladin, qcEntryHunter, qcEntryRogue, qcEntryPriest, qcEntryDeathknight, qcEntryShaman, qcEntryMage, qcEntryWarlock, qcEntryDruid = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
					local qcExistingWarrior, qcExistingPaladin, qcExistingHunter, qcExistingRogue, qcExistingPriest, qcExistingDeathknight, qcExistingShaman, qcExistingMage, qcExistingWarlock, qcExistingDruid = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
					local qcNewWarrior, qcNewPaladin, qcNewHunter, qcNewRogue, qcNewPriest, qcNewDeathknight, qcNewShaman, qcNewMage, qcNewWarlock, qcNewDruid = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

					if (bit.band(qcEntryClass, qcClassBits["WARRIOR"]) ~= 0) then qcEntryWarrior = qcClassBits["WARRIOR"] end
					if (bit.band(qcEntryClass, qcClassBits["PALADIN"]) ~= 0) then qcEntryPaladin = qcClassBits["PALADIN"] end
					if (bit.band(qcEntryClass, qcClassBits["HUNTER"]) ~= 0) then qcEntryHunter = qcClassBits["HUNTER"] end
					if (bit.band(qcEntryClass, qcClassBits["ROGUE"]) ~= 0) then qcEntryRogue = qcClassBits["ROGUE"] end
					if (bit.band(qcEntryClass, qcClassBits["PRIEST"]) ~= 0) then qcEntryPriest = qcClassBits["PRIEST"] end
					if (bit.band(qcEntryClass, qcClassBits["DEATHKNIGHT"]) ~= 0) then qcEntryDeathknight = qcClassBits["DEATHKNIGHT"] end
					if (bit.band(qcEntryClass, qcClassBits["SHAMAN"]) ~= 0) then qcEntryShaman = qcClassBits["SHAMAN"] end
					if (bit.band(qcEntryClass, qcClassBits["MAGE"]) ~= 0) then qcEntryMage = qcClassBits["MAGE"] end
					if (bit.band(qcEntryClass, qcClassBits["WARLOCK"]) ~= 0) then qcEntryWarlock = qcClassBits["WARLOCK"] end
					if (bit.band(qcEntryClass, qcClassBits["DRUID"]) ~= 0) then qcEntryDruid = qcClassBits["DRUID"] end

					if (bit.band(qcExistingClass, qcClassBits["WARRIOR"]) ~= 0) then qcExistingWarrior = qcClassBits["WARRIOR"] end
					if (bit.band(qcExistingClass, qcClassBits["PALADIN"]) ~= 0) then qcExistingPaladin = qcClassBits["PALADIN"] end
					if (bit.band(qcExistingClass, qcClassBits["HUNTER"]) ~= 0) then qcExistingHunter = qcClassBits["HUNTER"] end
					if (bit.band(qcExistingClass, qcClassBits["ROGUE"]) ~= 0) then qcExistingRogue = qcClassBits["ROGUE"] end
					if (bit.band(qcExistingClass, qcClassBits["PRIEST"]) ~= 0) then qcExistingPriest = qcClassBits["PRIEST"] end
					if (bit.band(qcExistingClass, qcClassBits["DEATHKNIGHT"]) ~= 0) then qcExistingDeathknight = qcClassBits["DEATHKNIGHT"] end
					if (bit.band(qcExistingClass, qcClassBits["SHAMAN"]) ~= 0) then qcExistingShaman = qcClassBits["SHAMAN"] end
					if (bit.band(qcExistingClass, qcClassBits["MAGE"]) ~= 0) then qcExistingMage = qcClassBits["MAGE"] end
					if (bit.band(qcExistingClass, qcClassBits["WARLOCK"]) ~= 0) then qcExistingWarlock = qcClassBits["WARLOCK"] end
					if (bit.band(qcExistingClass, qcClassBits["DRUID"]) ~= 0) then qcExistingDruid = qcClassBits["DRUID"] end

					if (qcEntryWarrior > qcExistingWarrior) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - WARRIOR ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewWarrior = math.max(qcEntryWarrior, qcExistingWarrior)
					if (qcEntryPaladin > qcExistingPaladin) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - PALADIN ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewPaladin = math.max(qcEntryPaladin, qcExistingPaladin)
					if (qcEntryHunter > qcExistingHunter) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - HUNTER ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewHunter = math.max(qcEntryHunter, qcExistingHunter)
					if (qcEntryRogue > qcExistingRogue) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - ROGUE ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewRogue = math.max(qcEntryRogue, qcExistingRogue)
					if (qcEntryPriest > qcExistingPriest) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - PRIEST ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewPriest = math.max(qcEntryPriest, qcExistingPriest)
					if (qcEntryDeathknight > qcExistingDeathknight) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - DEATHKNIGHT ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewDeathknight = math.max(qcEntryDeathknight, qcExistingDeathknight)
					if (qcEntryShaman > qcExistingShaman) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - SHAMAN ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewShaman = math.max(qcEntryShaman, qcExistingShaman)
					if (qcEntryMage > qcExistingMage) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - MAGE ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewMage = math.max(qcEntryMage, qcExistingMage)
					if (qcEntryWarlock > qcExistingWarlock) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - WARLOCK ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewWarlock = math.max(qcEntryWarlock, qcExistingWarlock)
					if (qcEntryDruid > qcExistingDruid) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - DRUID ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewDruid = math.max(qcEntryDruid, qcExistingDruid)
					qcNewClass = (qcNewWarrior + qcNewPaladin + qcNewHunter + qcNewRogue + qcNewPriest + qcNewDeathknight + qcNewShaman + qcNewMage + qcNewWarlock + qcNewDruid)
					qcWorkingDB[qcIndex][4][3] = qcNewClass
				end
				--[[ REPUTATIONS - TODO: NEEDS REVIEWED ]]--
				do
					if (qcEntry[6] ~= nil) then
						if (qcWorkingDB[qcIndex][6] == nil) then qcWorkingDB[qcIndex][6] = {} end
						for qcReputationIndex, qcReputationEntry in pairs(qcEntry[6]) do
							qcReputationUpdate = (qcReputationUpdate + 1)
							if (qcWorkingDB[qcIndex][6][qcReputationIndex] == nil) then
								--table.insert(qcWorkingLog, "    REPUTATION UPDATE - NEW FACTION: '" .. tostring(qcReputationIndex) .. "' WITH '" .. tostring(qcReputationEntry) .. "' REPUTATION GAIN")
								qcWorkingDB[qcIndex][6][qcReputationIndex] = qcReputationEntry
							else
								if (qcWorkingDB[qcIndex][6][qcReputationIndex] ~= qcReputationEntry) then
									--table.insert(qcWorkingLog, "    REPUTATION VALUE UPDATE FOR " .. tostring(qcReputationIndex) .. " - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][6][qcReputationIndex]) .. "' NEW VALUE: '" .. tostring(qcReputationEntry) .. "'")
									qcWorkingDB[qcIndex][6][qcReputationIndex] = qcReputationEntry
								end
							end
						end
					end
				end
			end
		end
		--[[ LOCK DATA IF PERSONAL MERGE ]]--
		if (qcPersonalMerge == true) then
			qcWorkingDB[qcIndex][3][9] = 1
		end
	end

	print(tostring(qcRecordCount) .. " quest records checked and updated. " .. tostring(qcNewCount) .. " brand new quests added. " .. tostring(qcClassUpdate) .. " class bitwise updates made, " .. tostring(qcRaceUpdate) .. " race bitwise updates made, " .. tostring(qcFactionUpdate) .. " faction bitwise updates made and " .. tostring(qcReputationUpdate) .. " reputation updates made.")
	
end

local function qcGetQuestCategory(qcQuestID)

	local _S, qcIsHeader, qcHeader
	local qcQuestIDIndex = GetQuestLogIndexByID(qcQuestID)

	if (qcQuestIDIndex == 0) then return nil end

	while (qcQuestIDIndex >= 1) do
		qcHeader, _S, _S, _S, qcIsHeader, _S, _S, _S, _S, _S, _S = GetQuestLogTitle(qcQuestIDIndex)
		if (qcIsHeader == 1) then return qcHeader end
		qcQuestIDIndex = (qcQuestIDIndex - 1)
	end

	return nil

end

function qcSetClassBits()

	local qcChangeCount = 0
	local qcRecordCount = 0
	table.insert(qcWorkingLog, "--------------------------")
	table.insert(qcWorkingLog, "PROCESSING CLASS QUESTS...")
	table.insert(qcWorkingLog, "--------------------------")

	for qcIndex, qcEntry in pairs(qcWorkingDB) do
		qcRecordCount = (qcRecordCount + 1)
		if (string.upper(qcEntry[1][4]) == "WARRIOR") then
			if (qcEntry[4][3] ~= 1) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 1
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR WARRIOR")
			end
		elseif (string.upper(qcEntry[1][4]) == "PALADIN") then
			if (qcEntry[4][3] ~= 2) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 2
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR PALADIN")
			end
		elseif (string.upper(qcEntry[1][4]) == "HUNTER") then
			if (qcEntry[4][3] ~= 4) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 4
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR HUNTER")
			end
		elseif (string.upper(qcEntry[1][4]) == "ROGUE") then
			if (qcEntry[4][3] ~= 8) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 8
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR ROGUE")
			end
		elseif (string.upper(qcEntry[1][4]) == "PRIEST") then
			if (qcEntry[4][3] ~= 16) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 16
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR PRIEST")
			end
		elseif (string.upper(qcEntry[1][4]) == "DEATHKNIGHT") then
			if (qcEntry[4][3] ~= 32) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 32
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR DEATHKNIGHT")
			end
		elseif (string.upper(qcEntry[1][4]) == "SHAMAN") then
			if (qcEntry[4][3] ~= 64) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 64
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR SHAMAN")
			end
		elseif (string.upper(qcEntry[1][4]) == "MAGE") then
			if (qcEntry[4][3] ~= 128) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 128
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR MAGE")
			end
		elseif (string.upper(qcEntry[1][4]) == "WARLOCK") then
			if (qcEntry[4][3] ~= 256) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 256
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR WARLOCK")
			end
		elseif (string.upper(qcEntry[1][4]) == "DRUID") then
			if (qcEntry[4][3] ~= 512) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 512
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR DRUID")
			end
		end
	end

	print(tostring(qcRecordCount) .. " records checked against class categories and updated. " .. tostring(qcChangeCount) .. " changes were made.")

end

function qcCollectQuestOnProgressComplete(qcQuestID)

	if (qcProgressComplete == nil) then qcProgressComplete = {} end

	if (GetLocale() == "enUS") or (GetLocale() == "enGB") then

		if (qcQuestID == 0) or (qcQuestID == nil) then return nil end
		if not (GetQuestLogIndexByID(qcQuestID) == 0) then return nil end

		local qcQuestName = GetTitleText()

		local qcFaction = 0
		do
			local qcEFaction, _S = UnitFactionGroup("player")
			qcFaction = qcFactionBits[string.upper(qcEFaction)]
		end

		local qcRace = 0
		do
			local _S, qcERace = UnitRace("player")
			qcRace = qcRaceBits[string.upper(qcERace)]
		end

		local qcClass = 0
		do
			local _S, qcEClass = UnitClass("player")
			qcClass = qcClassBits[string.upper(qcEClass)]
		end

		local qcMapID = GetCurrentMapAreaID()
		local qcInitiatorName = (UnitName("questnpc") or "<Unknown Initiator>")
		do
			if (qcInitiatorName == UnitName("player")) then
				qcInitiatorName = "<Player>"
			end
		end

		local qcX
		local qcY
		do
			qcX, qcY = GetPlayerMapPosition("player")
			qcX = qcRound((qcX*100), 1)
			qcY = qcRound((qcY*100), 1)
		end

		if (qcProgressComplete[qcQuestID] == nil) then
			qcProgressComplete[qcQuestID] = {true,true,true,true,true,true,true,true,true}
			qcProgressComplete[qcQuestID][1] = qcQuestID
			qcProgressComplete[qcQuestID][2] = qcQuestName
			qcProgressComplete[qcQuestID][3] = qcFaction
			qcProgressComplete[qcQuestID][4] = qcRace
			qcProgressComplete[qcQuestID][5] = qcClass
			qcProgressComplete[qcQuestID][6] = qcMapID
			qcProgressComplete[qcQuestID][7] = qcInitiatorName
			qcProgressComplete[qcQuestID][8] = qcX
			qcProgressComplete[qcQuestID][9] = qcY
		else
			qcProgressComplete[qcQuestID][1] = qcQuestID
			qcProgressComplete[qcQuestID][2] = qcQuestName
			if (bit.band(qcProgressComplete[qcQuestID][3], qcFaction) == 0) then
				qcProgressComplete[qcQuestID][3] = tonumber(qcProgressComplete[qcQuestID][3] + qcFaction)
			end
			if (bit.band(qcProgressComplete[qcQuestID][4], qcRace) == 0) then
				qcProgressComplete[qcQuestID][4] = tonumber(qcProgressComplete[qcQuestID][4] + qcRace)
			end
			if (bit.band(qcProgressComplete[qcQuestID][5], qcClass) == 0) then
				qcProgressComplete[qcQuestID][5] = tonumber(qcProgressComplete[qcQuestID][5] + qcClass)
			end
			qcProgressComplete[qcQuestID][6] = qcMapID
			qcProgressComplete[qcQuestID][7] = qcInitiatorName
			qcProgressComplete[qcQuestID][8] = qcX
			qcProgressComplete[qcQuestID][9] = qcY
		end

	end

end

function qcCollectQuestOnAccept(...)

	if (qcCollectedQuests == nil) then qcCollectedQuests = {} end

	if (GetLocale() == "enUS") or (GetLocale() == "enGB") then

		local qcQuestLogIndex, qcQuestID = ...
		if (qcQuestID == 0) or (qcQuestID == nil) then return nil end

		local qcQuestName
		local qcQuestLevel = 0
		local qcQuestType = 1 --[[ Cannot detect repeatables that enter your log, or weekly quests ]]--
		do
			local _S, qcName, qcLevel, qcTag, qcDaily
			qcName, qcLevel, qcTag, _S, _S, _S, _S, qcDaily, _S, _S, _S = GetQuestLogTitle(qcQuestLogIndex)
			if (qcTag == nil) then qcQuestName = tostring(qcName) else qcQuestName = tostring(qcName .. " [" .. qcTag .. "]") end
			if (qcLevel > 0) then qcQuestLevel = qcLevel end
			if (qcDaily == 1) then qcQuestType = 3 end
		end

		local qcQuestCategory = (qcGetQuestCategory(qcQuestID) or "<Category Unknown>")

		local qcCategoryID = 99999
		for qcIndex, qcEntry in ipairs(qcQuestCategories) do
			if (qcEntry[2] == qcQuestCategory) then
				qcCategoryID = qcEntry[1]
				break
			end
		end

		local qcFaction = 0
		do
			local qcEFaction, _S = UnitFactionGroup("player")
			qcFaction = qcFactionBits[string.upper(qcEFaction)]
		end

		local qcRace = 0
		do
			local _S, qcERace = UnitRace("player")
			qcRace = qcRaceBits[string.upper(qcERace)]
		end

		local qcClass = 0
		do
			local _S, qcEClass = UnitClass("player")
			qcClass = qcClassBits[string.upper(qcEClass)]
		end

		local qcProfession = 0 -- TODO: Assign profession flag based on Category Header
		local qcSeasonal = 0 -- TODO: Assign seasonal flag based on Category Header

		local qcInitiatorName = (UnitName("questnpc") or "<Unknown Initiator>")

		if (qcCollectedQuests[qcQuestID] == nil) then
			qcCollectedQuests[qcQuestID] = {true,true,true,true,true,true,true,true,true,true,true}
			qcCollectedQuests[qcQuestID][1] = qcQuestID
			qcCollectedQuests[qcQuestID][2] = qcQuestName
			qcCollectedQuests[qcQuestID][3] = qcQuestLevel
			qcCollectedQuests[qcQuestID][4] = qcQuestCategory
			qcCollectedQuests[qcQuestID][5] = qcCategoryID
			qcCollectedQuests[qcQuestID][6] = qcQuestType
			qcCollectedQuests[qcQuestID][7] = qcFaction
			qcCollectedQuests[qcQuestID][8] = qcRace
			qcCollectedQuests[qcQuestID][9] = qcClass
			qcCollectedQuests[qcQuestID][10] = qcProfession
			qcCollectedQuests[qcQuestID][11] = qcSeasonal
			SelectQuestLogEntry(qcQuestLogIndex)
			local qcReputationCount = GetNumQuestLogRewardFactions()
			if (qcReputationCount > 0) then
				qcCollectedQuests[qcQuestID][12] = {}
				local qcFactionName, qcReputationAmount, qcFactionID, qcIsHeader, qcHasRep, _S
				local qcIndex = 0
				for qcReputationIndex = 1, qcReputationCount do
					qcFactionID, qcReputationAmount = GetQuestLogRewardFactionInfo(qcReputationIndex)
					qcFactionName, _S, _S, _S, _S, _S, _S, _S, qcIsHeader, _S, qcHasRep = GetFactionInfoByID(qcFactionID)
					if (qcFactionName and (not qcIsHeader or qcHasRep)) then
						qcIndex = (qcIndex + 1)
						qcReputationAmount = floor(qcReputationAmount / 100)
						qcCollectedQuests[qcQuestID][12][qcFactionID] = qcReputationAmount
					end
				end
			end
		else
			qcCollectedQuests[qcQuestID][1] = qcQuestID
			qcCollectedQuests[qcQuestID][2] = qcQuestName
			qcCollectedQuests[qcQuestID][3] = qcQuestLevel
			qcCollectedQuests[qcQuestID][4] = qcQuestCategory
			qcCollectedQuests[qcQuestID][5] = qcCategoryID
			qcCollectedQuests[qcQuestID][6] = qcQuestType
			if (bit.band(qcCollectedQuests[qcQuestID][7], qcFaction) == 0) then
				qcCollectedQuests[qcQuestID][7] = tonumber(qcCollectedQuests[qcQuestID][7] + qcFaction)
			end
			if (bit.band(qcCollectedQuests[qcQuestID][8], qcRace) == 0) then
				qcCollectedQuests[qcQuestID][8] = tonumber(qcCollectedQuests[qcQuestID][8] + qcRace)
			end
			if (bit.band(qcCollectedQuests[qcQuestID][9], qcClass) == 0) then
				qcCollectedQuests[qcQuestID][9] = tonumber(qcCollectedQuests[qcQuestID][9] + qcClass)
			end
			qcCollectedQuests[qcQuestID][10] = qcProfession
			qcCollectedQuests[qcQuestID][11] = qcSeasonal
			SelectQuestLogEntry(qcQuestLogIndex)
			local qcReputationCount = GetNumQuestLogRewardFactions()
			if (qcReputationCount > 0) then
				qcCollectedQuests[qcQuestID][12] = {}
				local qcFactionName, qcReputationAmount, qcFactionID, qcIsHeader, qcHasRep, _S
				local qcIndex = 0
				for qcReputationIndex = 1, qcReputationCount do
					qcFactionID, qcReputationAmount = GetQuestLogRewardFactionInfo(qcReputationIndex)
					qcFactionName, _S, _S, _S, _S, _S, _S, _S, qcIsHeader, _S, qcHasRep = GetFactionInfoByID(qcFactionID)
					if (qcFactionName and (not qcIsHeader or qcHasRep)) then
						qcIndex = (qcIndex + 1)
						qcReputationAmount = floor(qcReputationAmount / 100)
						qcCollectedQuests[qcQuestID][12][qcFactionID] = qcReputationAmount
					end
				end
			end
		end

	end

end

qcSVData = {
}