﻿--[[

	Quest Completist - qcCore.lua
	Written by: Alistair Maxwell

--]]

--[[ Internal Tables ]]--
qcCompletedQuests = {}
qcMapQuests = {}
qcPins = {}
qcSparePins = {}
qcWorkingDB = {}
qcWorkingLog = {}
qcSharedIDs = {}

--[[ Map Icon Tables ]]--
local qcCurrentMapInitiators = {} --[[ Stores quest initiators for a single Map ID ]]--
local qcCurrentMapIcons = {} --[[ Stores the icons for a single Map ID ]]--
local qcSpareMapIcons = {} --[[ Stores icons that can be recycled ]]--

--[[ Localization Table ]]--
local L = QuestCompletistLocalization

--[[ qcUpdateQuestList\qcGetCategoryQuests Variables ]]--
local qcCurrentCategoryID = 0
local qcCurrentCategoryQuestCount = 0
local qcCategoryQuests = {}

--[[ Vars ]]--
local qcCurrentMapZone = -1
local qcCurrentScrollPosition = 1
local qcMap = nil
local qcMapTooltip = nil
local qcQuestReputationTooltip = nil
local qcQuestInformationTooltip = nil
local qcToastTooltip = nil
local qcNewDataAlertTooltip = nil
local qcMutuallyExclusiveAlertTooltip = nil

--[[ Constants ]]--
local QCADDON_MSG_PREFIX = "QCOMPLETIST"
local QCADDON_VERSION = 99.2
local QCADDON_PURGE = true
local QCADDON_DELIMITER = ":"
local QCSINGLEPADDING = " "
local QCDOUBLEPADDING = "  "
local QCTRIPLEPADDING = "   "
local QCADDON_CHAT_TITLE = "|cFF9482C9Quest Completist:|r "

local COLOUR_DEATHKNIGHT = "|cffc41f3b"
local COLOUR_DRUID = "|cffff7d0a"
local COLOUR_HUNTER = "|cffabd473"
local COLOUR_MAGE = "|cff69ccf0"
local COLOUR_PALADIN = "|cfff58cba"
local COLOUR_PRIEST = "|cffffffff"
local COLOUR_ROGUE = "|cfffff569"
local COLOUR_SHAMAN = "|cff0070de"
local COLOUR_WARLOCK = "|cff9482c9"
local COLOUR_WARRIOR = "|cffc79c6e"

--[[ Bitwise Values ]]--
qcFactionBits = {["ALLIANCE"]=1,["HORDE"]=2}
qcRaceBits = {["HUMAN"]=1,["ORC"]=2,["DWARF"]=4,["NIGHTELF"]=8,["SCOURGE"]=16,["TAUREN"]=32,["GNOME"]=64,["TROLL"]=128,["GOBLIN"]=256,["BLOODELF"]=512,["DRAENEI"]=1024,["WORGEN"]=2048}
qcClassBits = {["WARRIOR"]=1,["PALADIN"]=2,["HUNTER"]=4,["ROGUE"]=8,["PRIEST"]=16,["DEATHKNIGHT"]=32,["SHAMAN"]=64,["MAGE"]=128,["WARLOCK"]=256,["DRUID"]=512}
--[[ 171=Alchemy, 164=Blacksmithing, 333=Enchanting, 202=Engineering, 773=Inscription, 755=Jewelcrafting, 165=Leatherworking, 197=Tailoring, 182=Herbalism, 186=Mining, 393=Skinning, 794=Archaeology, 129=First Aid, 185=Cooking, 356=Fishing ]]--
qcProfessionBits = {[171]=1,[164]=2,[333]=4,[202]=8,[773]=16,[755]=32,[165]=64,[197]=128,[182]=256,[186]=512,[393]=1024,[794]=2048,[129]=4096,[185]=8192,[356]=16384}

local qcHolidayDates = {
   [1]={"110920","111005"}, -- Brewfest 2011
   [2]={"120429","120505"}, -- Children's Week 2012
   [4]={"111101","111102"}, -- Day of the Dead 2011
   [8]={"111215","120102"}, -- Feast of Winter Veil 2011-2012
   [16]={"111018","111031"}, -- Hallow's End 2011
   [32]={"110906","110912"}, -- Harvest Festival TODO FOR 2012
   [64]={"120205","120218"}, -- Love is in the Air 2012
   [128]={"120122","120211"}, -- Lunar Festival 2012
   [256]={"120621","120704"}, -- Midsummer Fire Festival 2012
   [512]={"120408","120414"}, -- Noblegarden 2012
   [1024]={"111120","111126"}, -- Pilgrim's Bounty 2011
   [2048]={"120919","120919"}, -- Pirates' Day 2012
} 

--	Constants for the Key Bindings & Slash Commands --
BINDING_HEADER_QCQUESTCOMPLETIST = "Quest Completist";
BINDING_NAME_QCTOGGLEFRAME = "Toggle Frame";
SLASH_QUESTCOMPLETIST1 = "/qc"
SLASH_QUESTCOMPLETIST2 = "/questc"

SlashCmdList["QUESTCOMPLETIST"] = function(msg, editbox)
	ShowUIPanel(qcQuestCompletistUI)
end

function qcCopyTable(qcTable)

	if (qcTable == nil) then return nil end
	if not (type(qcTable) == "table") then return nil end

	local qcNewTable = {}
	for qcKey, qcValue in pairs(qcTable) do
		if (type(qcValue) == "table") then
			qcNewTable[qcKey] = qcCopyTable(qcValue)
		else
			qcNewTable[qcKey] = qcValue
		end
	end

	return qcNewTable

end

function qcScrollUpdate(qcValue)

	if not (qcCurrentScrollPosition == qcValue) then
		qcCurrentScrollPosition = qcValue
		qcUpdateQuestList(nil, qcValue)
	end

end

function qcUpdateCurrentCategoryText(qcCategoryID)

	qcQuestCompletistUI.qcSelectedCategory:SetText("#")
	for qcIndex, qcEntry in pairs(qcQuestCategories) do
		if (qcEntry[1] == qcCategoryID) then
			qcQuestCompletistUI.qcSelectedCategory:SetText(qcEntry[2])
			break
		end
	end

end

function qcCollectUpdate()

	if (qcCollectedQuests == nil) then qcCollectedQuests = {} end

	local qcUpdateCount = 0
	for qcIndex, qcEntry in pairs(qcCollectedQuests) do
		qcUpdateCount = (qcUpdateCount + 1)
	end

	print(QCADDON_CHAT_TITLE .. "Thanks for using Quest Completist. You have gathered " .. tostring(qcUpdateCount) .. " updates so far. Every now and then, please consider sending me your Saved Variables file so that I can include these updates in the next version.")

end

function qcGetCategoryQuests(qcCategoryID)

	local tableinsert = table.insert
	local tableremove = table.remove
	local bitband = bit.band
	local tablesort = table.sort

	wipe(qcCategoryQuests)

	local qcHoldingTable = {}
	for qcIndex, qcEntry in pairs(qcQuestDatabase) do
		if (qcEntry[5] == qcCategoryID) then
			tableinsert(qcHoldingTable, qcEntry)
		end
	end
	qcCategoryQuests = qcCopyTable(qcHoldingTable)
	wipe(qcHoldingTable)

	if (qcSettings["QC_L_HIDE_COMPLETED"] == 1) then
		for qcCategoryQuestsIndex = #qcCategoryQuests, 1, -1 do
			if not (qcCompletedQuests[qcCategoryQuests[qcCategoryQuestsIndex][1]] == nil) then
				if (qcCompletedQuests[qcCategoryQuests[qcCategoryQuestsIndex][1]]["C"] == 1) or (qcCompletedQuests[qcCategoryQuests[qcCategoryQuestsIndex][1]]["C"] == 2) then
					tableremove(qcCategoryQuests, qcCategoryQuestsIndex)
				end
			end
		end
	end

	if (qcSettings["QC_ML_HIDE_FACTION"] == 1) then
		local qcCurrentPlayerFaction, _S = UnitFactionGroup("player")
		local qcCurrentFaction = qcFactionBits[string.upper(qcCurrentPlayerFaction)]
		for qcCategoryQuestsIndex = #qcCategoryQuests, 1, -1 do
			if (bitband(qcCategoryQuests[qcCategoryQuestsIndex][7], qcCurrentFaction) == 0) then
				tableremove(qcCategoryQuests, qcCategoryQuestsIndex)
			end
		end
	end

	if (qcSettings["QC_ML_HIDE_RACECLASS"] == 1) then
		local _S, qcCurrentPlayerRace = UnitRace("player")
		local qcCurrentRace = qcRaceBits[string.upper(qcCurrentPlayerRace)]
		local _S, qcCurrentPlayerClass = UnitClass("player")
		local qcCurrentClass = qcClassBits[string.upper(qcCurrentPlayerClass)]
		for qcCategoryQuestsIndex = #qcCategoryQuests, 1, -1 do
			if (bitband(qcCategoryQuests[qcCategoryQuestsIndex][8], qcCurrentRace) == 0) then
				tableremove(qcCategoryQuests, qcCategoryQuestsIndex)
			elseif (bitband(qcCategoryQuests[qcCategoryQuestsIndex][9], qcCurrentClass) == 0) then
				tableremove(qcCategoryQuests, qcCategoryQuestsIndex)
			end
		end
	end

	--[[ Sort the zones quests ]]--
	if (qcSettings["SORT"] == 1) then
		tablesort(qcCategoryQuests, function(a, b) return (a[3]<b[3] or (a[3] == b[3] and a[2]<b[2])) end)
	elseif (qcSettings["SORT"] == 2) then
		tablesort(qcCategoryQuests, function(a, b) return a[2]<b[2] end)
	else --[[ No sort preference found, so sorting by level ]]--
		tablesort(qcCategoryQuests, function(a, b) return (a[3]<b[3] or (a[3] == b[3] and a[2]<b[2])) end)
	end

end

function qcUpdateQuestList(qcCategoryID, qcStartIndex)

	local qcOffset = 0
	local qcQuestListRecord = "qcMenuButton"
	local qcOpenLevel = "["
	local qcCloseLevel = "] "

	if not (qcCategoryID == nil) then
		qcCurrentCategoryID = qcCategoryID
		qcUpdateCurrentCategoryText(qcCategoryID)
		qcGetCategoryQuests(qcCategoryID)
		qcCurrentCategoryQuestCount = (#qcCategoryQuests)
		qcQuestCompletistUI.qcCurrentCategoryQuestCount:SetText(qcCurrentCategoryQuestCount .. " Quests Found")
		if (qcCurrentCategoryQuestCount < 16) then
			qcMenuSlider:SetMinMaxValues(1, 1)
		else
			qcMenuSlider:SetMinMaxValues(1, qcCurrentCategoryQuestCount - 15)
		end
		qcMenuSlider:SetValue(qcStartIndex)
	end

	for qcQuestListIndex = 1, 16 do
		qcOffset = ((qcQuestListIndex + qcStartIndex) - 1)
		if (qcCurrentCategoryQuestCount >= qcOffset) then
			_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetText(qcOpenLevel .. qcCategoryQuests[qcOffset][3] .. qcCloseLevel .. qcCategoryQuests[qcOffset][2])
			_G[qcQuestListRecord .. qcQuestListIndex].QuestID = qcCategoryQuests[qcOffset][1]
			--[[ Set initial quest icon ]]--
			_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexture("Interface\\MINIMAP\\ObjectIcons")
			if (qcCategoryQuests[qcOffset][6] == 1) then --[[ Normal ]]--
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0.125,0.25,0.25,0.50)
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:Show()
				_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(1.0, 1.0, 1.0, 1.0)
				
			elseif (qcCategoryQuests[qcOffset][6] == 2) then --[[ Repeatable ]]--
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0.5,0.625,0.25,0.50)
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:Show()
				_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(0.0941176470588235, 0.6274509803921569, 0.9411764705882353, 1.0)
			elseif (qcCategoryQuests[qcOffset][6] == 3) then --[[ Daily ]]--
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0.375,0.5,0.25,0.50)
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:Show()
				_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(0.0941176470588235, 0.6274509803921569, 0.9411764705882353, 1.0)
			elseif (qcCategoryQuests[qcOffset][6] == 4) then --[[ Special ]]--
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0,0.125,0,0.25)
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:Show()
				_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(1.0, 0.6156862745098039, 0.0862745098039216, 1.0)
			elseif (qcCategoryQuests[qcOffset][6] == 5) then --[[ Weekly ]]--
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0.125,0.25,0.25,0.50)
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:Show()
				_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(1.0, 1.0, 1.0, 1.0)
			elseif (qcCategoryQuests[qcOffset][6] == 6) then --[[ Profession ]]--
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0.125,0.25,0.75,1)
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:Show()
				_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(1.0, 1.0, 1.0, 1.0)
			elseif (qcCategoryQuests[qcOffset][6] == 7) then --[[ Seasonal ]]--
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0.75,0.875,0.25,0.5)
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:Show()
				_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(1.0, 1.0, 1.0, 1.0)
			else
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0.125,0.25,0.25,0.50)
				_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:Show()
				_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(1.0, 1.0, 1.0, 1.0)
			end
			--[[ Set faction icon ]]--
			if (qcCategoryQuests[qcOffset][7] == 1) then
				_G[qcQuestListRecord .. qcQuestListIndex].FactionIcon:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\AllianceIcon")
				_G[qcQuestListRecord .. qcQuestListIndex].FactionIcon:Show()
			elseif(qcCategoryQuests[qcOffset][7] == 2) then
				_G[qcQuestListRecord .. qcQuestListIndex].FactionIcon:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\HordeIcon")
				_G[qcQuestListRecord .. qcQuestListIndex].FactionIcon:Show()
			elseif(qcCategoryQuests[qcOffset][7] == 3) then
				_G[qcQuestListRecord .. qcQuestListIndex].FactionIcon:Hide()
			else
				_G[qcQuestListRecord .. qcQuestListIndex].FactionIcon:Hide()
			end
			--[[ Check quest state ]]--
			if (GetQuestLogIndexByID(qcCategoryQuests[qcOffset][1]) ~= 0) then
				local _S
				local qcIsComplete
				_S, _S, _S, _S, _S, _S, qcIsComplete, _S, _S = GetQuestLogTitle(GetQuestLogIndexByID(qcCategoryQuests[qcOffset][1]))
				if (qcIsComplete == nil) then --[[ Quest is yet to reach a conclusion ]]--
					_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\IncompleteQuestIcon")
					_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0,1,0,1)
					_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(0.5803921568627451, 0.5882352941176471, 0.5803921568627451, 1.0)
				elseif (qcIsComplete == 1) then --[[ Is completed ]]--
					_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\ActiveQuestIcon")
					_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0,1,0,1)
					_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(1.0, 0.8196078431372549, 0.0, 1.0)
				elseif (qcIsComplete == -1) then --[[ Is failed or abandoned ]]--
					_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\IncompleteQuestIcon")
					_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0,1,0,1)
					_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(0.9372549019607843, 0.1490196078431373, 0.0627450980392157, 1.0)
				end
			end
			--[[ Check completed state. Does not colour repeatables	or daily quests. ]]--
			if not (qcCompletedQuests[qcCategoryQuests[qcOffset][1]] == nil) then
				if not (qcCategoryQuests[qcOffset][6] == 2) or (qcCategoryQuests[qcOffset][6] == 3) then
					--[[ Quest is not repeatable or daily ]]--
					if (qcCompletedQuests[qcCategoryQuests[qcOffset][1]]["C"] == 1) then --[[ Quest is complete ]]--
						_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\QuestCompleteIcon")
						_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0,1,0,1)
						_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(0.0, 1.0, 0.0, 1.0)
					elseif (qcCompletedQuests[qcCategoryQuests[qcOffset][1]]["C"] == 2) then --[[ Quest is unattainable ]]--
						_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\UnattainableIcon")
						_G[qcQuestListRecord .. qcQuestListIndex].QuestIcon:SetTexCoord(0,1,0,1)
						_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetTextColor(0.77, 0.12, 0.23, 1.0)
					end
				else
					--[[ Shouldn't need to do anything here? Should still be blue with correct icon? ]]--
				end
			end
			_G[qcQuestListRecord .. qcQuestListIndex]:Show()
			_G[qcQuestListRecord .. qcQuestListIndex]:Enable()
		else
			_G[qcQuestListRecord .. qcQuestListIndex].QuestName:SetText(QCSINGLEPADDING)
			_G[qcQuestListRecord .. qcQuestListIndex]:Hide()
			_G[qcQuestListRecord .. qcQuestListIndex]:Disable()
		end
	end

end

function qcQuestQueryCompleted()

	--TODO: I dont use qcUnknownQuestCount, strings need updated

	local qcRecievedQuestCount = 0
	local qcAddedToQuestHistory = 0
	local qcUnknownQuestCount = 0
	local qcCompletedTable = {}

	wipe(qcCompletedTable)
	GetQuestsCompleted(qcCompletedTable)

	for qcCompletedTableIndex, qcCompletedQuest in pairs(qcCompletedTable) do
		qcRecievedQuestCount = (qcRecievedQuestCount + 1)
		if (qcCompletedQuests[qcCompletedTableIndex] == nil) then
			qcAddedToQuestHistory = (qcAddedToQuestHistory + 1)
			qcCompletedQuests[qcCompletedTableIndex] = {}
		end
		qcCompletedQuests[qcCompletedTableIndex]["C"] = 1 --[[ Completed ]]--
	end

	print(QCADDON_CHAT_TITLE .. string.format(L["QC_QUERYRESULTS"], qcRecievedQuestCount, qcAddedToQuestHistory, qcUnknownQuestCount))
	qcUpdateQuestList(nil, qcMenuSlider:GetValue())

end

local function qcClearUpdateCache()

	--[[ Clear the caches ]]--
	wipe(qcCompletedQuests)
	print(QCADDON_CHAT_TITLE .. "Cache Cleared.")

end

local function qcPurgeCollectedCache()

	if (qcCollectedQuests == nil) then qcCollectedQuests = {} end
	if (qcProgressComplete == nil) then qcProgressComplete = {} end
	wipe(qcCollectedQuests)
	wipe(qcProgressComplete)
	print(QCADDON_CHAT_TITLE .. "Collected Cache Purged.")

end

function qcMenuMouseWheel(self, delta)

	local qcCurrentPosition = qcMenuSlider:GetValue()

	if (delta < 0) and (qcCurrentPosition < qcCurrentCategoryQuestCount) then
		qcMenuSlider:SetValue(qcCurrentPosition + 2)
	elseif (delta > 0) and (qcCurrentPosition > 1) then
		qcMenuSlider:SetValue(qcCurrentPosition - 2)
	end

end

function qcCategoryDropdown_Initialize(self, level)

local qcDropDownInfo

	if (level == 1) then

		--[[ Continents & Continent Zones ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Continents"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		for qcContinentIndex, qcContinent in ipairs(qcContinents) do
			qcDropDownInfo = UIDropDownMenu_CreateInfo()
			qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. qcContinent[2])
			qcDropDownInfo.notCheckable = true
			qcDropDownInfo.hasArrow = true
			qcDropDownInfo.value = qcContinent[1]
			UIDropDownMenu_AddButton(qcDropDownInfo, level)
		end
		--[[ Dungeon\Raid Categories and Dungeons\Raids ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Dungeons & Raids"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		for qcDungeonCategoryIndex, qcDungeonCategory in ipairs(qcDungeonCategories) do
			qcDropDownInfo = UIDropDownMenu_CreateInfo()
			qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. qcDungeonCategory[2])
			qcDropDownInfo.notCheckable = true
			qcDropDownInfo.hasArrow = true
			qcDropDownInfo.value = qcDungeonCategory[1]
			UIDropDownMenu_AddButton(qcDropDownInfo, level)
		end
		--[[ Class Quests ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Class Quests"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Classes")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.hasArrow = true
		qcDropDownInfo.value = 20
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		--[[ Miscellaneous Section ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Miscellaneous"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		for qcMiscellaneousCategoriesIndex, qcMiscellaneousCategory in ipairs(qcMiscellaneousCategories) do
			qcDropDownInfo = UIDropDownMenu_CreateInfo()
			qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. qcMiscellaneousCategory[2])
			qcDropDownInfo.notCheckable = true
			qcDropDownInfo.hasArrow = true
			qcDropDownInfo.value = qcMiscellaneousCategory[1]
			UIDropDownMenu_AddButton(qcDropDownInfo, level)
		end
		--[[ Options ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Options"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Perform Server Query...")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.colorCode = COLOUR_DRUID
		qcDropDownInfo.arg1 = "SERVERQUERY"
		function qcDropDownInfo.func(button, arg1)
			print(QCADDON_CHAT_TITLE .. L["QC_QUERYREQUESTED"])
			qcQuestCompletistUI:RegisterEvent("QUEST_QUERY_COMPLETE")
			QueryQuestsCompleted()
		end
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Clear Update Cache...")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.colorCode = COLOUR_DRUID
		qcDropDownInfo.arg1 = "CLEARUPDATECACHE"
		function qcDropDownInfo.func(button, arg1)
			print(QCADDON_CHAT_TITLE .. "Clearing your update Cache...")
			qcClearUpdateCache()
		end
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Sort: Level")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.colorCode = COLOUR_DRUID
		qcDropDownInfo.arg1 = "SORTLEVEL"
		function qcDropDownInfo.func(button, arg1)
			qcSettings["SORT"] = 1
			qcUpdateQuestList(qcCurrentCategoryID, qcMenuSlider:GetValue())
		end
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Sort: Alphabetically")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.colorCode = COLOUR_DRUID
		qcDropDownInfo.arg1 = "SORTAPLHA"
		function qcDropDownInfo.func(button, arg1)
			qcSettings["SORT"] = 2
			qcUpdateQuestList(qcCurrentCategoryID, qcMenuSlider:GetValue())
		end
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

	elseif (level == 2) then

		--[[ Kalimdor ]]--
		if (UIDROPDOWNMENU_MENU_VALUE == 1) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 1) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Eastern Kingdoms ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 2) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 2) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Outland ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 3) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 3) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Northrend ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 4) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 4) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ The Maelstrom ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 5) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 5) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Classic Dungeon ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 10) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 10) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ The Burning Crusade Dungeon ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 11) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 11) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Wrath of the Lich King Dungeon ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 12) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 12) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Cataclysm Dungeon ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 13) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 13) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Raid ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 14) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 14) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Classes ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 20) then
			for qcClassIndex, qcClass in ipairs(qcClasses) do
				if (qcClass[1] == 20) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcClass[3]
					qcDropDownInfo.notCheckable = false
					qcDropDownInfo.colorCode = qcClass[4]
					if (qcCurrentCategoryID == qcClass[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcClass[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Professions ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 30) then
			for qcProfessionIndex, qcProfession in ipairs(qcProfessions) do
				if (qcProfession[1] == 30) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcProfession[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcProfession[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcProfession[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ World Events ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 31) then
			for qcWorldEventIndex, qcWorldEvent in ipairs(qcWorldEvents) do
				if (qcWorldEvent[1] == 31) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcWorldEvent[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcWorldEvent[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcWorldEvent[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Battlegrounds ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 32) then
			for qcBattlegroundIndex, qcBattleground in ipairs(qcBattlegrounds) do
				if (qcBattleground[1] == 32) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcBattleground[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcBattleground[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcBattleground[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Specials ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 33) then
			for qcSpecialIndex, qcSpecial in ipairs(qcSpecials) do
				if (qcSpecial[1] == 33) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcSpecial[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentCategoryID == qcSpecial[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcSpecial[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateQuestList(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		end

	end

end

function qcCategoryDropdownButton_OnClick(self, button, down)

	local name = self:GetName()
	ToggleDropDownMenu(1, nil, qcCategoryDropdownMenu, name, 0, 0)

end

function qcCategoryDropdown_OnLoad(self)

	UIDropDownMenu_Initialize(self, qcCategoryDropdown_Initialize)

end

local function qcGetZoneIDFromName(zoneName)

	for qcZoneIndex, qcZone in ipairs(qcQuestCategories) do
		if (qcZone[2] == zoneName) then
			return qcZone[1]
		end
	end

	return nil

end

local function qcZoneChangedNewArea()

	local qcZoneName
	SetMapToCurrentZone()
	if (GetCurrentMapContinent() > 0) then
		local qcCurrentContinentZones = {GetMapZones(GetCurrentMapContinent())}
		qcZoneName = qcCurrentContinentZones[GetCurrentMapZone()]
	else --[[ Instance\Exception support ]]--
		qcZoneName = GetRealZoneText()
	end

	local qcZoneID = qcGetZoneIDFromName(qcZoneName)
	if (qcZoneID ~= nil) then
		qcUpdateQuestList(qcZoneID, 1)
	end

end

function qcUpdateTooltip(qcButtonIndex)

	local qcQuestListRecord = "qcMenuButton"
	local qcQuestID = _G[qcQuestListRecord .. qcButtonIndex].QuestID

	if not (qcQuestID == nil) then
		qcQuestInformationTooltip:SetOwner(qcQuestCompletistUI, "ANCHOR_BOTTOMRIGHT", -30, 500)
		qcQuestInformationTooltip:ClearLines()
		qcQuestInformationTooltip:SetHyperlink("quest:" .. qcQuestID)
		qcQuestInformationTooltip:AddLine(" ")
		qcQuestInformationTooltip:AddDoubleLine("Quest ID:", COLOUR_MAGE .. qcQuestID)
		if not (qcQuestDatabase[qcQuestID][13] == nil) then
			for qcInitiatorIndex, qcInitiatorEntry in ipairs(qcQuestDatabase[qcQuestID][13]) do
				local qcInitiatorID = qcInitiatorEntry[1]
				local qcInitiatorName = qcInitiatorEntry[2]
				local qcInitiatorMapID = qcInitiatorEntry[3]
				local qcInitiatorMapLevel = qcInitiatorEntry[4]
				local qcInitiatorX = qcInitiatorEntry[5]
				local qcInitiatorY = qcInitiatorEntry[6]
				if not (qcInitiatorID == 0) then
					qcQuestInformationTooltip:AddDoubleLine("Quest Giver:", COLOUR_HUNTER .. qcInitiatorName .. " [" .. qcInitiatorID .. "]")
				else
					qcQuestInformationTooltip:AddDoubleLine("Quest Giver:", COLOUR_HUNTER .. qcInitiatorName)
				end
				if not (qcInitiatorMapLevel == 0) then
					qcQuestInformationTooltip:AddDoubleLine("  - Location:", COLOUR_HUNTER .. GetMapNameByID(qcInitiatorMapID) .. ", " .. qcInitiatorMapLevel .. " @ " .. qcInitiatorX .. ", " .. qcInitiatorY, nil, nil, nil, true)
				else
					qcQuestInformationTooltip:AddDoubleLine("  - Location:", COLOUR_HUNTER .. GetMapNameByID(qcInitiatorMapID) .. " @ " .. qcInitiatorX .. ", " .. qcInitiatorY, nil, nil, nil, true)
				end
			end
		end
		qcQuestInformationTooltip:Show()
		qcQuestReputationTooltip:SetOwner(qcQuestInformationTooltip, "ANCHOR_BOTTOMRIGHT", -qcQuestInformationTooltip:GetWidth())
		qcQuestReputationTooltip:ClearLines()
		local qcReputationCount = 0
		if not (qcQuestDatabase[qcQuestID][12] == nil) then
			qcQuestReputationTooltip:AddLine(GetText("COMBAT_TEXT_SHOW_REPUTATION_TEXT"))
			qcQuestReputationTooltip:AddLine(" ")
			for qcReputationIndex, qcReputationEntry in pairs(qcQuestDatabase[qcQuestID][12]) do
				qcReputationCount = (qcReputationCount + 1)
				qcQuestReputationTooltip:AddDoubleLine(tostring(qcFactions[qcReputationIndex] or qcReputationIndex), COLOUR_DRUID .. tostring(qcReputationEntry .. " rep"))
			end
			if (qcReputationCount > 0) then
				qcQuestReputationTooltip:Show()
			else
				qcQuestReputationTooltip:Hide()
			end
		end
	else
		qcQuestReputationTooltip:Hide()
	end

end
function RunSlashCmd(cmd)

local slash, rest = cmd:match("^(%S+)%s*(.-)$")

	for name, func in pairs(SlashCmdList) do
		local i, slashCmd = 1
		repeat
		slashCmd, i = _G["SLASH_"..name..i], i + 1
			if slashCmd == slash then
				return true, func(rest)
			end
		until not slashCmd
	end

local i = 1

	while _G["EMOTE" .. i .. "_TOKEN"] do
		local j, cn = 2, _G["EMOTE" .. i .. "_CMD1"]
		while cn do
			if cn == slash then
				return true, DoEmote(_G["EMOTE" .. i .. "_TOKEN"], rest)
			end
			j, cn = j+1, _G["EMOTE" .. i .. "_CMD" .. j]
		end
	i = i + 1
	end

end

function qcQuestClick(qcButtonIndex)

	local qcQuestListRecord = "qcMenuButton"
	local qcQuestID = _G[qcQuestListRecord .. qcButtonIndex].QuestID

	if (IsLeftShiftKeyDown() == nil) and (IsLeftAltKeyDown() == nil) then
		--if IsAddOnLoaded('TomTom') then
		--	if (qcQuestProviderZone == nil) or (qcQuestProviderX == nil) or (qcQuestProviderY == nil) then
		--		return nil
		--	else
		--		if (qcQuestProviderName == nil) then
		--			qcQuestProviderName = "Quest Giver"
		--		end
		--		RunSlashCmd("/tway " .. qcQuestProviderZone .. " " .. qcQuestProviderX .. " " .. qcQuestProviderY .. " " .. qcQuestProviderName .. " in " .. qcQuestProviderZone)
		--		RunSlashCmd("/cway")
		--	end
		--end
	elseif (IsLeftShiftKeyDown() ~= nil) then --[[ User wants to toggle the completed status of a quest ]]--
		if (qcCompletedQuests[qcQuestID] == nil) then
			qcCompletedQuests[qcQuestID] = {}
			qcCompletedQuests[qcQuestID]["C"] = 1
		else
			if (qcCompletedQuests[qcQuestID]["C"] == nil) then
				qcCompletedQuests[qcQuestID]["C"] = 1
			else
				if (qcCompletedQuests[qcQuestID]["C"] == 1) then
					qcCompletedQuests[qcQuestID]["C"] = 0
				elseif (qcCompletedQuests[qcQuestID]["C"] == 0) then
					qcCompletedQuests[qcQuestID]["C"] = 1
				elseif (qcCompletedQuests[qcQuestID]["C"] == 2) then
					qcCompletedQuests[qcQuestID]["C"] = 1
				end
			end
		end
	elseif (IsLeftAltKeyDown() ~= nil) then --[[ User wants to toggle the unattainable status of a quest ]]--
		if (qcCompletedQuests[qcQuestID] == nil) then
			qcCompletedQuests[qcQuestID] = {}
			qcCompletedQuests[qcQuestID]["C"] = 2
		else
			if (qcCompletedQuests[qcQuestID]["C"] == nil) then
				qcCompletedQuests[qcQuestID]["C"] = 2
			else
				if (qcCompletedQuests[qcQuestID]["C"] == 2) then
					qcCompletedQuests[qcQuestID]["C"] = 0
				elseif (qcCompletedQuests[qcQuestID]["C"] == 0) then
					qcCompletedQuests[qcQuestID]["C"] = 2
				elseif (qcCompletedQuests[qcQuestID]["C"] == 1) then
					qcCompletedQuests[qcQuestID]["C"] = 2
				end
			end
		end
	end

	qcUpdateQuestList(nil, qcMenuSlider:GetValue())

end

function qcCloseTooltip()

	qcQuestInformationTooltip:Hide()
	qcQuestReputationTooltip:Hide()

end

local function qcUpdateCompletedQuest(qcQuestID)

	if (qcCompletedQuests[qcQuestID] == nil) then
		qcCompletedQuests[qcQuestID] = {}
		qcCompletedQuests[qcQuestID]["C"] = 1
	else
		--[[ Possible future update code ]]--
	end

end

local function qcUpdateMutuallyExclusiveCompletedQuest(qcQuestID)

	if not (qcMutuallyExclusive[qcQuestID] == nil) then
		for qcMutuallyExclusiveIndex, qcMutuallyExclusiveEntry in pairs(qcMutuallyExclusive[qcQuestID]) do
			if not (qcQuestDatabase[qcMutuallyExclusiveEntry] == nil) then
				qcCompletedQuests[qcMutuallyExclusiveEntry] = {}
				qcCompletedQuests[qcMutuallyExclusiveEntry]["C"] = 1
				local qcQuestName = qcMutuallyExclusiveQuestInformation(qcMutuallyExclusiveEntry)
				if (qcQuestName and qcMutuallyExclusiveEntry) then
					print(QCADDON_CHAT_TITLE .. qcQuestName .. " [" .. qcMutuallyExclusiveEntry .. "] was mutually exclusive and flagged as completed.")
				end
			end
		end
	end

end

local function qcUpdateSkippedBreadcrumbQuest(qcQuestID)

	if not (qcBreadcrumbQuests[qcQuestID] == nil) then
		for qcBreadcrumbIndex, qcBreadcrumbEntry in pairs(qcBreadcrumbQuests[qcQuestID]) do
			if not (qcQuestDatabase[qcBreadcrumbEntry] == nil) then
				qcCompletedQuests[qcBreadcrumbEntry] = {}
				qcCompletedQuests[qcBreadcrumbEntry]["C"] = 1
				--[[ TODO: Inform user like mutually exclusives? ]]--
			end
		end
	end

end

local function qcNewDataChecks(qcQuestID)

	if (qcQuestID == nil) or (qcQuestID == 0) then return nil end

	if (qcQuestDatabase[qcQuestID] == nil) then
		qcNewDataAlert:Show()
	else
		qcNewDataAlert:Hide()
	end

end

local function qcBreadcrumbChecks(qcQuestID)

	if (qcQuestID == nil) or (qcQuestID == 0) then return nil end

	if (qcBreadcrumbQuests[qcQuestID] == nil) then
		qcToast.QuestID = nil
		qcToast:Hide()
	else
		qcToast.QuestID = qcQuestID
		local qcCount = 0
		for qcBreadcrumbIndex, qcBreadcrumbEntry in pairs(qcBreadcrumbQuests[qcQuestID]) do
			if (qcCompletedQuests[qcBreadcrumbEntry] == nil) then
				qcCount = (qcCount + 1)
				if (qcCount <= 1) then
					qcToastText:SetText(qcCount .. " Breadcrumb Available!")
				else
					qcToastText:SetText(qcCount .. " Breadcrumbs Available!")
				end
			else
				if not (qcCompletedQuests[tonumber(qcBreadcrumbEntry)]["C"] == 1) then
					qcCount = (qcCount + 1)
					if (qcCount <= 1) then
						qcToastText:SetText(qcCount .. " Breadcrumb Available!")
					else
						qcToastText:SetText(qcCount .. " Breadcrumbs Available!")
					end
				end
			end
		end
		if (qcCount == 0) then
			qcToast.QuestID = nil
			qcToast:Hide()
		else
			qcToast:Show()
		end
	end

end

local function qcMutuallyExclusiveChecks(qcQuestID)

	if (qcQuestID == nil) or (qcQuestID == 0) then return nil end

	if (qcMutuallyExclusive[qcQuestID] == nil) then
		qcMutuallyExclusiveAlert.QuestID = nil
		qcMutuallyExclusiveAlert:Hide()
	else
		qcMutuallyExclusiveAlert.QuestID = qcQuestID
		qcMutuallyExclusiveAlert:Show()
	end

end

function qcMapTooltipSetup()

local qcFrame = CreateFrame("GameTooltip", "qcMapTooltip", WorldMapDetailFrame, "GameTooltipTemplate")

	qcFrame:SetFrameStrata("TOOLTIP")
	qcMapTooltip = qcFrame
	--[[ Hook to allow scale changes with the world map ]]--
	WorldMapDetailFrame:HookScript("OnSizeChanged", function(self)
	qcMapTooltip:SetScale(1/self:GetScale())
	end)

end

function qcQuestReputationTooltipSetup()

local qcFrame = CreateFrame("GameTooltip", "qcQuestReputationTooltip", qcQuestCompletistUI, "GameTooltipTemplate")

	qcFrame:SetFrameStrata("TOOLTIP")
	qcQuestReputationTooltip = qcFrame

end

function qcQuestInformationTooltipSetup()

local qcFrame = CreateFrame("GameTooltip", "qcQuestInformationTooltip", qcQuestCompletistUI, "GameTooltipTemplate")

	qcFrame:SetFrameStrata("TOOLTIP")
	qcQuestInformationTooltip = qcFrame

end

function qcToastTooltipSetup()

local qcFrame = CreateFrame("GameTooltip", "qcToastTooltip", qcToast, "GameTooltipTemplate")

	qcFrame:SetFrameStrata("TOOLTIP")
	qcToastTooltip = qcFrame

end

function qcNewDataAlertTooltipSetup()

local qcFrame = CreateFrame("GameTooltip", "qcNewDataAlertTooltip", qcNewDataAlert, "GameTooltipTemplate")

	qcFrame:SetFrameStrata("TOOLTIP")
	qcNewDataAlertTooltip = qcFrame

end

function qcMutuallyExclusiveAlertTooltipSetup()

local qcFrame = CreateFrame("GameTooltip", "qcMutuallyExclusiveAlertTooltip", qcMutuallyExclusiveAlert, "GameTooltipTemplate")

	qcFrame:SetFrameStrata("TOOLTIP")
	qcMutuallyExclusiveAlertTooltip = qcFrame

end

function qcGetToastQuestInformation(qcQuestID)

	if (qcQuestID) then
		if (qcQuestDatabase[qcQuestID]) then
			local qcQuestName
			qcQuestName = (qcQuestDatabase[qcQuestID][2] or nil)
			return qcQuestName
		end
	end

end

function qcToast_OnEnter(self)

	if (self.QuestID == nil) then
		qcToastTooltip:Hide()
	else
		qcToastTooltip:SetOwner(qcToast, "ANCHOR_CURSOR")
		qcToastTooltip:ClearLines()
		qcToastTooltip:AddLine("Breadcrumb Quests")
		for qcBreadcrumbIndex, qcBreadcrumbEntry in pairs(qcBreadcrumbQuests[tonumber(self.QuestID)]) do
			if (qcCompletedQuests[tonumber(qcBreadcrumbEntry)] == nil) then
				local qcQuestName = qcGetToastQuestInformation(qcBreadcrumbEntry)
				if (qcQuestName and qcBreadcrumbEntry) then qcToastTooltip:AddLine(tostring(COLOUR_DRUID .. qcQuestName .. COLOUR_MAGE .. " [" .. qcBreadcrumbEntry .. "]")) end
			else
				if (not qcCompletedQuests[tonumber(qcBreadcrumbEntry)]["C"] == 1) then
				local qcQuestName = qcGetToastQuestInformation(qcBreadcrumbEntry)
				if (qcQuestName and qcBreadcrumbEntry) then qcToastTooltip:AddLine(tostring(COLOUR_DRUID .. qcQuestName .. " [" .. qcBreadcrumbEntry .. "]")) end
				end
			end
		end
		qcToastTooltip:Show()
	end

end

function qcToast_OnLeave(self)

	qcToastTooltip:Hide()

end

function qcNewDataAlert_OnEnter(self)

	qcNewDataAlertTooltip:SetOwner(qcNewDataAlert, "ANCHOR_CURSOR")
	qcNewDataAlertTooltip:ClearLines()
	qcNewDataAlertTooltip:AddLine("Quest Completist")
	qcNewDataAlertTooltip:AddLine(COLOUR_HUNTER .. "Quest Completist can't find information on this quest. Please help me improve the addon by reading the 'How To Contribute' file in the addon folder, and submitting your saved variable data.", nil, nil, nil, true)
	qcNewDataAlertTooltip:Show()

end

function qcNewDataAlert_OnLeave(self)

	qcNewDataAlertTooltip:Hide()

end

function qcMutuallyExclusiveQuestInformation(qcQuestID)

	if (qcQuestID) then
		if (qcQuestDatabase[qcQuestID]) then
			local qcQuestName
			qcQuestName = (qcQuestDatabase[qcQuestID][2] or nil)
			return qcQuestName
		end
	end

	return nil

end

function qcMutuallyExclusiveAlert_OnEnter(self)

	if (self.QuestID == nil) then
		qcMutuallyExclusiveAlertTooltip:Hide()
	else
		qcMutuallyExclusiveAlertTooltip:SetOwner(qcMutuallyExclusiveAlert, "ANCHOR_BOTTOMRIGHT")
		qcMutuallyExclusiveAlertTooltip:ClearLines()
		qcMutuallyExclusiveAlertTooltip:AddLine("Quest Completist")
		qcMutuallyExclusiveAlertTooltip:AddLine(COLOUR_MAGE .. "This quest is mutually exclusive with others, meaning you can only complete one of them. The other quests are:", nil, nil, nil, true)
		for qcMutuallyExclusiveIndex, qcMutuallyExclusiveEntry in pairs(qcMutuallyExclusive[tonumber(self.QuestID)]) do
			if (qcQuestDatabase[tonumber(qcMutuallyExclusiveEntry)] == nil) then
				qcMutuallyExclusiveAlertTooltip:AddLine(tostring(COLOUR_DRUID .. "<Quest Not Found In DB> [" .. qcMutuallyExclusiveEntry .. "]"))
			else
				local qcQuestName = qcMutuallyExclusiveQuestInformation(qcMutuallyExclusiveEntry)
				if (qcQuestName and qcMutuallyExclusiveEntry) then qcMutuallyExclusiveAlertTooltip:AddLine(tostring(COLOUR_DRUID .. qcQuestName .. " [" .. qcMutuallyExclusiveEntry .. "]")) end
			end
		end
		qcMutuallyExclusiveAlertTooltip:Show()
	end
end

function qcMutuallyExclusiveAlert_OnLeave(self)

	qcMutuallyExclusiveAlertTooltip:Hide()

end

function qcShowPin(x, y, qcQuestType, qcQuestID, qcQuestName)

local qcPin = qcPreparePin(qcQuestType, qcQuestID, qcQuestName)

	qcPin:ClearAllPoints()
	qcPin:SetFrameLevel(WorldMapPOIFrame:GetFrameLevel()-1)
	qcPin:SetPoint("CENTER", WorldMapDetailFrame, "TOPLEFT", (x/100)*WorldMapDetailFrame:GetWidth(), (-y/100)*WorldMapDetailFrame:GetHeight())
	qcPin:Show()

end

function qcPreparePin(qcQuestType, qcQuestID, qcQuestName)

local qcNewPin

	if (#qcSparePins > 0) then
		qcNewPin = qcSparePins[1]
		table.remove(qcSparePins, 1)
		--print("Reusing pin: " .. tostring(qcNewPin))
	end

	if (not qcNewPin) then
		qcNewPin = CreateFrame("Frame", nil, WorldMapDetailFrame)
		qcNewPin.Texture = qcNewPin:CreateTexture()

		--[[ Tooltip Hooks ]]--
		qcNewPin:HookScript("OnEnter", function(self)
		qcMapTooltip:SetOwner(self, "ANCHOR_RIGHT")
		qcMapTooltip:ClearLines()
		if (self.QuestID ~= nil) then qcMapTooltip:SetHyperlink("quest:" .. self.QuestID) end
		qcMapTooltip:AddLine(" ")
		qcMapTooltip:AppendText(" [" .. (self.QuestID or "ID_ERROR") .. "]")
		if (self.QuestID ~= nil) and (qcQuestDBv2[self.QuestID][3][2] ~= nil) and (qcQuestDBv2[self.QuestID][3][1] ~= nil) then qcMapTooltip:AddDoubleLine("Starts from:", tostring(COLOUR_DRUID .. qcQuestDBv2[self.QuestID][3][2] .. " [" .. qcQuestDBv2[self.QuestID][3][1]  .. "]")) end
		if (self.QuestID ~= nil) and (qcQuestDBv2[self.QuestID][1][4] ~= nil) then qcMapTooltip:AddDoubleLine("Category:", tostring(COLOUR_DRUID .. qcQuestDBv2[self.QuestID][1][4])) end
		if (self.QuestID ~= nil) and (qcQuestDBv2[self.QuestID][5][1] ~= nil) then qcMapTooltip:AddLine(COLOUR_HUNTER .. (qcQuestDBv2[self.QuestID][5][1] or ""), nil, nil, nil, true) end
		qcMapTooltip:AddLine(COLOUR_MAGE .. "Please report errors to me on Curse.com")
		qcMapTooltip:Show()
		end)
		qcNewPin:HookScript("OnLeave", function(self)
		qcMapTooltip:ClearLines()
		qcMapTooltip:Hide()
		end)

		if (qcQuestType == 3) then --[[ Daily ]]--
			qcNewPin.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewPin.Texture:SetTexCoord(0.375, 0.500, 0.250, 0.500)
			qcNewPin.Texture:SetAllPoints()
			qcNewPin:SetWidth(16)
			qcNewPin:SetHeight(16)
			qcNewPin:EnableMouse(true)
		elseif (qcQuestType == 2) then --[[ Repeatable ]]--
			qcNewPin.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewPin.Texture:SetTexCoord(0.500, 0.625, 0.250, 0.500)
			qcNewPin.Texture:SetAllPoints()
			qcNewPin:SetWidth(16)
			qcNewPin:SetHeight(16)
			qcNewPin:EnableMouse(true)
		else
			qcNewPin.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewPin.Texture:SetTexCoord(0.125, 0.250, 0.250, 0.250)
			qcNewPin.Texture:SetAllPoints()
			qcNewPin:SetWidth(16)
			qcNewPin:SetHeight(16)
			qcNewPin:EnableMouse(true)
		end
		table.insert(qcPins, qcNewPin)
	else
		if (qcQuestType == 3) then --[[ Daily ]]--
			qcNewPin.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewPin.Texture:SetTexCoord(0.375, 0.500, 0.250, 0.500)
			qcNewPin.Texture:SetAllPoints()
			qcNewPin:SetWidth(16)
			qcNewPin:SetHeight(16)
			qcNewPin:EnableMouse(true)
		elseif (qcQuestType == 2) then --[[ Repeatable ]]--
			qcNewPin.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewPin.Texture:SetTexCoord(0.500, 0.625, 0.250, 0.500)
			qcNewPin.Texture:SetAllPoints()
			qcNewPin:SetWidth(16)
			qcNewPin:SetHeight(16)
			qcNewPin:EnableMouse(true)
		else
			qcNewPin.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewPin.Texture:SetTexCoord(0.125, 0.250, 0.250, 0.500)
			qcNewPin.Texture:SetAllPoints()
			qcNewPin:SetWidth(16)
			qcNewPin:SetHeight(16)
			qcNewPin:EnableMouse(true)
		end
		qcNewPin.QuestID = qcQuestID
		qcNewPin.QuestName = qcQuestName
		table.insert(qcPins, qcNewPin)
	end

	return qcNewPin

end

function qcHidePin(qcPinID, qcEntry)

	if (qcPinID) then
		qcEntry:Hide()
		table.insert(qcSparePins, qcEntry)
		table.remove(qcPins, qcPinID)
	end

end

function qcHideAllPins()

	for qcIndex = #qcPins, 1, -1 do
		qcHidePin(qcIndex, qcPins[qcIndex])
	end

end

function qcRefreshPins(qcMapID)

	qcHideAllPins()
	wipe(qcMapQuests)

	if (qcSettings["SHOWMAPICONS"] == 0) then
		return nil
	end

	--[[ Pull all quests related to this map from the database ]]--
	for qcIndex, qcEntry in pairs(qcQuestDBv2) do
		if (qcEntry[3][6] == qcMapID) then
			if (qcEntry[3][8] == 0) then
				table.insert(qcMapQuests, qcEntry)
			end
		end
	end

	--[[ Completed filtering - regardless of user filtering options ]]--
	for qcMapQuestsIndex = #qcMapQuests, 1, -1 do
		if (qcCompletedQuests[qcMapQuests[qcMapQuestsIndex][1][1]] ~= nil) then
			if (qcCompletedQuests[qcMapQuests[qcMapQuestsIndex][1][1]]["C"] == 1) or (qcCompletedQuests[qcMapQuests[qcMapQuestsIndex][1][1]]["C"] == 2) then
				if (qcMapQuests[qcMapQuestsIndex][1][5] == 1) or (qcMapQuests[qcMapQuestsIndex][1][5] == 4) then --[[ Normal quest or special quest ]]--
					--print("Removing completed quest: " .. qcMapQuests[qcMapQuestsIndex][1][2])
					table.remove(qcMapQuests, qcMapQuestsIndex)
				end
			end
		end
	end

	--[[ Faction filtering - regardless of user filtering options ]]--
	do
		local qcCurrentPlayerFaction, _S = UnitFactionGroup("player")
		local qcCurrentFaction = qcFactionBits[string.upper(qcCurrentPlayerFaction)]
		for qcMapQuestsIndex = #qcMapQuests, 1, -1 do
			if (bit.band(qcMapQuests[qcMapQuestsIndex][4][1], qcCurrentFaction) == 0) then
				--print("Removing opposing faction quest: " .. qcMapQuests[qcMapQuestsIndex][1][2])
				table.remove(qcMapQuests, qcMapQuestsIndex)
			end
		end
	end

	--[[ Class\Race filtering - regardless of user filtering options ]]--
	do
		local _S, qcCurrentPlayerClass = UnitClass("player")
		local qcCurrentClass = qcClassBits[string.upper(qcCurrentPlayerClass)]
		local _S, qcCurrentPlayerRace = UnitRace("player")
		local qcCurrentRace = qcRaceBits[string.upper(qcCurrentPlayerRace)]
		for qcMapQuestsIndex = #qcMapQuests, 1, -1 do
			if (bit.band(qcMapQuests[qcMapQuestsIndex][4][3], qcCurrentClass) == 0) then
				--print("Removing other class quest.")
				table.remove(qcMapQuests, qcMapQuestsIndex)
			elseif (bit.band(qcMapQuests[qcMapQuestsIndex][4][2], qcCurrentRace) == 0) then
				--print("Removing other races quest.")
				table.remove(qcMapQuests, qcMapQuestsIndex)
			end
		end
	end

	--[[ Add pins to the map ]]--
	do
		for qcMapQuestsIndex = #qcMapQuests, 1, -1 do
			--print("Creating pin to show incomplete quest: " .. qcMapQuests[qcMapQuestsIndex][1][2])
			qcShowPin(qcMapQuests[qcMapQuestsIndex][3][4], qcMapQuests[qcMapQuestsIndex][3][5], qcMapQuests[qcMapQuestsIndex][1][5], qcMapQuests[qcMapQuestsIndex][1][1], qcMapQuests[qcMapQuestsIndex][1][2])
		end
	end

end

--[[ ##### MAP ICONS START ##### ]]--

local function qcColouredQuestName(qcQuestID)

	if not (qcQuestID) then return nil end
	if (qcQuestDatabase[qcQuestID] == nil) then return nil end

	--[[ Order by most likely first, always leaving completed until the end ]]--
	if (qcQuestDatabase[qcQuestID][6] == 3) then
		return tostring("|cff178ed5" .. qcQuestDatabase[qcQuestID][2])
	elseif (qcQuestDatabase[qcQuestID][6] == 2) then
		return tostring("|cff178ed5" .. qcQuestDatabase[qcQuestID][2])
	elseif (qcCompletedQuests[qcQuestID] == nil) then
		return tostring("|cffffffff" .. qcQuestDatabase[qcQuestID][2])
	elseif (qcCompletedQuests[qcQuestID]["C"] == 1) then
		return tostring("|cff00ff00" .. qcQuestDatabase[qcQuestID][2])
	end

end

local function qcPrepareIcon(qcIconType)

	local qcNewIcon = nil
	local qcOpenBracket = "["
	local qcCloseBracket = "]"

	if (#qcSpareMapIcons > 0) then
		qcNewIcon = qcSpareMapIcons[1]
		table.remove(qcSpareMapIcons, 1)
	end

	if not (qcNewIcon) then
		qcNewIcon = CreateFrame("Frame", "qcNewIcon", WorldMapDetailFrame)
		qcNewIcon.Texture = qcNewIcon:CreateTexture()

		--[[ Tooltip Hooks ]]--
		qcNewIcon:HookScript("OnEnter", function(self, motion)
		local tableinsert = table.insert
		local qcFrames = {}
		local qcFrame = EnumerateFrames()
		while qcFrame do
			if (qcFrame:IsVisible() and MouseIsOver(qcFrame) and (qcFrame:GetName() == "qcNewIcon")) then
				tableinsert(qcFrames, qcFrame)
			end
			qcFrame = EnumerateFrames(qcFrame)
		end
		qcMapTooltip:SetOwner(self, "ANCHOR_RIGHT")
		qcMapTooltip:ClearLines()
		for qcFrameIndex, qcFrameEntry in pairs(qcFrames) do
			if (qcCurrentMapInitiators[qcFrames[qcFrameIndex].InitiatorsIndex][3] == 0) then
				qcMapTooltip:AddLine(qcCurrentMapInitiators[qcFrames[qcFrameIndex].InitiatorsIndex][4] or tostring(UnitName("player") .. " |cff69ccf0<Yourself>"))
			else
				qcMapTooltip:AddDoubleLine(qcCurrentMapInitiators[qcFrames[qcFrameIndex].InitiatorsIndex][4] or tostring(UnitName("player") .. " |cff69ccf0<Yourself>"), COLOUR_DRUID .. qcOpenBracket .. tostring(qcCurrentMapInitiators[qcFrames[qcFrameIndex].InitiatorsIndex][3]) .. qcCloseBracket)
			end
			for qcIndex, qcEntry in ipairs(qcCurrentMapInitiators[qcFrames[qcFrameIndex].InitiatorsIndex][7]) do
				qcMapTooltip:AddDoubleLine(tostring(qcColouredQuestName(qcEntry)), COLOUR_DRUID .. qcOpenBracket .. tostring(qcEntry) .. qcCloseBracket)
				if (#qcCurrentMapInitiators[qcFrames[qcFrameIndex].InitiatorsIndex][7] <= 10) then
					--[[ Order by most likely first, always leaving completed until the end ]]--
					--[[ TODO: Create a texture object and use in-game texture with SetTexCoord ]]--
					if not (qcQuestDatabase[qcEntry] == nil) then
						if (qcQuestDatabase[qcEntry][6] == 3) then
							qcMapTooltip:AddTexture("Interface\\Addons\\QuestCompletist\\Images\\DailyQuestIcon")
						elseif (qcQuestDatabase[qcEntry][6] == 2) then
							qcMapTooltip:AddTexture("Interface\\Addons\\QuestCompletist\\Images\\DailyActiveQuestIcon")
						elseif (qcCompletedQuests[qcEntry] ~= nil) and (qcCompletedQuests[qcEntry]["C"] == 1 or qcCompletedQuests[qcEntry]["C"] == 2) then
							qcMapTooltip:AddTexture("Interface\\Addons\\QuestCompletist\\Images\\QuestCompleteIcon")
						else
							qcMapTooltip:AddTexture("Interface\\Addons\\QuestCompletist\\Images\\AvailableQuestIcon")
						end
					end
				end
			end
			if (qcCurrentMapInitiators[qcFrames[qcFrameIndex].InitiatorsIndex][8]) then
				qcMapTooltip:AddLine(tostring(COLOUR_HUNTER .. qcCurrentMapInitiators[qcFrames[qcFrameIndex].InitiatorsIndex][8]),nil,nil,nil,true)
			end
		end
		qcMapTooltip:Show()
		end)
		qcNewIcon:HookScript("OnLeave", function(self)
		qcMapTooltip:Hide()
		end)

		if (qcIconType == 3) then --[[ Daily ]]--
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.41015625, 0.4609375, 0.2578125, 0.484375)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(7)
			qcNewIcon:SetHeight(15)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 2) then --[[ Repeatable ]]--
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.500, 0.625, 0.250, 0.500)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(16)
			qcNewIcon:SetHeight(16)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 5) then --[[ Seasonal ]]--
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.66015625, 0.7109375, 0.2578125, 0.484375)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(7)
			qcNewIcon:SetHeight(15)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 6) then --[[ Profession ]]--
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.13671875,0.2421875,0.7734375,0.9765625)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(14)
			qcNewIcon:SetHeight(13)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 7) then --[[ Item Drop Standard ]]--
			qcNewIcon.Texture:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\DropQuestIcon")
			qcNewIcon.Texture:SetTexCoord(0,1,0,1)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(16)
			qcNewIcon:SetHeight(16)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 8) then --[[ Item Drop Repeatable ]]--
			qcNewIcon.Texture:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\DropQuestIconRepeatable")
			qcNewIcon.Texture:SetTexCoord(0,1,0,1)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(16)
			qcNewIcon:SetHeight(16)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 9) then --[[ Class ]]--
			qcNewIcon.Texture:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\ClassQuestIcon")
			qcNewIcon.Texture:SetTexCoord(0,1,0,1)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(16)
			qcNewIcon:SetHeight(16)
			qcNewIcon:EnableMouse(true)
		else
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.16015625, 0.2109375, 0.2578125, 0.484375)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(7)
			qcNewIcon:SetHeight(15)
			qcNewIcon:EnableMouse(true)
		end
		table.insert(qcCurrentMapIcons, qcNewIcon)
	else
		if (qcIconType == 3) then --[[ Daily ]]--
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.41015625, 0.4609375, 0.2578125, 0.484375)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(7)
			qcNewIcon:SetHeight(15)
		elseif (qcIconType == 2) then --[[ Repeatable ]]--
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.500, 0.625, 0.250, 0.500)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(16)
			qcNewIcon:SetHeight(16)
		elseif (qcIconType == 5) then --[[ Seasonal ]]--
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.66015625, 0.7109375, 0.2578125, 0.484375)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(7)
			qcNewIcon:SetHeight(15)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 6) then --[[ Profession ]]--
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.13671875,0.2421875,0.7734375,0.9765625)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(14)
			qcNewIcon:SetHeight(13)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 7) then --[[ Item Drop ]]--
			qcNewIcon.Texture:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\DropQuestIcon")
			qcNewIcon.Texture:SetTexCoord(0,1,0,1)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(16)
			qcNewIcon:SetHeight(16)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 8) then --[[ Item Drop Repeatable ]]--
			qcNewIcon.Texture:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\DropQuestIconRepeatable")
			qcNewIcon.Texture:SetTexCoord(0,1,0,1)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(16)
			qcNewIcon:SetHeight(16)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 9) then --[[ Class ]]--
			qcNewIcon.Texture:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\ClassQuestIcon")
			qcNewIcon.Texture:SetTexCoord(0,1,0,1)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(16)
			qcNewIcon:SetHeight(16)
			qcNewIcon:EnableMouse(true)
		elseif (qcIconType == 10) then --[[ Kill Start ]]--
			qcNewIcon.Texture:SetTexture("Interface\\Addons\\QuestCompletist\\Images\\KillQuestIcon")
			qcNewIcon.Texture:SetTexCoord(0,1,0,1)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(17)
			qcNewIcon:SetHeight(17)
			qcNewIcon:EnableMouse(true)
		else
			qcNewIcon.Texture:SetTexture("Interface\\MINIMAP\\ObjectIcons.blp")
			qcNewIcon.Texture:SetTexCoord(0.16015625, 0.2109375, 0.2578125, 0.484375)
			qcNewIcon.Texture:SetAllPoints()
			qcNewIcon:SetWidth(7)
			qcNewIcon:SetHeight(15)
		end
		table.insert(qcCurrentMapIcons, qcNewIcon)
	end

	return qcNewIcon

end

local function qcShowIcon(qcCurrentMapInitiatorsIndex, qcX, qcY, qcIconType)

	local qcIcon = qcPrepareIcon(qcIconType)

	qcIcon:ClearAllPoints()
	qcIcon:SetFrameLevel(WorldMapPOIFrame:GetFrameLevel()-1)
	qcIcon:SetPoint("CENTER", WorldMapDetailFrame, "TOPLEFT", (qcX/100)*WorldMapDetailFrame:GetWidth(), (-qcY/100)*WorldMapDetailFrame:GetHeight())
	qcIcon.InitiatorsIndex = qcCurrentMapInitiatorsIndex
	qcIcon:Show()

end

local function qcHideIcon(qcIconID, qcEntry)

	if (qcIconID) then
		qcEntry:Hide()
		table.insert(qcSpareMapIcons, qcEntry)
		table.remove(qcCurrentMapIcons, qcIconID)
	end

end

local function qcHideAllMapIcons()

	for qcIndex = #qcCurrentMapIcons, 1, -1 do
		qcHideIcon(qcIndex, qcCurrentMapIcons[qcIndex])
	end

end

local function qcRefreshMapIcons(qcMapID, qcMapLevel)

	local tableremove = table.remove
	local bitband = bit.band

	qcHideAllMapIcons()
	wipe(qcCurrentMapInitiators)

	if (qcSettings["QC_M_SHOW_ICONS"] == 0) then return nil end
	if (qcMapIcon[qcMapID] == nil) then return nil end

	qcCurrentMapInitiators = qcCopyTable(qcMapIcon[qcMapID])

	--[[ Map level ]]--
	for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
		if not (qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][1] == qcMapLevel) then
			tableremove(qcCurrentMapInitiators, qcCurrentMapInitiatorsIndex)
		end
	end

	--[[ Low level quests ]]--
	if (qcSettings["QC_M_HIDE_LOWLEVEL"] == 1) then
		for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
			for qcQuestIndex = #qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], 1, -1 do
				local qcQuestID = qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7][qcQuestIndex]
				if not (qcQuestDatabase[qcQuestID] == nil) then
					local qcQuestLevel = qcQuestDatabase[qcQuestID][3] or 0
					local qcQuestCutoff = (UnitLevel("player") - GetQuestGreenRange())
					if (qcQuestLevel < qcQuestCutoff) then
						tableremove(qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], qcQuestIndex)
					end
				else
					tableremove(qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], qcQuestIndex)
				end
			end
			if (#qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7] == 0) then
				tableremove(qcCurrentMapInitiators, qcCurrentMapInitiatorsIndex)
			end
		end
	end

	--[[ Completed quests ]]--
	if (qcSettings["QC_M_HIDE_COMPLETED"] == 1) then
		for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
			for qcQuestIndex = #qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], 1, -1 do
				local qcQuestID = qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7][qcQuestIndex]
				if (qcCompletedQuests[qcQuestID] ~= nil) and (qcCompletedQuests[qcQuestID]["C"] == 1 or qcCompletedQuests[qcQuestID]["C"] == 2) then
					tableremove(qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], qcQuestIndex)
				end
			end
			if (#qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7] == 0) then
				tableremove(qcCurrentMapInitiators, qcCurrentMapInitiatorsIndex)
			end
		end
	end

	--[[ Faction ]]--
	if (qcSettings["QC_ML_HIDE_FACTION"] == 1) then
		for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
			for qcQuestIndex = #qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], 1, -1 do
				local qcQuestID = qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7][qcQuestIndex]
				local qcCurrentPlayerFaction, _S = UnitFactionGroup("player")
				local qcCurrentFaction = qcFactionBits[string.upper(qcCurrentPlayerFaction)]
				if (qcQuestDatabase[qcQuestID] ~= nil) and (bitband(qcQuestDatabase[qcQuestID][7], qcCurrentFaction) == 0) then
					tableremove(qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], qcQuestIndex)
				end
			end
			if (#qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7] == 0) then
				tableremove(qcCurrentMapInitiators, qcCurrentMapInitiatorsIndex)
			end
		end
	end

	--[[ Race\Class ]]--
	if (qcSettings["QC_ML_HIDE_RACECLASS"] == 1) then
		for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
			for qcQuestIndex = #qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], 1, -1 do
				local qcQuestID = qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7][qcQuestIndex]
				local _S, qcCurrentPlayerRace = UnitRace("player")
				local qcCurrentRace = qcRaceBits[string.upper(qcCurrentPlayerRace)]
				local _S, qcCurrentPlayerClass = UnitClass("player")
				local qcCurrentClass = qcClassBits[string.upper(qcCurrentPlayerClass)]
				if (qcQuestDatabase[qcQuestID] ~= nil) and (bitband(qcQuestDatabase[qcQuestID][8], qcCurrentRace) == 0) then
					tableremove(qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], qcQuestIndex)
				elseif (qcQuestDatabase[qcQuestID] ~= nil) and (bitband(qcQuestDatabase[qcQuestID][9], qcCurrentClass) == 0) then
					tableremove(qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], qcQuestIndex)
				end
			end
			if (#qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7] == 0) then
				tableremove(qcCurrentMapInitiators, qcCurrentMapInitiatorsIndex)
			end
		end
	end

	--[[ Seasonal ]]--
	if (qcSettings["QC_M_HIDE_SEASONAL"] == 1) then
		local qcToday = date("%y%m%d")
		for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
			for qcQuestIndex = #qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], 1, -1 do
				local qcQuestID = qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7][qcQuestIndex]
				if (qcQuestDatabase[qcQuestID] ~= nil) and (qcQuestDatabase[qcQuestID][11] > 0) then
					if not ((qcToday >= qcHolidayDates[qcQuestDatabase[qcQuestID][11]][1]) and (qcToday <= qcHolidayDates[qcQuestDatabase[qcQuestID][11]][2])) then
						tableremove(qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], qcQuestIndex)
					end
				end
			end
			if (#qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7] == 0) then
				tableremove(qcCurrentMapInitiators, qcCurrentMapInitiatorsIndex)
			end
		end
	end

	--[[ Professions ]]--
	if (qcSettings["QC_M_HIDE_PROFESSION"] == 1) then
		local qcProfessionBitwise = 0
		local qcProfessions = {GetProfessions()}
		for qcIndex, qcEntry in pairs(qcProfessions) do
			local qcName, qcTexture, _S, _S, _S, _S, qcProfessionID, _S = GetProfessionInfo(qcEntry)
			qcProfessionBitwise = (qcProfessionBitwise + qcProfessionBits[qcProfessionID])
		end
		for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
			for qcQuestIndex = #qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], 1, -1 do
				local qcQuestID = qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7][qcQuestIndex]
				if (qcQuestDatabase[qcQuestID] ~= nil) and (qcQuestDatabase[qcQuestID][10] > 0) then
					if (bitband(qcQuestDatabase[qcQuestID][10], qcProfessionBitwise) == 0) then
						tableremove(qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], qcQuestIndex)
					end
				end
			end
			if (#qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7] == 0) then
				tableremove(qcCurrentMapInitiators, qcCurrentMapInitiatorsIndex)
			end
		end
	end

	--[[ In progress ]]--
	if (qcSettings["QC_M_HIDE_INPROGRESS"] == 1) then
		for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
			for qcQuestIndex = #qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], 1, -1 do
				local qcQuestID = qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7][qcQuestIndex]
				if (GetQuestLogIndexByID(qcQuestID) ~= 0) then
					tableremove(qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7], qcQuestIndex)
				end
			end
			if (#qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7] == 0) then
				tableremove(qcCurrentMapInitiators, qcCurrentMapInitiatorsIndex)
			end
		end
	end

	--[[ Empty quest sub-tables ]]--
	for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
		if (#qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][7] == 0) then
			tableremove(qcCurrentMapInitiators, qcCurrentMapInitiatorsIndex)
		end
	end

	--[[ Add pins to the map ]]--
	for qcCurrentMapInitiatorsIndex = #qcCurrentMapInitiators, 1, -1 do
		qcShowIcon(qcCurrentMapInitiatorsIndex, qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][5], qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][6], qcCurrentMapInitiators[qcCurrentMapInitiatorsIndex][2])
	end

end

--[[ ##### MAP ICONS END ##### ]]--

--[[ ##### INTERFACE OPTIONS START ##### ]]--

function qcInterfaceOptions_Okay(self)

	qcUpdateQuestList(qcCurrentCategoryID, 1)
	qcRefreshMapIcons(GetCurrentMapAreaID(), GetCurrentMapDungeonLevel())

end

function qcInterfaceOptions_Cancel(self)

end

function qcInterfaceOptions_OnShow(self)

	-- CreateFrame("frameType" [, "name" [, parent [, "template"]]])

    qcConfigTitle = self:CreateFontString("qcConfigTitle", "ARTWORK", "GameFontNormalLarge")
    qcConfigTitle:SetPoint("TOPLEFT", 16, -16)
    qcConfigTitle:SetText("Quest Completist - Configuration")

    qcConfigSubtitle = self:CreateFontString("qcConfigSubtitle", "ARTWORK", "GameFontHighlightSmall")
    qcConfigSubtitle:SetHeight(22)
    qcConfigSubtitle:SetPoint("TOPLEFT", qcConfigTitle, "BOTTOMLEFT", 0, -8)
    qcConfigSubtitle:SetPoint("RIGHT", self, -32, 0)
    qcConfigSubtitle:SetNonSpaceWrap(true)
    qcConfigSubtitle:SetJustifyH("LEFT")
    qcConfigSubtitle:SetJustifyV("TOP")
    qcConfigSubtitle:SetText("These options allow you to change the way that Quest Completist handles and filters its data.")

    qcMapFiltersTitle = self:CreateFontString("qcMapFiltersTitle", "ARTWORK", "GameFontNormal")
    qcMapFiltersTitle:SetPoint("TOPLEFT", qcConfigSubtitle, "BOTTOMLEFT", 16, -4)
    qcMapFiltersTitle:SetText("Map Filters")

	qcIO_M_SHOW_ICONS = CreateFrame("CheckButton", "qcIO_M_SHOW_ICONS", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_M_SHOW_ICONS:SetPoint("TOPLEFT", qcMapFiltersTitle, "BOTTOMLEFT", 16, -6)
	_G[qcIO_M_SHOW_ICONS:GetName().."Text"]:SetText("Show Map Icons")
	qcIO_M_SHOW_ICONS:SetScript("OnClick", function(self)
		if (qcIO_M_SHOW_ICONS:GetChecked() == nil) then
			qcSettings["QC_M_SHOW_ICONS"] = 0
		else
			qcSettings["QC_M_SHOW_ICONS"] = 1
		end
	end)

	qcIO_M_HIDE_COMPLETED = CreateFrame("CheckButton", "qcIO_M_HIDE_COMPLETED", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_M_HIDE_COMPLETED:SetPoint("TOPLEFT", qcIO_M_SHOW_ICONS, "BOTTOMLEFT", 0, 0)
	_G[qcIO_M_HIDE_COMPLETED:GetName().."Text"]:SetText("Hide Completed Quests")
	qcIO_M_HIDE_COMPLETED:SetScript("OnClick", function(self)
		if (qcIO_M_HIDE_COMPLETED:GetChecked() == nil) then
			qcSettings["QC_M_HIDE_COMPLETED"] = 0
		else
			qcSettings["QC_M_HIDE_COMPLETED"] = 1
		end
	end)

	qcIO_M_HIDE_LOWLEVEL = CreateFrame("CheckButton", "qcIO_M_HIDE_LOWLEVEL", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_M_HIDE_LOWLEVEL:SetPoint("TOPLEFT", qcIO_M_HIDE_COMPLETED, "BOTTOMLEFT", 0, 0)
	_G[qcIO_M_HIDE_LOWLEVEL:GetName().."Text"]:SetText("Hide Low Level Quests")
	qcIO_M_HIDE_LOWLEVEL:SetScript("OnClick", function(self)
		if (qcIO_M_HIDE_LOWLEVEL:GetChecked() == nil) then
			qcSettings["QC_M_HIDE_LOWLEVEL"] = 0
		else
			qcSettings["QC_M_HIDE_LOWLEVEL"] = 1
		end
	end)

	qcIO_M_HIDE_PROFESSION = CreateFrame("CheckButton", "qcIO_M_HIDE_PROFESSION", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_M_HIDE_PROFESSION:SetPoint("TOPLEFT", qcIO_M_HIDE_LOWLEVEL, "BOTTOMLEFT", 0, 0)
	_G[qcIO_M_HIDE_PROFESSION:GetName().."Text"]:SetText("Hide Other Profession Quests")
	qcIO_M_HIDE_PROFESSION:SetScript("OnClick", function(self)
		if (qcIO_M_HIDE_PROFESSION:GetChecked() == nil) then
			qcSettings["QC_M_HIDE_PROFESSION"] = 0
		else
			qcSettings["QC_M_HIDE_PROFESSION"] = 1
		end
	end)

	qcIO_M_HIDE_SEASONAL = CreateFrame("CheckButton", "qcIO_M_HIDE_SEASONAL", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_M_HIDE_SEASONAL:SetPoint("TOPLEFT", qcIO_M_HIDE_PROFESSION, "BOTTOMLEFT", 0, 0)
	_G[qcIO_M_HIDE_SEASONAL:GetName().."Text"]:SetText("Hide Non-Active Seasonal Quests")
	qcIO_M_HIDE_SEASONAL:SetScript("OnClick", function(self)
		if (qcIO_M_HIDE_SEASONAL:GetChecked() == nil) then
			qcSettings["QC_M_HIDE_SEASONAL"] = 0
		else
			qcSettings["QC_M_HIDE_SEASONAL"] = 1
		end
	end)

	qcIO_M_HIDE_INPROGRESS = CreateFrame("CheckButton", "qcIO_M_HIDE_INPROGRESS", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_M_HIDE_INPROGRESS:SetPoint("TOPLEFT", qcIO_M_HIDE_SEASONAL, "BOTTOMLEFT", 0, 0)
	_G[qcIO_M_HIDE_INPROGRESS:GetName().."Text"]:SetText("Hide In Progress Quests")
	qcIO_M_HIDE_INPROGRESS:SetScript("OnClick", function(self)
		if (qcIO_M_HIDE_INPROGRESS:GetChecked() == nil) then
			qcSettings["QC_M_HIDE_INPROGRESS"] = 0
		else
			qcSettings["QC_M_HIDE_INPROGRESS"] = 1
		end
	end)

    qcListFiltersTitle = self:CreateFontString("qcListFiltersTitle", "ARTWORK", "GameFontNormal")
    qcListFiltersTitle:SetPoint("TOPLEFT", qcConfigSubtitle, "BOTTOMLEFT", 16, -185)
    qcListFiltersTitle:SetText("Quest List Filters")

	qcIO_L_HIDE_COMPLETED = CreateFrame("CheckButton", "qcIO_L_HIDE_COMPLETED", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_L_HIDE_COMPLETED:SetPoint("TOPLEFT", qcListFiltersTitle, "BOTTOMLEFT", 16, -6)
	_G[qcIO_L_HIDE_COMPLETED:GetName().."Text"]:SetText("Hide Completed Quests")
	qcIO_L_HIDE_COMPLETED:SetScript("OnClick", function(self)
		if (qcIO_L_HIDE_COMPLETED:GetChecked() == nil) then
			qcSettings["QC_L_HIDE_COMPLETED"] = 0
		else
			qcSettings["QC_L_HIDE_COMPLETED"] = 1
		end
	end)

	qcIO_L_HIDE_LOWLEVEL = CreateFrame("CheckButton", "qcIO_L_HIDE_LOWLEVEL", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_L_HIDE_LOWLEVEL:SetPoint("TOPLEFT", qcIO_L_HIDE_COMPLETED, "BOTTOMLEFT", 0, 0)
	_G[qcIO_L_HIDE_LOWLEVEL:GetName().."Text"]:SetText("Hide Low Level Quests " .. COLOUR_DEATHKNIGHT .. "(Not Yet Implemented)")
	qcIO_L_HIDE_LOWLEVEL:SetScript("OnClick", function(self)
		if (qcIO_L_HIDE_LOWLEVEL:GetChecked() == nil) then
			qcSettings["QC_L_HIDE_LOWLEVEL"] = 0
		else
			qcSettings["QC_L_HIDE_LOWLEVEL"] = 1
		end
	end)

	qcIO_L_HIDE_PROFESSION = CreateFrame("CheckButton", "qcIO_L_HIDE_PROFESSION", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_L_HIDE_PROFESSION:SetPoint("TOPLEFT", qcIO_L_HIDE_LOWLEVEL, "BOTTOMLEFT", 0, 0)
	_G[qcIO_L_HIDE_PROFESSION:GetName().."Text"]:SetText("Hide Other Profession Quests " .. COLOUR_DEATHKNIGHT .. "(Not Yet Implemented)")
	qcIO_L_HIDE_PROFESSION:SetScript("OnClick", function(self)
		if (qcIO_L_HIDE_PROFESSION:GetChecked() == nil) then
			qcSettings["QC_L_HIDE_PROFESSION"] = 0
		else
			qcSettings["QC_L_HIDE_PROFESSION"] = 1
		end
	end)

    qcCombinedFiltersTitle = self:CreateFontString("qcCombinedFiltersTitle", "ARTWORK", "GameFontNormal")
    qcCombinedFiltersTitle:SetPoint("TOPLEFT", qcConfigSubtitle, "BOTTOMLEFT", 16, -290)
    qcCombinedFiltersTitle:SetText("Combined Map & Quest List Filters")

	qcIO_ML_HIDE_FACTION = CreateFrame("CheckButton", "qcIO_ML_HIDE_FACTION", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_ML_HIDE_FACTION:SetPoint("TOPLEFT", qcCombinedFiltersTitle, "BOTTOMLEFT", 16, -6)
	_G[qcIO_ML_HIDE_FACTION:GetName().."Text"]:SetText("Hide Other Faction Quests")
	qcIO_ML_HIDE_FACTION:SetScript("OnClick", function(self)
		if (qcIO_ML_HIDE_FACTION:GetChecked() == nil) then
			qcSettings["QC_ML_HIDE_FACTION"] = 0
		else
			qcSettings["QC_ML_HIDE_FACTION"] = 1
		end
	end)

	qcIO_ML_HIDE_RACECLASS = CreateFrame("CheckButton", "qcIO_ML_HIDE_RACECLASS", self, "InterfaceOptionsCheckButtonTemplate")
    qcIO_ML_HIDE_RACECLASS:SetPoint("TOPLEFT", qcIO_ML_HIDE_FACTION, "BOTTOMLEFT", 0, 0)
	_G[qcIO_ML_HIDE_RACECLASS:GetName().."Text"]:SetText("Hide Other Race & Class Quests")
	qcIO_ML_HIDE_RACECLASS:SetScript("OnClick", function(self)
		if (qcIO_ML_HIDE_RACECLASS:GetChecked() == nil) then
			qcSettings["QC_ML_HIDE_RACECLASS"] = 0
		else
			qcSettings["QC_ML_HIDE_RACECLASS"] = 1
		end
	end)

    self:SetScript("OnShow", qcConfigRefresh) 
    qcConfigRefresh(self)

end

function qcConfigRefresh(self)
	if not (self:IsVisible()) then return end
	--[[ Set control values here ]]--
end

function qcInterfaceOptions_OnLoad(self)

	self.name = "Quest Completist"
	self.okay = function(self) qcInterfaceOptions_Okay(self) end
	self.cancel = function(self) qcInterfaceOptions_Cancel(self) end
	InterfaceOptions_AddCategory(self)

end

--[[ ##### INTERFACE OPTIONS END ##### ]]--

local function qcCheckSettings()

	if (qcSettings == nil) then
		qcSettings = {}
	end

	if (qcSettings["SORT"] == nil) then --[[ 1:Level, 2:Alpha, 3:Quest Giver ]]--
		qcSettings["SORT"] = 1
	end
	if (qcSettings["PURGED"] == nil) then
		qcSettings["PURGED"] = 0
	end

	--[[ Interface Options ]]--

	if (qcSettings["QC_M_SHOW_ICONS"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_M_SHOW_ICONS"] = 1
	end
	if (qcSettings["QC_M_HIDE_COMPLETED"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_M_HIDE_COMPLETED"] = 0
	end
	if (qcSettings["QC_M_HIDE_LOWLEVEL"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_M_HIDE_LOWLEVEL"] = 0
	end
	if (qcSettings["QC_M_HIDE_PROFESSION"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_M_HIDE_PROFESSION"] = 1
	end
	if (qcSettings["QC_M_HIDE_SEASONAL"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_M_HIDE_SEASONAL"] = 1
	end
	if (qcSettings["QC_M_HIDE_INPROGRESS"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_M_HIDE_INPROGRESS"] = 0
	end
	if (qcSettings["QC_L_HIDE_COMPLETED"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_L_HIDE_COMPLETED"] = 0
	end
	if (qcSettings["QC_L_HIDE_LOWLEVEL"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_L_HIDE_LOWLEVEL"] = 0
	end
	if (qcSettings["QC_L_HIDE_PROFESSION"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_L_HIDE_PROFESSION"] = 1
	end
	if (qcSettings["QC_ML_HIDE_FACTION"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_ML_HIDE_FACTION"] = 1
	end
	if (qcSettings["QC_ML_HIDE_RACECLASS"] == nil) then --[[ 0:No, 1:Yes ]]--
		qcSettings["QC_ML_HIDE_RACECLASS"] = 1
	end

	--[[ Interface Options End ]]--

end

local function qcApplySettings()

	--[[ Interface Options ]]--

	if (qcSettings["QC_M_SHOW_ICONS"] == 0) then
		qcIO_M_SHOW_ICONS:SetChecked(false)
	else
		qcIO_M_SHOW_ICONS:SetChecked(true)
	end
	if (qcSettings["QC_M_HIDE_COMPLETED"] == 0) then
		qcIO_M_HIDE_COMPLETED:SetChecked(false)
	else
		qcIO_M_HIDE_COMPLETED:SetChecked(true)
	end
	if (qcSettings["QC_M_HIDE_LOWLEVEL"] == 0) then
		qcIO_M_HIDE_LOWLEVEL:SetChecked(false)
	else
		qcIO_M_HIDE_LOWLEVEL:SetChecked(true)
	end
	if (qcSettings["QC_M_HIDE_PROFESSION"] == 0) then
		qcIO_M_HIDE_PROFESSION:SetChecked(false)
	else
		qcIO_M_HIDE_PROFESSION:SetChecked(true)
	end
	if (qcSettings["QC_M_HIDE_SEASONAL"] == 0) then
		qcIO_M_HIDE_SEASONAL:SetChecked(false)
	else
		qcIO_M_HIDE_SEASONAL:SetChecked(true)
	end
	if (qcSettings["QC_M_HIDE_INPROGRESS"] == 0) then
		qcIO_M_HIDE_INPROGRESS:SetChecked(false)
	else
		qcIO_M_HIDE_INPROGRESS:SetChecked(true)
	end
	if (qcSettings["QC_L_HIDE_COMPLETED"] == 0) then
		qcIO_L_HIDE_COMPLETED:SetChecked(false)
	else
		qcIO_L_HIDE_COMPLETED:SetChecked(true)
	end
	if (qcSettings["QC_L_HIDE_LOWLEVEL"] == 0) then
		qcIO_L_HIDE_LOWLEVEL:SetChecked(false)
	else
		qcIO_L_HIDE_LOWLEVEL:SetChecked(true)
	end
	if (qcSettings["QC_L_HIDE_PROFESSION"] == 0) then
		qcIO_L_HIDE_PROFESSION:SetChecked(false)
	else
		qcIO_L_HIDE_PROFESSION:SetChecked(true)
	end
	if (qcSettings["QC_ML_HIDE_FACTION"] == 0) then
		qcIO_ML_HIDE_FACTION:SetChecked(false)
	else
		qcIO_ML_HIDE_FACTION:SetChecked(true)
	end
	if (qcSettings["QC_ML_HIDE_RACECLASS"] == 0) then
		qcIO_ML_HIDE_RACECLASS:SetChecked(false)
	else
		qcIO_ML_HIDE_RACECLASS:SetChecked(true)
	end

	--[[ Interface Options End ]]--

	if (QCADDON_PURGE == true) then
		if not (qcSettings["PURGED"] == QCADDON_VERSION) then
			qcPurgeCollectedCache()
			qcSettings["PURGED"] = QCADDON_VERSION
		end
	end

end

local function qcEventHandler(self, event, ...)

	if (event == "WORLD_MAP_UPDATE") then

		qcRefreshMapIcons(GetCurrentMapAreaID(), GetCurrentMapDungeonLevel())
		qcRefreshPins(GetCurrentMapAreaID())

	elseif (event == "UNIT_QUEST_LOG_CHANGED") then

		if (... == "player") then qcUpdateQuestList(nil, qcMenuSlider:GetValue()) end

	elseif (event == "ZONE_CHANGED_NEW_AREA") then

		qcZoneChangedNewArea()

	elseif (event == "QUEST_ITEM_UPDATE") then

		if (QuestFrame:IsShown()) then QuestFrameNpcNameText:SetText(UnitName("questnpc") .. " [" .. GetQuestID() .. "]") end

	elseif (event == "QUEST_DETAIL") then

		local qcQuestID = GetQuestID()
		if (QuestFrame:IsShown()) then QuestFrameNpcNameText:SetText(tostring(UnitName("questnpc") or nil) .. " [" .. qcQuestID .. "]") end
		qcBreadcrumbChecks(qcQuestID)
		qcNewDataChecks(qcQuestID)
		qcMutuallyExclusiveChecks(qcQuestID)

	elseif (event == "QUEST_ACCEPTED") then

		qcCollectQuestOnAccept(...)
		qcUpdateQuestList(nil, qcMenuSlider:GetValue())

	elseif (event == "QUEST_PROGRESS") then

		local qcQuestID = GetQuestID()
		if (QuestFrame:IsShown()) then QuestFrameNpcNameText:SetText(UnitName("questnpc") .. " [" .. qcQuestID .. "]") end
		if not (qcQuestID == 0) then
			qcCollectQuestOnProgressComplete(qcQuestID)
			qcBreadcrumbChecks(qcQuestID)
			qcNewDataChecks(qcQuestID)
			qcMutuallyExclusiveChecks(qcQuestID)
		end

	elseif (event == "QUEST_COMPLETE") then

		local qcQuestID = GetQuestID()
		if (QuestFrame:IsShown()) then QuestFrameNpcNameText:SetText(UnitName("questnpc") .. " [" .. qcQuestID .. "]") end
		if not (qcQuestID == 0) then
			qcCollectQuestOnProgressComplete(qcQuestID)
			qcBreadcrumbChecks(qcQuestID)
			qcNewDataChecks(qcQuestID)
			qcMutuallyExclusiveChecks(qcQuestID)
			qcUpdateCompletedQuest(qcQuestID)
			qcUpdateMutuallyExclusiveCompletedQuest(qcQuestID)
			qcUpdateSkippedBreadcrumbQuest(qcQuestID)
			qcUpdateQuestList(nil, qcMenuSlider:GetValue())
		end

	elseif (event == "QUEST_QUERY_COMPLETE") then

		qcQuestQueryCompleted()
		qcQuestCompletistUI:UnregisterEvent("QUEST_QUERY_COMPLETE")

	elseif (event == "PLAYER_ENTERING_WORLD") then

		qcZoneChangedNewArea()

	elseif (event == "ADDON_LOADED") then

		if (... == "QuestCompletist") then
			qcCheckSettings()
			qcApplySettings()
			qcCollectUpdate()
		end

	end

end

function qcQuestCompletistUI_OnLoad(self)

	SetPortraitToTexture(self.qcPortrait, "Interface\\ICONS\\TRADE_ARCHAEOLOGY_DRAENEI_TOME")
	self.qcTitleText:SetText("Quest Completist v" .. QCADDON_VERSION)
	self.qcCategoryDropdownButton:SetText(GetText("CATEGORIES"))
	self.qcOptionsButton:SetText(GetText("FILTERS"))
	self:RegisterForDrag("LeftButton")

	self:RegisterEvent("QUEST_COMPLETE")
	self:RegisterEvent("QUEST_DETAIL")
	self:RegisterEvent("QUEST_PROGRESS")
	self:RegisterEvent("QUEST_ACCEPTED")
	self:RegisterEvent("QUEST_ITEM_UPDATE")
	self:RegisterEvent("UNIT_QUEST_LOG_CHANGED")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("WORLD_MAP_UPDATE")
	self:SetScript("OnEvent", qcEventHandler)

	qcQuestInformationTooltipSetup()
	qcQuestReputationTooltipSetup()
	qcMapTooltipSetup()
	qcToastTooltipSetup()
	qcNewDataAlertTooltipSetup()
	qcMutuallyExclusiveAlertTooltipSetup()

end