local GetCVar = GetCVar
local SetCVar = SetCVar
local IsConsumableAction = IsConsumableAction
local IsStackableAction = IsStackableAction
local GetActionCount = GetActionCount
local GetActionInfo = GetActionInfo
local GetMacroItem = GetMacroItem
local GetActionText = GetActionText

local tooltip = CreateFrame("GameTooltip", "ThereIsNoCostTooltip", nil, "GameTooltipTemplate")

local CHARGE = ITEM_SPELL_CHARGES:gsub("%%d%s+|4(.+):.*;", "^%%d+ %1$") -- Charge
local CHARGES = ITEM_SPELL_CHARGES:gsub("%%d%s+|4.+:(.*);", "^%%d+ %1$") -- Charges


hooksecurefunc("ActionButton_UpdateCount", function(self, event)
	local text = _G[self:GetName().."Count"]
	if text then
		local action = self.action
		local isConsumableAction = IsConsumableAction(action)
		local isStackableAction = IsStackableAction(action)
		
		-- checks to prevent massive performance drop due to all the :SetAction()s
		if isConsumableAction or isStackableAction or (text:GetText() and #text:GetText() > 0) then
			local hasReagent, hasCharges

			-- need to enable detailed tooltips to be able to parse all the information
			local oldCVar = GetCVar("UberTooltips")
			SetCVar("UberTooltips", "1")
			
			tooltip:SetOwner(UIParent, "ANCHOR_NONE")
			tooltip:SetAction(action)
			
			-- scan tooltip for reagent req or charges
			for line = 1, tooltip:NumLines() do
				local lineText = _G["ThereIsNoCostTooltipTextLeft"..line]:GetText()
				if lineText then
					if lineText:match("^"..SPELL_REAGENTS) then
						hasReagent = true
						break
					elseif lineText:match(CHARGE) or lineText:match(CHARGES) then
						hasCharges = true
						break
					end
				end
			end
			
			SetCVar("UberTooltips", oldCVar)
			tooltip:Hide()
			
			local actionType, id = GetActionInfo(action)
			local macroName = _G[self:GetName().."Name"]
			
			if (isConsumableAction and (actionType == "item" or GetMacroItem(id))) or isStackableAction or hasReagent or hasCharges then
				text:SetText(GetActionCount(action))
				macroName:SetText("")
			else
				text:SetText("")
				-- show macro text instead, if applicable
				macroName:SetText(GetActionText(action))
			end
		end
	end
end)


hooksecurefunc("ActionButton_OnEvent", function(self, event, ...)
	if not self.glyphChecked and event == "PLAYER_ENTERING_WORLD" then
		-- we want to register this event for all action buttons so they can be updated immediately after adding a glyph
		self:RegisterEvent("GLYPH_ADDED")
		self.glyphChecked = true
	elseif event == "ACTIONBAR_UPDATE_USABLE" or event == "GLYPH_ADDED" then
		ActionButton_UpdateCount(self, event)
	end
end)