-- Initialise

local ZLI_VERSION = "1.4.8";

local L,zones,zoneNumberTable,continentNumberTable;	--arrays
local currentLocale;					--strings
local currentLocaleSupported,missingZones;		--booleans

local zoneDefault=" ";

local notActive = true;

local lastX,lastY = 100;
local lastTime = 0;

local savedZoneName = "";
local savedMapText = "";

local savedScale = 0;
--local indentX = 75;   --was 60
--local indentY = 50;  --was -40, now negative at use

ZoneLevelInfoSettings = {};
local ZoneLevelInfoDefaultsVersion = 3;
local ZoneLevelInfoDefaults = {
		 ["fontLarge"]		= 22
		,["fontOutline"]	= true
		,["showAtZoneZoom"]	= true
		,["scale"]		= 100
		,["indentX"]		= 75
		,["indentY"]		= 50
	};

local ZoneLevelInfo_Frame = CreateFrame("frame");
ZoneLevelInfo_Frame:RegisterEvent("ADDON_LOADED");



-------------------------------
-- OnUpdate
function ZoneLevelInfo_OnUpdate(self, elapsed)

	-------------------------------
	-- OnUpdate functions

	local function ZoneLevelInfo_LevelColour(maxLevelZone, minLevelZone) 

		local playerLevel = UnitLevel("player");
		local levelNull = maxLevelZone + ceil(GetQuestGreenRange()/2) + 1;
		local levelVeryHard = minLevelZone - 3;
		--if (playerLevel >= 85 and maxLevelZone <= 85) then    --New blizzard changes to force you to move on to more appropriate content
		--	levelNull = 85;
		--elseif (playerLevel >= 80 and maxLevelZone <= 80) then
		--	levelNull = 80;
		--elseif (playerLevel >= 70 and maxLevelZone <= 70) then
		--	levelNull = 70;
		--elseif (playerLevel >= 60 and maxLevelZone <= 60) then
		--	levelNull = 60;
		--end
		local levelHard = minLevelZone;
		local levelLow = maxLevelZone;
		local colour = {["R"]=1, ["G"]=1, ["B"]=1};

		if ( minLevelZone <= 0 ) then	-- capital city / unknown area
			colour = {["R"]=0.5, ["G"]=0.5, ["B"]=0.5};
		elseif ( playerLevel >= levelNull ) then	-- grey, little/no xp
			colour = {["R"]=0.5, ["G"]=0.5, ["B"]=0.5};
		elseif ( playerLevel <= levelVeryHard ) then	-- solid red, hard & probably no quests.
			colour = {["R"]=1, ["G"]=0.2, ["B"]=0};
		elseif (  ( playerLevel > levelHard and playerLevel < levelLow )
		        or( (levelLow-levelHard)<2 and (playerLevel==levelLow or playerLevel==levelHard) ) ) then -- yellow, the 'sweetspot'
			colour = {["R"]=1, ["G"]=1, ["B"]=0.2};
		elseif ( playerLevel >= levelLow and playerLevel < levelNull ) then -- green, on the easy side
			colour = {["R"]=0, ["G"]=1, ["B"]=0.2};
		elseif ( playerLevel > levelVeryHard and playerLevel <= levelHard ) then -- orange, tough going
			colour = {["R"]=1, ["G"]=0.6, ["B"]=0.2};
		end

		return colour;
	end


	-------------------------------
	-- OnUpdate main procedure

	if (notActive) then return; end

	local x, y = GetCursorPosition();
	local theTime = GetTime();
	local mapText = WorldMapFrameAreaLabel:GetText();
	local scale = WorldMapDetailFrame:GetEffectiveScale();
	if (lastX==x and lastY==y and mapText==savedMapText and scale==savedScale and (lastTime+0.5)>theTime) then return; else lastX = x; lastY = y; lastTime=theTime; end
	local currentContinentIndex = GetCurrentMapContinent();
	local zoneName;
	if (currentContinentIndex==0) then
		if (mapText) then
			if (zones[mapText..zoneDefault]) then
				zoneName = mapText..zoneDefault;
			elseif (zones[mapText]) then
				zoneName = mapText;
			end
		end
		x = 0;
		y = 0;
	elseif (zones[mapText]) then
		zoneName = mapText;
		x = 0;
		y = 0;
	else
		x = x / scale;
		y = y / scale;
		local width = WorldMapDetailFrame:GetWidth();
		local height = WorldMapDetailFrame:GetHeight();
		local centerX, centerY = WorldMapDetailFrame:GetCenter();
		local topleftX = centerX - (width/2);
		local topleftY = centerY + (height/2);
		x = (x - topleftX) / width;
		y = (topleftY - y) / height;
		zoneName, _, _, _, _, _, _, _ = UpdateMapHighlight(x, y);
	end

	if (zoneName and not zones[zoneName]) or (x<0 or x>1 or y<0 or y>1) then
		zoneName = nil;
	end
	
	if (zoneName == nil) then
		local currentZoneIndex = GetCurrentMapZone();
		if ( currentZoneIndex > 0 ) then
			if ( currentContinentIndex > 0 and ZoneLevelInfoSettings.showAtZoneZoom ) then
				zoneName = zoneNumberTable[currentContinentIndex][currentZoneIndex];
			end
		else
			if ( currentContinentIndex > 0 ) then
				zoneName = continentNumberTable[currentContinentIndex]..zoneDefault;
			end
		end
	end

	if (scale ~= savedScale) then
		savedScale = scale;
		ZoneLevelInfoFrame_MainText:SetPoint("TOPLEFT","WorldMapDetailFrame","TOPLEFT",floor(ZoneLevelInfoSettings.indentX*(savedScale+0.3)), floor(-ZoneLevelInfoSettings.indentY*(savedScale+0.3)));
	end

	if (zoneName == savedZoneName and mapText == savedMapText) then
		return
	else
		savedZoneName = zoneName;
		savedMapText  = mapText;

		if (savedZoneName == nil) then
			ZoneLevelInfoFrame_MainText:SetTextColor(1, 0.75, 0.15);
			ZoneLevelInfoFrame_MainText:SetText(savedMapText);
			ZoneLevelInfoFrame_Key:SetText("");
			ZoneLevelInfoFrame_ExtraText:SetText("");
			ZoneLevelInfoFrame_LevelRange:SetText("");
		else
			local zoneInfo = zones[savedZoneName];
			local controlInfo = "";
			local setColour;
			if (zoneInfo[3] == "N") then
				controlInfo = L["Contested Zone"];
				setColour = {["R"]=1, ["G"]=1, ["B"]=0.2};

			elseif (zoneInfo[3] == "H") then
				controlInfo = L["Horde"];
				setColour = {["R"]=1, ["G"]=0.2, ["B"]=0};

			elseif (zoneInfo[3] == "A") then
				controlInfo = L["Alliance"];
				setColour = {["R"]=0, ["G"]=0.5, ["B"]=1};

			elseif (zoneInfo[3] == "X") then
				controlInfo = L["PvP Combat Zone"];
				setColour = {["R"]=1, ["G"]=0.5, ["B"]=0};

			elseif (zoneInfo[3] == "S") then
				controlInfo = L["Sanctuary"];
				setColour = {["R"]=0.2, ["G"]=0.8, ["B"]=0};

			elseif (zoneInfo[3] == "C") then
				setColour = {["R"]=1, ["G"]=0.75, ["B"]=0.15};

			else -- should not be any 'else'
				setColour = {["R"]=1, ["G"]=1, ["B"]=1};
			end

			ZoneLevelInfoFrame_MainText:SetTextColor(setColour["R"], setColour["G"], setColour["B"]);
			ZoneLevelInfoFrame_Key:SetTextColor(setColour["R"], setColour["G"], setColour["B"]);
			ZoneLevelInfoFrame_ExtraText:SetTextColor(setColour["R"], setColour["G"], setColour["B"]);

			ZoneLevelInfoFrame_MainText:SetText(savedZoneName);
			ZoneLevelInfoFrame_Key:SetText(controlInfo);

			if (savedMapText ~= nil and savedMapText ~= "BLAH!" and currentContinentIndex > 0 and not zones[savedMapText]) then
				ZoneLevelInfoFrame_ExtraText:SetText(savedMapText);
			elseif (zoneInfo[3]~="C" and not zones[savedZoneName]) then
				ZoneLevelInfoFrame_ExtraText:SetText(savedZoneName);
			else
				ZoneLevelInfoFrame_ExtraText:SetText("");
			end

			setColour = ZoneLevelInfo_LevelColour(0,0);
			local levelRange = "";
			if (zoneInfo == zoneDefault) then
				levelRange = "["..L["unknown zone"].."]";

			elseif (zoneInfo[1] < 0 and zoneInfo[2] < 0) then
				levelRange = L["Capital City"];

			elseif (zoneInfo[1] == 0 and zoneInfo[2] == 0 and zoneInfo[3] ~= "C") then
				--levelRange = L["Staging Area"];
				levelRange = ZoneLevelInfoFrame_ExtraText:GetText();
				ZoneLevelInfoFrame_ExtraText:SetText("");
				setColour = {["R"]=1, ["G"]=1, ["B"]=0.2};

			elseif (zoneInfo[1] > 0 and zoneInfo[2] > 0) then
				if (zoneInfo[1] < zoneInfo[2]) then
					levelRange = "["..zoneInfo[1].."-"..zoneInfo[2].."]";
					setColour = ZoneLevelInfo_LevelColour(zoneInfo[2], zoneInfo[1]);
				else
					levelRange = "["..zoneInfo[1].."]";
					setColour = ZoneLevelInfo_LevelColour(zoneInfo[1], zoneInfo[1]);
				end
			end
			if (zoneInfo["Note"]) then
				if (levelRange ~= "") then
					levelRange = levelRange.." ";
				end;
				levelRange = levelRange..L[zoneInfo["Note"]];
			end

			ZoneLevelInfoFrame_LevelRange:SetTextColor(setColour["R"], setColour["G"], setColour["B"]);
			ZoneLevelInfoFrame_LevelRange:SetText(levelRange);
		end
	end
end





local function ZoneLevelInfo_SetFonts(fontLarge,fontOutlineBool)

		local minSize = ceil((6+1)/0.75);
		local maxSize = 24;

		if (fontLarge>maxSize) then fontLarge=maxSize elseif (fontLarge<minSize) then fontLarge=minSize end;
		local fontSmall = floor(fontLarge*0.75)-1;
		local fontSpacing = fontLarge;
		if (fontLarge > 18) then
			fontSpacing = fontSpacing + fontLarge - 18;
		elseif (fontLarge < 14) then
			fontSpacing = 14;
		end

		local theFont,_ = GameFontHighlight:GetFont();
		local fontOutline = "";
		if (fontOutlineBool) then fontOutline = "OUTLINE"; end
		
		ZoneLevelInfoFrame_MainText:SetFont(theFont,fontLarge,fontOutline);
		ZoneLevelInfoFrame_Key:SetFont(theFont,fontSmall,fontOutline);
		ZoneLevelInfoFrame_ExtraText:SetFont(theFont,fontSmall,fontOutline);
		ZoneLevelInfoFrame_LevelRange:SetFont(theFont,fontSmall,fontOutline);
		ZoneLevelInfoFrame_Key:SetPoint("BOTTOMLEFT", "ZoneLevelInfoFrame_MainText", "BOTTOMLEFT", 0, -fontSpacing);
		ZoneLevelInfoFrame_LevelRange:SetPoint("BOTTOMLEFT", "ZoneLevelInfoFrame_Key", "BOTTOMLEFT", 0, -fontSpacing);
		ZoneLevelInfoFrame_ExtraText:SetPoint("BOTTOMLEFT", "ZoneLevelInfoFrame_LevelRange", "BOTTOMLEFT", 0, -fontSpacing);

end


--============================
--= Register the Slash Command
--============================
SlashCmdList["ZLI"] = function(_msg)
	if (_msg) then
		local _, _, cmd, arg1 = string.find(string.upper(_msg), "([%w]+)%s*(.*)$");
		if ("DEFAULT" == cmd or "RESET" == cmd) then		-- reset the list
			for ZLI_default_key,ZLI_default_value in next,ZoneLevelInfoDefaults do
				ZoneLevelInfoSettings[ZLI_default_key] = ZLI_default_value;
			end
			ZoneLevelInfo_SetFonts(ZoneLevelInfoSettings.fontLarge,ZoneLevelInfoSettings.fontOutline);
			ZoneLevelInfoFrame:SetScale(ZoneLevelInfoSettings.scale/100);
			ZoneLevelInfoFrame_MainText:SetPoint("TOPLEFT","WorldMapDetailFrame","TOPLEFT",floor(ZoneLevelInfoSettings.indentX*(savedScale+0.3)), floor(-ZoneLevelInfoSettings.indentY*(savedScale+0.3)));
			WorldMapFrameAreaLabel:Hide();
			ZoneLevelInfoFrame:Show();
			notActive=false;
			local confirmMsg = "ZoneLevelInfo: defaults reset";
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00".. confirmMsg);

		elseif ("SIZE" == cmd) then		-- set the text size
			local confirmMsg = "ZoneLevelInfo size ";
			local setSize = arg1:match("^%s*(%d+)%s*$");
			if (setSize ~= nil and tonumber(setSize)>=10 and tonumber(setSize)<=24) then
				ZoneLevelInfoSettings.fontLarge = floor(tonumber(setSize));
				ZoneLevelInfo_SetFonts(ZoneLevelInfoSettings.fontLarge,ZoneLevelInfoSettings.fontOutline);
				confirmMsg = confirmMsg .. "set to " ..ZoneLevelInfoSettings.fontLarge;
			else
				confirmMsg = confirmMsg .. "currently " ..ZoneLevelInfoSettings.fontLarge..". Settable range: 10-24.";
				if (arg1 ~= "") then
					confirmMsg = confirmMsg .. " (Option unknown: '"..arg1.."')";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00" .. confirmMsg);

		elseif ("OUTLINE" == cmd) then
			local confirmMsg = "ZoneLevelInfo map text outline switched ";
			if (ZoneLevelInfoSettings.fontOutline) then
				ZoneLevelInfoSettings.fontOutline = false;
				confirmMsg = confirmMsg .. "off.";
			else
				ZoneLevelInfoSettings.fontOutline = true;
				confirmMsg = confirmMsg .. "on.";
			end
			ZoneLevelInfo_SetFonts(ZoneLevelInfoSettings.fontLarge,ZoneLevelInfoSettings.fontOutline);
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00" .. confirmMsg);

		elseif ("ZONE" == cmd) then
			local confirmMsg = "ZoneLevelInfo details at zone zoom-level switched ";
			if (ZoneLevelInfoSettings.showAtZoneZoom) then
				ZoneLevelInfoSettings.showAtZoneZoom = false;
				confirmMsg = confirmMsg .. "off.";
			else
				ZoneLevelInfoSettings.showAtZoneZoom = true;
				confirmMsg = confirmMsg .. "on.";
			end
			ZoneLevelInfo_SetFonts(ZoneLevelInfoSettings.fontLarge,ZoneLevelInfoSettings.fontOutline);
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00" .. confirmMsg);

		elseif ("TOGGLE" == cmd) then
			local confirmMsg = "ZoneLevelInfo switched ";
			if (notActive) then
				WorldMapFrameAreaLabel:Hide();
				ZoneLevelInfoFrame:Show();
				notActive=false;
				confirmMsg = confirmMsg .. "ON";
			else
				WorldMapFrameAreaLabel:Show();
				ZoneLevelInfoFrame:Hide();
				notActive=true;
				confirmMsg = confirmMsg .. "OFF";
			end
			confirmMsg = confirmMsg .. ". This setting is not saved between sessions.";
			ZoneLevelInfo_SetFonts(ZoneLevelInfoSettings.fontLarge,ZoneLevelInfoSettings.fontOutline);
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00" .. confirmMsg);

		elseif ("SCALE" == cmd) then		-- set the frame scale (relative to world map frame)
			local confirmMsg = "ZoneLevelInfo scale ";
			local setScale = arg1:match("^%s*(%d+)%s*$");
			if (setScale ~= nil and tonumber(setScale)>=20 and tonumber(setScale)<=500) then
				ZoneLevelInfoSettings.scale = floor(tonumber(setScale));
				ZoneLevelInfoFrame:SetScale(ZoneLevelInfoSettings.scale/100);
				confirmMsg = confirmMsg .. "set to " ..ZoneLevelInfoSettings.scale.."%.";
			else
				confirmMsg = confirmMsg .. "currently " ..ZoneLevelInfoSettings.scale.."%. Settable range: 20-500.";
				if (arg1 ~= "") then
					confirmMsg = confirmMsg .. " (Option unknown: '"..arg1.."')";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00" .. confirmMsg);

		elseif ("INDENTX" == cmd) then		-- set the X indent
			local confirmMsg = "ZoneLevelInfo indentX ";
			local setIndent = arg1:match("^%s*(%d+)%s*$");
			if (setIndent ~= nil and tonumber(setIndent)>=0 and tonumber(setIndent)<=10000) then
				ZoneLevelInfoSettings.indentX = floor(tonumber(setIndent));
				ZoneLevelInfoFrame_MainText:SetPoint("TOPLEFT","WorldMapDetailFrame","TOPLEFT",floor(ZoneLevelInfoSettings.indentX*(savedScale+0.3)), floor(-ZoneLevelInfoSettings.indentY*(savedScale+0.3)));
				confirmMsg = confirmMsg .. "set to " ..ZoneLevelInfoSettings.indentX..".";
			else
				confirmMsg = confirmMsg .. "currently " ..ZoneLevelInfoSettings.indentX..". Settable range: 0-10000.";
				if (arg1 ~= "") then
					confirmMsg = confirmMsg .. " (Option unknown: '"..arg1.."')";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00" .. confirmMsg);

		elseif ("INDENTY" == cmd) then		-- set the Y indent
			local confirmMsg = "ZoneLevelInfo indentY ";
			local setIndent = arg1:match("^%s*(%d+)%s*$");
			if (setIndent ~= nil and tonumber(setIndent)>=0 and tonumber(setIndent)<=10000) then
				ZoneLevelInfoSettings.indentY = floor(tonumber(setIndent));
				ZoneLevelInfoFrame_MainText:SetPoint("TOPLEFT","WorldMapDetailFrame","TOPLEFT",floor(ZoneLevelInfoSettings.indentX*(savedScale+0.3)), floor(-ZoneLevelInfoSettings.indentY*(savedScale+0.3)));
				confirmMsg = confirmMsg .. "set to " ..ZoneLevelInfoSettings.indentY..".";
			else
				confirmMsg = confirmMsg .. "currently " ..ZoneLevelInfoSettings.indentY..". Settable range: 0-10000.";
				if (arg1 ~= "") then
					confirmMsg = confirmMsg .. " (Option unknown: '"..arg1.."')";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00" .. confirmMsg);

		else -- either a gap command or an incorrect option
			local badCmd = "";
			if (_msg ~= "" and _msg ~= "?" and _msg ~= "HELP") then badCmd = " Unknown command: '" .. _msg .. "'"; end;

			local outlineSetting="OFF";
			if (ZoneLevelInfoSettings.fontOutline) then outlineSetting="ON"; end
			local showAtZoneZoom="OFF";
			if (ZoneLevelInfoSettings.showAtZoneZoom) then showAtZoneZoom="ON"; end
			local zliShow="ON";
			if (notActive) then zliShow="OFF"; end

			local helpMsg = "/zonelevelinfo or /zli " .. badCmd .. "\n" ..
				"/zli default: resets options (size:22; outline:ON; zone:ON; toggle:ON).\n" ..
				"/zli size n: sets zone text size to 'n', where 'n' is 10-24. ["..ZoneLevelInfoSettings.fontLarge.."]\n" ..
				"/zli outline: toggles text outline on/off. ["..outlineSetting.."]\n" ..
				"/zli zone: toggles ZoneLevelInfo details on zone zoom-level. ["..showAtZoneZoom.."]\n" ..
				"/zli toggle: toggles ZoneLevelInfo on/off for the current session ["..zliShow.."]";
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00" .. helpMsg);
		end
	end
end

SLASH_ZLI1 = "/zonelevelinfo";
SLASH_ZLI2 = "/zli";	



--============================
--= Loaded
--============================

local function ZoneLevelInfo_OnEvent(self, event, arg1, ...)
	if ( event == "ADDON_LOADED" and arg1:lower() == "zonelevelinfo" ) then

		currentLocale,currentLocaleSupported,L,zones,zoneNumberTable,continentNumberTable,missingZones = ZoneLevelInfo_init(zoneDefault);

		local defaultReset = false;
		if (ZoneLevelInfoDefaultsVersion ~= ZoneLevelInfoSettings["defaultVersion"]) then
			defaultReset = true;
			ZoneLevelInfoSettings = {};
			ZoneLevelInfoSettings["defaultVersion"] = ZoneLevelInfoDefaultsVersion;
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00ZoneLevelInfo defaults are being reset (new version).");
		end
		for ZLI_default_key,ZLI_default_value in next,ZoneLevelInfoDefaults do
			if (ZoneLevelInfoSettings[ZLI_default_key] == nil or defaultReset) then
				ZoneLevelInfoSettings[ZLI_default_key] = ZLI_default_value;
			end
		end

		ZoneLevelInfo_SetFonts(ZoneLevelInfoSettings.fontLarge,ZoneLevelInfoSettings.fontOutline);
		ZoneLevelInfoFrame:SetScale(ZoneLevelInfoSettings.scale/100);

		local okMessage = "OK";
		if (missingZones) then
			okMessage = "with errors, see above..";
		end

		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00ZoneLevelInfo "..ZLI_VERSION.." loaded for ["..currentLocale.."] "..okMessage..". For options use /zli");

		WorldMapFrameAreaLabel:Hide();
		notActive=false;

	end
end
ZoneLevelInfo_Frame:SetScript("OnEvent", ZoneLevelInfo_OnEvent);