﻿local addon, ns = ...

local Module, ModuleList, Event, Tags, Task = {}, {}, {}, {}, {};


local BAGSTRINGS = {
	["backpack"]			= { 0 };
	["bags"]				= { 1, 2, 3, 4 };
	["backpack+bags"]		= { 0, 1, 2, 3, 4, };
	["bank"]				= { 5, 6, 7, 8, 9, 10, 11 };
	["bankframe"]			= { -1 };
	["bankframe+bank"]		= { -1, 5, 6, 7, 8, 9, 10, 11 };
	["keyring"]				= { -2 };
};

local LOCALES = { 
	["deDE"] 				= true; 
	["enGB"] 				= true; 
	["enUS"] 				= true; 
	["esES"] 				= true;
	["esMX"] 				= true;
	["frFR"] 				= true; 
	["koKR"] 				= true; 
	["ruRU"] 				= true; 
	["zhCN"] 				= true; 
	["zhTW"] 				= true; 
};

local RESERVEDEVENTS = { 
	ADDON_LOADED 			= true; 
	PLAYER_LOGIN 			= true; 
	PLAYER_LOGOUT 			= true;
	PLAYER_ENTERING_WORLD 	= true;
};

local GARBAGECOLLECTEVENTS = {
	ADDON_LOADED 			= true; 
	PLAYER_LOGIN 			= true; 
};

local _G = _G;
local abs, date, error, floor, format, gsub, mod, pairs, setmetatable, sort, strlen, strsub, strupper, tinsert, tremove, tonumber, tostring, type, unpack = abs, date, error, floor, format, gsub, mod, pairs, setmetatable, sort, strlen, strsub, strupper, tinsert, tremove, tonumber, tostring, type, unpack;

noop = function() return end


------------------------------------------------------------------------------------------------------------
-- 	Main Frame
------------------------------------------------------------------------------------------------------------

local FRAME = CreateFrame( "Frame" );

for event, reg in pairs( RESERVEDEVENTS ) do
	if reg then 
		FRAME:RegisterEvent( event );
	end
end

-- credits to Lonny and Paul Emmerich for inspiring this
local function OnUpdate( self, elapsed )
	for addon, tasks in pairs( Task ) do
		for i,v in pairs( tasks ) do
			v.elapsed = ( v.elapsed or 0 ) + elapsed;

			if ( v.elapsed >= v.interval ) then
				v.func( Module[addon] );
				v.elapsed = v.elapsed - v.interval;
			end
		end
	end
end
local function OnEvent( self, event, ... )
	if RESERVEDEVENTS[event] then
		FRAME[event]( self, event, ... );
		return
	end
	
	if Event[event] then
		for addon, func in pairs( Event[event] ) do
			if ( type( func ) == "string" ) then
				if Module[addon][event] then
					Module[addon][event]( Module[addon], ... );
				end
				
			elseif ( type( func ) == "function" ) then
				func( Module[addon], event, ... );
			end
			
		end
	end
end

function FRAME:ADDON_LOADED( event, name )
	if ( name == addon ) then
		for name, addon in pairs( Module ) do
			addon:OnInit();
			addon.OnInit = nil;
		end
	end
end
function FRAME:PLAYER_LOGIN( self, event )
	for name, addon in pairs( Module ) do
		addon:OnEnable();
		addon.OnEnable = nil;
	end
end
function FRAME:PLAYER_LOGOUT( self, event )
	for name, addon in pairs( Module ) do
		addon:OnDisable();
	end
end
function FRAME:PLAYER_ENTERING_WORLD( self, event )
	for name, addon in pairs( Module ) do
		addon:OnEnter();
	end
end

FRAME:SetScript( "OnUpdate", OnUpdate );
FRAME:SetScript( "OnEvent", OnEvent );


------------------------------------------------------------------------------------------------------------
-- 	Template Module
------------------------------------------------------------------------------------------------------------
local MODULE = {
	ADDON 		= addon;
	TITLE 		= GetAddOnMetadata( addon, "Title" ) or "";
	VERSION 	= GetAddOnMetadata( addon, "Version" ) or "";
	REALM 		= ( GetRealmName() );
	CHARNAME 	= ( select( 1, UnitName( "player" ) ) );
	FACTION 	= ( UnitFactionGroup( "player" ) ); 
};

function MODULE:OnInit() return end
function MODULE:OnEnable() return end
function MODULE:OnDisable() return end
function MODULE:OnEnter() return end

function MODULE:RegisterEvent( event, func )
	--[[
		usage:	myModule:RegisterEvent( event[, func] )
				- Same as the normal API RegisterEvent
				- Registers the event with ns.lib's handler
	]]--
	if ( type( event ) ~= "string") or ( event == "" ) then return end
	if RESERVEDEVENTS[event] then return end

	if not Event[event] then
		Event[event] = {};
		FRAME:RegisterEvent( event );
	end
	
	Event[event][self:GetName()] = func or event
end
function MODULE:RegisterEvents( events )
	--[[ 
				- Registers multiple events at once
				- If the value is a function, then the index must be the eventname
				- If the value is a string, the value will be the event name, and the myModule:EVENTNAME() will be called when the event fire
	]]--
	if ( type( events ) ~= "table" ) then return end
	
	for i,v in pairs( events ) do
		if ( type(v) == "string" ) then
			self:RegisterEvent(v);
		elseif type(v) == "function" then
			self:RegisterEvent(i, v);
		end
	end
end
function MODULE:UnregisterEvent( event )
	if ( type( event ) ~= "string" ) or ( event == "" ) then return end
	if RESERVEDEVENTS[event] and not dying then return end
	
	if Event[event][self] then
		Event[event][self] = nil
	end
	if #Event[event] == 0 then
		Event[event] = nil;
		FRAME:UnregisterEvent( event );
	end
end
function MODULE:Schedule( task, interval )
	--[[
		usage: 	myModule:Schedule( task[, interval] )
				- registers the function <task> to be called every <interval> seconds
				- interval must be between 0.1s and 999s, or it will default to 0.5s
	]]--
	if ( type( task ) ~= "function" ) then return end
	
	Task[self:GetName()] = Task[self:GetName()] or {};
	
	local thetask = {
		func = task;
		interval = min( max( ( ( type( interval ) == "number" ) and abs( interval ) or 0.5 ), 0.1 ), 999 );
	};
	
	tinsert( Task[self:GetName()], thetask );
	
end
function MODULE:GetName()
	return tostring( self.NAME ) or tostring( ns.lib.NAME ) or "nil";
end
function MODULE:Kill()
	self.dying = true;

	for event, status in pairs( RESERVEDEVENTS ) do
		self:UnregisterEvent( event );
	end
	
	for event, addons in pairs( Event ) do
		for addon, func in pairs( addons ) do
			if ( addon == self:GetName() ) then
				Event[event][addon] = nil;
				self:UnregisterEvent( event );
			end
		end
	end

	Task[self:GetName()] = nil;
	Module[self:GetName()] = nil;
	
	for i, module in pairs( ModuleList ) do
		if ( module == self:GetName() ) then
			tremove( ModuleList, i );
		end
	end
	
	for i, v in pairs( self ) do
		self[i] = nil;
	end

	self = nil;
end
function MODULE:Slash( name, func, slashes )
	if not( name ) or type( name )~= "string" then 
		name = ( type( slashes ) == "table" ) and slashes[1] or ( type( slashes ) == "string" ) and slashes or  "";
		elseif ( type( slashes ) == "string" ) then 
	end
	func = ( ( type(func) == "function" ) and func ) or ( function() end )

	if self and self:GetName() then 
		name = self:GetName()..name;
	end
	
	if ( type( slashes ) == "table" ) then 
		for i,v in pairs( slashes ) do
			_G["SlashCmdList"][name] = function( argString ) 
				func( unpack( split( argString:lower(), " " ) ) );
			end;
			_G["SLASH_"..name..i] = "/"..v;
		end
	elseif ( type( slashes ) == "string" ) then 
		_G["SlashCmdList"][name] = function( argString ) 
			func( unpack( split( argString:lower(), " " ) ) );
		end;
		_G["SLASH_"..name.."1"] = "/"..slashes;
	end
end


------------------------------------------------------------------------------------------------------------
-- 	Extended Frame- And Texture Handling
------------------------------------------------------------------------------------------------------------
local function SetSize( obj, w, h )
	if type( w ) ~= "number" then return end
	if not( tonumber( h ) ) then h = w; end

	obj:SetWidth( w );
	obj:SetHeight( h );
end
local function Place( obj, arg1, arg2, arg3, arg4, arg5 )
	obj:ClearAllPoints();
	obj:SetPoint( arg1, arg2, arg3, arg4, arg5 );
end
local function Kill( obj )
	if not obj then return end

	if ( obj:GetObjectType() == "Frame" ) then
		obj:UnregisterAllEvents();
		obj:EnableMouse( false );
		
	elseif ( obj:GetObjectType() == "Texture" ) then
		obj:SetTexture( "" );
		obj.SetTexture = noop;
	end
	
	if obj.SetPoint then 
		obj:SetSize( 0.01 );
		obj:Place( "BOTTOMLEFT", "UIParent", "BOTTOMLEFT", -1, -1 );

		obj.SetSize = noop;
		obj.Place = noop;


		obj.SetPoint = noop;
		obj.SetAllPoints = noop;
	end

	if obj.Show then
		obj:Hide();
		obj.Show = noop;
	end
end

-- 4.0.6, hiding the MainMenuBar causes the sidebars to disappear after /rl in secondary specs,
-- if you're a warrior or priest. 
local function Shoo( obj )
	if not obj then return end

	if ( obj:GetObjectType() == "Frame" ) then
		obj:UnregisterAllEvents();
		obj:EnableMouse( false );
		
	elseif ( obj:GetObjectType() == "Texture" ) then
		obj:SetTexture( "" );
		obj.SetTexture = noop;
	end
	
	if obj.SetPoint then 
		obj:SetSize( 0.01 );
		obj:Place( "BOTTOMLEFT", "UIParent", "BOTTOMLEFT", -1, -1 );

		obj.Size = noop;
		obj.Place = noop;


		obj.SetPoint = noop;
		obj.SetAllPoints = noop;
	end

	if obj.Show then
		obj:Show();
		obj.Hide = noop;
		obj.Show = noop;
		obj:SetAlpha( 0 );
	end
end

local function Shrink( obj )
	if not obj.Size then return end
	obj:SetSize( 0.01 );
end

local function Release( obj )
	if not obj or not obj.GetName then return end
	local n = obj:GetName()
	if n and UIPARENT_MANAGED_FRAME_POSITIONS[n] then
		UIPARENT_MANAGED_FRAME_POSITIONS[n] = nil;
	end
end

-- GetPoint is not the best of functions. Here's our own version;
local function GetAllPoints( obj )
	if not obj or not obj.GetPoint then return end

	local w, h 		= GetScreenWidth(), GetScreenHeight();
	
	local left 		= obj:GetLeft();
	local right 	= w - obj:GetRight();
	local top 		= h - obj:GetTop();
	local bottom 	= obj:GetBottom();
	
	local horizontal, vertical, x, y 

	if ( top > bottom ) then
		vertical = "BOTTOM";
		y = bottom;
	else
		vertical = "TOP";
		y = -top;
	end
	
	if ( left > right ) then
		horizontal = "RIGHT";
		x = -right;
	else
		horizontal = "LEFT";
		x = left;
	end
	
	return { vertical..horizontal, "UIParent", vertical..horizontal, x, y }
end

do
	local function setAPI( frame )
		local meta = getmetatable( frame ).__index;

		meta.SetSize 		= SetSize;
		meta.Place 			= Place;
		meta.Kill 			= Kill;
		meta.Shoo 			= Shoo;
		meta.Shrink 		= Shrink;
		meta.Release 		= Release;
		meta.GetAllPoints 	= GetAllPoints;
	end

	setAPI( FRAME );
	setAPI( FRAME:CreateTexture() );
	setAPI( FRAME:CreateFontString() );

	local done = {
		["Frame"] = true;
	};

	local frames = EnumerateFrames();
	while frames do
		if not done[frames:GetObjectType()] then
			setAPI( frames );
			done[frames:GetObjectType()] = true;
		end

		frames = EnumerateFrames( frames );
	end
	
	done = nil;
end

------------------------------------------------------------------------------------------------------------
-- 	Local functions
------------------------------------------------------------------------------------------------------------
local function CreateIcon( iconPath, iconValues )
	if not iconPath or type( iconPath ) ~= "string" then return end
	iconValues = ( type( iconValues ) == "string" and iconValues ) or "0:0:0:0";

	if( type( iconValues ) == "table" ) then
		iconValues = table.concat( iconValues, ":" );
	end
	return ("|T%s:%s|t"):format( iconPath, iconValues )
end

local function Crop( str, maxlength )
	if not(str) or not( type( str ) == "string" ) or ( string.len( str ) == 0 ) then return end
	if maxlength and type( maxlength ) ~= "number" then maxlength = 4; end
	
	maxlength = ( maxlength < 4 ) and 4 or maxlength;

	if ( strlen( str ) <= maxlength ) then return str end

	return strsub( str, 1, maxlength-3 ).."..."
end

local function Cvars( cvars )
	for i,v in pairs( cvars ) do
		SetCVar( i, v );
	end
end

local function Embed( target, source )
	-- embeds all the values and methods from source into target
	if not target or not source then return end
	if type( source ) ~= "table" or type( target ) ~= "table" then return end 

	for i,v in pairs( source ) do
		target[i] = v;
	end
end

local function GetBagSpace(bags)
	if not bags or type( bags ) ~= "string" or not BAGSTRINGS[bags] then return end

	local free, total, used = 0, 0, 0;
	for _,i in pairs( BAGSTRINGS[bags] ) do
		free, total, used = free + GetContainerNumFreeSlots(i), total + GetContainerNumSlots(i), total - free;
	end
	
	return free, total, used, floor( free / total * 100 )
end

local function GetChoice ( choice )
	return ( ( ( choice == 1 ) or ( choice == true ) ) and 1 ) or ( not( choice ) or ( choice == 0 ) ) and 0 or 0
end

local function GetTrueChoice( choice )
	
	return ( ( ( choice == true ) or ( choice == 1 ) ) and true ) or ( ( not( choice ) or ( choice == 0 ) ) and false ) or false
end

local function RegisterTag( tag, func )
	if ( type( tag ) ~= "string" ) then return end
	if ( type( func ) ~= "function" ) then return end
	if Tags[tag] then return end
	
	Tags[tag] = func;
end

-- our theory here is that our function is better than the built-in
local function split( str, separator )
	if type( str ) ~= "string" or type( separator ) ~= "string" then return end
	--
	-- 	usage: 	{substr1, substr2,...} = string.split(str, separator) or str:split(separator)
	--
	local t = {}  
	local fpat = "(.-)" .. separator
	local last_end = 1
	local s, e, cap = str:find(fpat, 1)
	while s do
		if s ~= 1 or cap ~= "" then
			tinsert(t,cap)
		end
		last_end = e+1
		s, e, cap = str:find(fpat, last_end)
	end
	if last_end <= #str then
		cap = str:sub(last_end)
		tinsert(t, cap)
	end
	return t
end

local function SortByKey( tbl )
	-- create a sorted array of the keys in a given table

	if not tbl or type( tbl ) ~= "table" then return end

	-- create a new empty table, and insert all the indexes 
	-- from our original table as keys into the new table
	local ntbl = {};
	for i, v in pairs( tbl ) do
		tinsert( ntbl, i );
	end
	
	-- sort our new table
	sort( ntbl );
	
	-- return our sorted table of indexes;
	-- 		you can iterate through the returned table from ntbl[1] to ntbl[#ntbl]
	-- 		the keys will be sorted index names of tbl; The first one will be tbl[ntbl[1]]
	return ntbl
end

local function Lower( str )
	-- return only the lowercase characters from a given string
	if not str or type( str ) ~= "string" then return end

	-- usual syntax checks	
	if not(str) then return end
	if type(str) ~= "string" then return end
	if strlen(str) == 0 then return end
	
	-- return the string stripped of uppercase characters
	return ( gsub(str, "[%u]", "") )
end
local function Upper( str )
	-- return only the uppercase characters from a given string

	-- usual syntax checks	
	if not( str ) then return end
	if type( str ) ~= "string" then return end
	if strlen( str ) == 0 then return end
	
	-- return the string stripped of lowercase characters
	return (gsub(str, "[%l]", ""))
end

-- the following functions depend on one or more of the above. do not change the order.
local function Validate( user, defaults ) 
	-- compare the types of the user input to the default settings, 
	-- and fill in the holes with the given defaults

	if not( defaults ) or type( defaults ) ~= "table" then return end

	if ( not(user) ) or ( type(user) ~= "table" ) then 
		user = {}; 
	end

	for i,v in pairs( defaults ) do
		if ( user[i] == nil ) or ( type( user[i] ) ~= type( defaults[i] ) ) then
			if ( type( defaults[i] ) == "table" ) then
				user[i] = CopyTable( defaults[i] );
			else
				user[i] = defaults[i];
			end
		elseif ( type( user[i] ) == "table" ) then
			user[i] = Validate( user[i], defaults[i] );	
		end
	end
	
	return user
end
local function Tag( str  )
	--[[
		usage: 	string.tag("[tagname:arg1,arg2,arg3...] some text [tagname:arg1,arg2,arg3...]") or str:tag()
			credits to cargor for the inspiration to this
	]]--
	if not( str ) or ( type( str ) ~= "string" ) then return end

	return ( str:gsub( "%[([^%]:]+):?(.-)%]", function( a, b ) return Tags[a]( unpack( split( b, "[,]+") ) ) end ) )
end

------------------------------------------------------------------------------------------------------------
-- 	API calls
------------------------------------------------------------------------------------------------------------
GL_GetChoice = GetChoice;
GL_GetTrueChoice = GetTrueChoice;
GL_SetCVars = Cvars;
GL_SortByKey = SortByKey;

_G.Embed = Embed;
_G.ValidateTable = Validate;


string.upperonly = function(str) return Upper(str); end
string.tag = function(tagstr) return Tag(tagstr); end


--
-- create a new addon that inherits all methods from MODULE
--
function ns.lib:New( name )
	if not( name ) or type( name ) ~= "string" or name == "" then return end

	Module[name] = ns.lib:newClass( name, MODULE );
	Module[name].NAME = name;

	tinsert( ModuleList, name );

	return Module[name]
end

--
-- create a new instance of the class parentClass
-- that inherits all of parentClass' methods
--
-- 	newClass = ns.lib:newClass( "myClass", yourClass )
-- 	
function ns.lib:newClass( name, parent )

	if not name or type( name ) ~= "string" then return end

    local new = {};
    local meta = { __index = new };

    function new:New()
        local new = {};

        setmetatable( new, meta );

        return new
    end

    if parent then
        setmetatable( new, { __index = parent } );
    end
	
    return new
	
end

-- debugging
-- prints a list of currently handled ns.lib addons/objects
function ns.lib:list()
	for name, addon in pairs( Module ) do
		print( ("%s - v%s - %s"):format(name, addon.VERSION, addon.NAME) );
	end
end

-- returns a list of the addon names
function ns.lib:GetList()
	return ModuleList
end


------------------------------------------------------------------------------------------------------------
-- 	Tags
------------------------------------------------------------------------------------------------------------

Tags["money"] = function( money, full )
	money = money and tonumber(money) or GetMoney();

	local str;
	local g,s,c = floor(money/1e4), floor(money/100) % 100, money % 100;
	local pos = "0:0:-2:0"

	if full then
		if ( g > 0 ) then str = ( str and str.."" or "" ) .. g .. CreateIcon( "Interface\\MoneyFrame\\UI-GoldIcon", pos ) end
		if ( s > 0 ) or ( g > 0 ) then str = ( str and str.."" or "" ) .. s .. CreateIcon( "Interface\\MoneyFrame\\UI-SilverIcon", pos ) end
		str = ( str and str.."" or "" ) .. c .. CreateIcon( "Interface\\MoneyFrame\\UI-CopperIcon", pos );
		return str
	else
		if ( g > 0 ) then str = ( str and str.."" or "" ) .. g .. CreateIcon( "Interface\\MoneyFrame\\UI-GoldIcon", pos ) end
		if ( s > 0 ) then str = ( str and str.."" or "" ) .. s .. CreateIcon( "Interface\\MoneyFrame\\UI-SilverIcon", pos ) end
		if ( c > 0 ) or ( ( g + s + c ) == 0 ) then str = ( str and str.."" or "" ) .. c .. CreateIcon( "Interface\\MoneyFrame\\UI-CopperIcon", pos ) end
		return str
	end
end
Tags["free"] = function( bags )
	if not BAGSTRINGS[bags] then return end
	return ( select( 1, GetBagSpace( bags ) ) )
end
Tags["max"] = function( bags )
	if not BAGSTRINGS[bags] then return end
	return ( select( 2, GetBagSpace( bags ) ) ) 
end
Tags["used"] = function( bags )
	if not BAGSTRINGS[bags] then return end
	return ( select( 3, GetBagSpace( bags ) ) )
end
Tags["freepercent"] = function( bags )
	if not BAGSTRINGS[bags] then return end
	return ( select( 4, GetBagSpace( bags ) ) ).."%"
end
Tags["usedpercent"] = function( bags )
	if not BAGSTRINGS[bags] then return end
	return ( 100 - ( select( 4, GetBagSpace( bags ) ) ) ).."%"
end
Tags["shortvalue"] = function( value )
	if not( value ) then return "0.0"; end
	
	value = tonumber( value );
	
	if value >= 1e6 then
		return ( "%.1fm" ):format( value / 1e6 ):gsub( "%.?0+([km])$", "%1" )
	elseif value >= 1e3 or value <= -1e3 then
		return ( "%.1fk" ):format( value / 1e3 ):gsub( "%.?0+([km])$", "%1" )
	else
		return value
	end	
end
Tags["smarttime"] = function( seconds )
	if not( seconds ) then return "0.0"; end
	
	seconds = tonumber( seconds );
	
	local day, hour, minute = 86400, 3600, 60
	if seconds >= day then
		return format( "%dd", floor( seconds / day + 0.5 ) ), seconds % day
	elseif seconds >= hour then
		return format( "%dh", floor( seconds / hour + 0.5 ) ), seconds % hour
	elseif seconds >= minute then
		if seconds <= minute * 5 then
			return format( "%d:%02d", floor( seconds / 60 ), seconds % minute ), seconds - floor( seconds )
		end
		return format( "%dm", floor( seconds / minute + 0.5 ) ), seconds % minute
	elseif seconds >= minute / 12 then
		return floor( seconds + 0.5 ), ( seconds * 100 - floor( seconds * 100 ) ) / 100
	end
	return format( "%.1f", seconds ), ( seconds * 100 - floor( seconds * 100 ) ) / 100
end
Tags["time"] = function( timezone )
	--[[
		time
			timezone can be; 
				game24/game (or nil) 	- 24 hour server/game time
				game12 		 		- 12 hour server/game time
				local24/local 		- 24 hour local time
				local12 				- 12 hour local time
				pvp 					- time until pvp wears off
				wintergrasp 			- time until next Wintergrasp
	]]--
	timezone = timezone and timezone:lower() or "game"

	if ( ( timezone == "local" ) or ( timezone == "local24" ) ) then
		return date( "%H:%M" )

	elseif ( timezone == "local12" ) then
		return date( "%I:%M %p" )

	elseif ( timezone == "game" ) or ( timezone == "game24" ) then
		local hours, minutes = GetGameTime();
		return ( format( "%02d", hours )..":"..format( "%02d", minutes ) )

	elseif ( timezone == "game12" ) then
		local hours, minutes = GetGameTime();
		return ( format( "%02d", ( ( hours > 12 ) or ( hours < 1 ) ) and ( hours - 12 ) or hours )..":"..format( "%02d", minutes )..( ( hours > 12 ) and " PM" or " AM" ) );

	elseif ( timezone == "pvp" ) then
		local sec = GetPVPTimer()
		if ( sec >= 0 ) and ( sec <= 301000 ) then
			seconds = floor( seconds / 1000 );
			return ( format( "%d", floor( seconds / 60 ) % 60 )..":"..format( "%02d", seconds % 60 ) )
		else
			return "--:--"
		end

	elseif ( timezone == "wintergrasp" ) then
		local seconds = GetWintergraspWaitTime();
		if seconds then
			return ( format( "%d", floor( seconds / 3600 ) )..":"..format( "%02d", floor( seconds / 60 ) % 60 )..":"..format( "%02d", seconds % 60 ) )
		else
			return ( "--:--:--" )
		end
	end
end
Tags["ap"] = function( unit )
	local base, posBuff, negBuff = UnitAttackPower( unit or "player" );
	return ( base + posBuff + negBuff )
end
Tags["rap"] = function( unit )
	local base, posBuff, negBuff = UnitRangedAttackPower( unit or "player" );
	return ( base + posBuff + negBuff )
end

--
-- Following ratings are for the unit "player" only
--
Tags["mcrit"] = function()
	return format( "%.1f",GetCritChance() )
end
Tags["mhaste"] = function()
	return format( "%.1f", GetCombatRatingBonus(18) )
end
Tags["rcrit"] = function()
	return format( "%.1f", GetRangedCritChance() )
end
Tags["rhaste"] = function()
	return format( "%.1f", GetCombatRatingBonus(19) )
end
Tags["sp"] = function()
	return max( GetSpellBonusDamage(2), GetSpellBonusDamage(3), GetSpellBonusDamage(4), GetSpellBonusDamage(5), GetSpellBonusDamage(6), GetSpellBonusDamage(7), GetSpellBonusHealing() )
end
Tags["scrit"] = function()
	return format( "%.1f",( max( GetSpellCritChance(2), GetSpellCritChance(3), GetSpellCritChance(4), GetSpellCritChance(5), GetSpellCritChance(6), GetSpellCritChance(7) ) ) )
end
Tags["shaste"] = function()
	return format( "%.1f", GetCombatRatingBonus(20) )
end
