﻿local addon, ns = ...;

ns.modules.font = ns:New( "font" );

function ns.modules.font:OnInit()
	local function SetFont( fontObject, font, size, style, shadowX, shadowY, shadowAlpha, shadowR, shadowG, shadowB )
		if not fontObject then return end
		
		fontObject:SetFont( font, size, style );

		if shadowX and shadowY then
			fontObject:SetShadowOffset( shadowX, shadowY );
		else
			fontObject:SetShadowOffset( 0, 0 );
		end
		
		if shadowR and shadowG and shadowB then
			fontObject:SetShadowColor( shadowR, shadowG, shadowB );
		else
			shadowR, shadowG, shadowB = 0, 0, 0;
		end
		
		if shadowAlpha then 
			fontObject:SetShadowColor( shadowR, shadowG, shadowB, shadowAlpha );
		else
			fontObject:SetShadowColor( shadowR, shadowG, shadowB, 2 / 3 );
		end
	end

	local text 								= ns.global.STANDARD_TEXT_FONT;
	local number							= ns.global.STANDARD_NUMBER_FONT;
	local large 							= ns.global.LARGE_TEXT_FONT;
	local damage 							= ns.global.DAMAGE_TEXT_FONT;
	local panel 							= ns.global.PANEL_TEXT_FONT;
	local unit 								= ns.global.UNIT_NAME_FONT;
	local name 								= ns.global.NAMEPLATE_FONT;

	CHAT_FONT_HEIGHTS 						= { 12, 14, 16, 18, 20, 22 };
	UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT 		= 12;

	UNIT_NAME_FONT 							= unit;
	NAMEPLATE_FONT 							= name;
	DAMAGE_TEXT_FONT 						= damage;
	STANDARD_TEXT_FONT 						= text;
	LARGE_TEXT_FONT 						= large; 	-- these are ours, not Blizzard globals
	PANEL_TEXT_FONT 						= panel;	-- *
	STANDARD_NUMBER_FONT 					= number;	-- *

	-- the order coincides with Blizzards order

	-- fonts
	SetFont( SystemFont_Large,  						text, 		15											);
	SetFont( SystemFont_Med1,  							text, 		12											);
	SetFont( SystemFont_Med3,  							text, 		13											);
	SetFont( SystemFont_Outline_Small,  				number, 	12,		"THINOUTLINE"						);
	SetFont( SystemFont_OutlineThick_Huge2,  			text, 		20,		"THINOUTLINE"						);
	SetFont( SystemFont_Shadow_Huge1,  					text, 		20,		"THINOUTLINE", 		0.75, -0.75		);
	SetFont( SystemFont_Shadow_Large,  					text, 		15,		nil, 				0.75, -0.75		);
	SetFont( SystemFont_Shadow_Med1,  					text, 		12,		nil, 				0.75, -0.75		);
	SetFont( SystemFont_Shadow_Med3,  					text, 		13,		nil, 				0.75, -0.75		);
	SetFont( SystemFont_Shadow_Outline_Huge2,  			text, 		20,		"THINOUTLINE", 		0.75, -0.75		);
	SetFont( SystemFont_Shadow_Small,  					text,	 	12,		nil,				0.75, -0.75		);
	SetFont( SystemFont_Small,  						text,	 	12											);
	SetFont( SystemFont_Tiny,  							text, 		12											);

	SetFont( NumberFont_OutlineThick_Mono_Small,  		number, 	12,		"THINOUTLINE"						);
	SetFont( NumberFont_Outline_Huge,  					number, 	28,		"THINOUTLINE"						);
	SetFont( NumberFont_Outline_Large,  				number, 	16,		"THINOUTLINE"						);
	SetFont( NumberFont_Outline_Med,  					number, 	14,		"THINOUTLINE"						);
	SetFont( NumberFont_Shadow_Med,  					text, 		12, 	nil, 				0.75, -0.75		);
	SetFont( NumberFont_Shadow_Small,  					text, 		12, 	nil, 				0.75, -0.75		);

	SetFont( QuestFont,  								text, 		12											);
	SetFont( QuestFontNormalSmall,  					text, 		12											);
	SetFont( QuestFont_Large,  							text, 		14											);

	SetFont( GameTooltipHeader,  						text, 		14,		"THINOUTLINE"						);

	SetFont( Tooltip_Med,  								text, 		12,		"THINOUTLINE"						);
	SetFont( Tooltip_Small,  							text, 		12,		nil,				0.75, -0.75		);
	
	-- fontstyles
	SetFont( GameFontNormal,  							text, 		12, 	nil, 				0.75, -0.75		);
	SetFont( GameFontHighlight,  						text, 		12, 	nil, 				0.75, -0.75		);
	SetFont( GameFontDisable,  							text, 		10										 	);
	SetFont( GameFontNormalSmall,  						text, 		10, 	nil, 				0.75, -0.75		);

	SetFont( NumberFontNormal,  						number, 	12,		"THINOUTLINE"						);
	SetFont( NumberFontNormalSmallGray,  				number, 	10,		"THINOUTLINE"						);

	SetFont( ChatFontNormal,  							text, 		14,		"THINOUTLINE"						);
	SetFont( ChatFontSmall,  							text, 		12,		"THINOUTLINE"						);

	SetFont( ZoneTextFont,  							large, 		32,		"THINOUTLINE"						);
	SetFont( SubZoneTextFont,  							large, 		24,		"THINOUTLINE"						);
	SetFont( PVPInfoTextFont,  							large, 		18, 	"THINOUTLINE"						);
	SetFont( ErrorFont,  								large,	 	14,		"THINOUTLINE"						);
	SetFont( CombatTextFont,  							damage, 	100, 	"THINOUTLINE"						);

	SetFont( AchievementDescriptionFont,  				text,	 	12											);

end
