﻿local addon, ns = ...

gLib = {};
gLib.ADDON = addon
gLib.VERSION = GetAddOnMetadata( addon, "Version" ); 

local Addon, AddonList, Event, L, Tags, Task = {}, {}, {}, {}, {}, {};

-- thanks to cargor for the idea to do this
local bagstrings = {
	["backpack"]		= { 0 };
	["bags"]			= { 1, 2, 3, 4 };
	["backpack+bags"]	= { 0, 1, 2, 3, 4, };
	["bank"]			= { 5, 6, 7, 8, 9, 10, 11 };
	["bankframe"]		= { -1 };
	["bankframe+bank"]	= { -1, 5, 6, 7, 8, 9, 10, 11 };
	["keyring"]			= { -2 };
};

-- any missing?
local LOCALES = { 
	deDE = true; 
	enGB = true; 
	enUS = true; 
	esES = true;
	esMX = true;
	frFR = true; 
	koKR = true; 
	ruRU = true; 
	zhCN = true; 
	zhTW = true; 
};

local RESERVEDEVENTS = { 
	ADDON_LOADED 	= true; 
	PLAYER_LOGIN 	= true; 
	PLAYER_LOGOUT 	= true;
};

-- god do I hate the way I've done this
local VALIDPOINTS = { 
	BOTTOM 			= true; 
	BOTTOMLEFT 		= true; 
	BOTTOMRIGHT 	= true; 
	CENTER 			= true; 
	LEFT 			= true; 
	RIGHT 			= true; 
	TOP 			= true; 
	TOPLEFT 		= true; 
	TOPRIGHT 		= true; 
};

local _G = _G;
local abs, date, error, floor, format, gsub, mod, pairs, setmetatable, sort, strlen, strsub, strupper, tinsert, tremove, tonumber, tostring, type, unpack = abs, date, error, floor, format, gsub, mod, pairs, setmetatable, sort, strlen, strsub, strupper, tinsert, tremove, tonumber, tostring, type, unpack;

noop = function() 
	return 
end

------------------------------------------------------------------------------------------------------------
-- 	Main Frame
------------------------------------------------------------------------------------------------------------

local frame = CreateFrame( "Frame" );

for event, reg in pairs( RESERVEDEVENTS ) do
	if reg then 
		frame:RegisterEvent( event );
	end
end

-- credits to Lonny and Paul Emmerich for inspiring this
local function OnUpdate( self, elapsed )
	for addon, tasks in pairs( Task ) do
		for i,v in pairs( tasks ) do
			v.elapsed = ( v.elapsed or 0 ) + elapsed;

			if ( v.elapsed >= v.interval ) then
				v.func( _G[addon] );
				v.elapsed = v.elapsed - v.interval;
			end
		end
	end
end
local function OnEvent( self, event, ... )
	if RESERVEDEVENTS[event] then
		frame[event]( self, event, ... );
		return
	end
	
	if Event[event] then
		for addon, func in pairs(Event[event]) do
			if (type(func) == "string") then
				_G[addon][event]( _G[addon], ... );
				
			elseif (type(func) == "function") then
				func( _G[addon], event, ... );
			end
		end
	end
end

function frame:ADDON_LOADED( event, name )
	if Addon[name] then
		Addon[name]:OnInit();
		
		if Addon[name].ADDON_LOADED then
			Addon[name]:ADDON_LOADED();
		end
	end
end
function frame:PLAYER_LOGIN( self, event )
	for name, addon in pairs( Addon ) do
		addon:OnEnable();

		if addon.PLAYER_LOGIN then
			addon:PLAYER_LOGIN();
		end
	end
end
function frame:PLAYER_LOGOUT( self, event )
	for name, addon in pairs( Addon ) do
		addon:OnDisable();

		if addon.PLAYER_LOGOUT then
			addon:PLAYER_LOGOUT();
		end
	end
end

frame:SetScript( "OnUpdate", OnUpdate );
frame:SetScript( "OnEvent", OnEvent );


------------------------------------------------------------------------------------------------------------
-- 	Template Addon
------------------------------------------------------------------------------------------------------------

local prototypeAddon = {};

function prototypeAddon:OnInit() return end
function prototypeAddon:OnEnable() return end
function prototypeAddon:OnDisable() return end

function prototypeAddon:RegisterEvent( event, func )
	--[[
		usage:	myAddon:RegisterEvent( event[, func] )
				- Same as the normal API RegisterEvent
				- Registers the event with gLib's handler
	]]--
	if ( type( event ) ~= "string") or ( event == "" ) then return end
	if RESERVEDEVENTS[event] then return end

	if not Event[event] then
		Event[event] = {};
		frame:RegisterEvent( event );
	end
	
	Event[event][self:GetName()] = func or event
end
function prototypeAddon:RegisterEvents( events )
	--[[ 
				- Registers multiple events at once
				- If the value is a function, then the index must be the eventname
				- If the value is a string, the value will be the event name, and the myAddon:EVENTNAME() will be called when the event fire
	]]--
	if ( type( events ) ~= "table" ) then return end
	
	for i,v in pairs( events ) do
		if ( type(v) == "string" ) then
			self:RegisterEvent(v);
		elseif type(v) == "function" then
			self:RegisterEvent(i, v);
		end
	end
end
function prototypeAddon:UnregisterEvent( event )
	if ( type( event ) ~= "string" ) or ( event == "" ) then return end
	if RESERVEDEVENTS[event] then return end
	
	if Event[event][self] then
		Event[event][self] = nil
	end
	if #Event[event] == 0 then
		Event[event] = nil;
		frame:UnregisterEvent( event );
	end
end
function prototypeAddon:Schedule( task, interval )
	--[[
		usage: 	myAddon:Schedule( task[, interval] )
				- registers the function <task> to be called every <interval> seconds
				- interval must be between 0.1s and 999s, or it will default to 0.5s
	]]--
	if ( type( task ) ~= "function" ) then return end
	
	Task[self:GetName()] = Task[self:GetName()] or {};
	
	local thetask = {
		func = task;
		interval = min( max( ( ( type( interval ) == "number" ) and abs( interval ) or 0.5 ), 0.1 ), 999 );
	};
	
	tinsert( Task[self:GetName()], thetask );
	
end
function prototypeAddon:GetName()
	return tostring( self.ADDON ) or tostring( gLib.ADDON ) or "nil";
end
function prototypeAddon:GetLocale()
	local addon = self.ADDON;
	
	if not L[addon] then return end
	
	local l = GetLocale();

	if not L[addon][l] then return end
	
	-- clean out all other locales except the current
	for locale,_ in pairs( L[addon] ) do
		if not( locale == l ) then
			L[addon][locale] = nil;
		end
	end
	return L[addon][l]
end


local function Size( obj, w, h )
	if type( w ) ~= "number" then return end
	if not( tonumber( h ) ) then h = w; end

	obj:SetWidth( w );
	obj:SetHeight( h );
end
local function Place( obj, arg1, arg2, arg3, arg4, arg5 )
	obj:ClearAllPoints();
	obj:SetPoint( arg1, arg2, arg3, arg4, arg5 );
end
local function Kill( obj )
	if not obj then return end

	if ( obj:GetObjectType() == "Frame" ) then
		obj:UnregisterAllEvents();
		obj:EnableMouse( false );
	end
	
	if not obj.SetPoint then return end

	obj:Size( 0.01 );
	obj:Place( "BOTTOMLEFT", "UIParent", "BOTTOMLEFT", -1, -1 );
	obj:Hide();

	obj.SetPoint = noop;
	obj.SetAllPoints = noop;
	obj.Show = noop;
end
local function Shrink( obj )
	if not obj.Size then return end
	obj:Size( 0.01 );
end
local function Release( obj )
	if not obj or not obj.GetName then return end
	local n = obj:GetName()
	if n and UIPARENT_MANAGED_FRAME_POSITIONS[n] then
		UIPARENT_MANAGED_FRAME_POSITIONS[n] = nil;
	end
end

local function GetScreenHeight()
	local res = GetCVar("gxResolution");
	if res then
		local h = tonumber( string.match( res, "%d+x(%d+)") );
		return ( h and h > 0 ) and h or UIParent:GetHeight()
	else
		return UIParent:GetHeight()
	end
end

local function GetScreenWidth()
	local res = GetCVar("gxResolution");
	if res then
		local w = tonumber( string.match( res, "(%d+)x%d+") );
		return ( w and w > 0 ) and w or UIParent:GetWidth()
	else
		return UIParent:GetWidth()
	end
end

-- GetPoint is not the best of functions. Here's our own version;
local function GetAllPoints( obj )
	if not obj or not obj.GetPoint then return end
	
	local left = obj:GetLeft();
	local right = GetScreenWidth() - obj:GetRight();
	local bottom = obj:GetBottom();
	local top = GetScreenHeight() - obj:GetTop();
	
	local hp, vp, x, y 

	if ( top > bottom ) then
		vp = "BOTTOM";
		y = bottom;
	else
		vp = "TOP";
		y = -top;
	end
	
	if ( left > right ) then
		hp = "RIGHT";
		x = -right;
	else
		hp = "LEFT";
		x = left;
	end
	
	return { vp..hp, "UIParent", vp..hp, x, y }
end

local function fix( object, isframe )
	getmetatable( object ).__index.Size = Size;
	getmetatable( object ).__index.Place = Place;
	getmetatable( object ).__index.Kill = Kill;
	getmetatable( object ).__index.Shrink = Shrink;
	getmetatable( object ).__index.Release = Release;
	getmetatable( object ).__index.GetAllPoints = GetAllPoints;
end

fix( CreateFrame( "Frame" ):CreateTexture() );
fix( CreateFrame( "Frame" ):CreateFontString() );

local done = {};
local frames = EnumerateFrames();

while frames do
	if not done[frames:GetObjectType()] then
		fix( frames );
		done[frames:GetObjectType()] = true;
	end
	frames = EnumerateFrames( frames );
end


------------------------------------------------------------------------------------------------------------
-- 	Local functions
------------------------------------------------------------------------------------------------------------

-- do NOT call this without both arguments, as this gives unpredictable behavior!! ( Recount bug anyone...? )
local function DeepCopy( old, new )
	-- fully copy a (nested) table, instead of simply creating pointers

	-- if old is not a table, then just return it
	if ( type(old) ~= "table" ) then 
		return old 
	end 

	-- if this is a frame, get the hell out
	if old.RegisterEvent then 
		return 
	end
	
	-- if this is a texture or otherwise movable item, get out
	if old.SetPoint then 
		return 
	end

	-- create our new table if it doesn't exist already
	local new = ( ( new and type( new ) == "table") and new ) or {}

	-- smart function, hell yeah
	local function iterate( oldt, newt )
		for i,v in pairs( oldt ) do
			if type(oldt[i]) == "table" then 
				newt[i] = {};
				iterate( oldt[i], newt[i] );
			else
				newt[i] = oldt[i];
			end
		end
	end

	iterate(old, new);

	-- return the new or modified table
	return new
end
local function CreateIcon( iconPath, iconValues )
	if not iconPath or type( iconPath ) ~= "string" then return end
	iconValues = ( type( iconValues ) == "string" and iconValues ) or "0:0:0:0";

	if( type( iconValues ) == "table" ) then
		iconValues = table.concat( iconValues, ":" );
	end
	return ("|T%s:%s|t"):format( iconPath, iconValues )
end
local function Crop( str, maxlength )
	if not(str) or not( type( str ) == "string" ) or ( string.len( str ) == 0 ) then return end
	if maxlength and type( maxlength ) ~= "number" then maxlength = 4; end
	
	maxlength = ( maxlength < 4 ) and 4 or maxlength;

	if ( strlen( str ) <= maxlength ) then return str end

	return strsub( str, 1, maxlength-3 ).."..."
end
local function Cvars( cvars )
	for i,v in pairs( cvars ) do
		SetCVar( i, v );
	end
end
local function Embed( target, source )
	-- embeds all the values and methods from source into target
	if not target or not source then return end
	if type( source ) ~= "table" or type( target ) ~= "table" then return end 

	for i,v in pairs( source ) do
		target[i] = v;
	end
end
local function GetBagSpace(bags)
	if not bags or type( bags ) ~= "string" or not bagstrings[bags] then return end

	local free, total, used = 0, 0, 0;
	for _,i in pairs( bagstrings[bags] ) do
		free, total, used = free + GetContainerNumFreeSlots(i), total + GetContainerNumSlots(i), total - free;
	end
	
	return free, total, used, floor( free / total * 100 )
end
local function GetChoice ( choice )
	return ( ( ( choice == 1 ) or ( choice == true ) ) and 1 ) or ( not( choice ) or ( choice == 0 ) ) and 0 or 0
end
local function GetTrueChoice( choice )
	return ( ( ( choice == true ) or ( choice == 1 ) ) and true ) or ( ( not( choice ) or ( choice == 0 ) ) and false ) or false
end
local function Lower( str )
	-- return only the lowercase characters from a given string
	if not str or type( str ) ~= "string" then return end

	-- usual syntax checks	
	if not(str) then return end
	if type(str) ~= "string" then return end
	if strlen(str) == 0 then return end
	
	-- return the string stripped of uppercase characters
	return ( gsub(str, "[%u]", "") )
end
local function Print( ... )
	--[[
		Prints out any number of messages 
		on separated lines in the main chat frame.
		
		{...} can be any number of comma-separated messages, 
		or a table containing strings. 
		
		Empty values "" will add a blank line.
	]]--
	
	local args = { ... };
	local function printmessage( args )
		for i, a in pairs( args ) do
			-- call itself if the input string is a table
			if type(a) == "table" then 
				printmessage( a ); 
			else
				DEFAULT_CHAT_FRAME:AddMessage( ( a ~= "" ) and a or " " );
			end
		end
	end
	printmessage(args);
end
local function RegisterTag( tag, func )
	if ( type( tag ) ~= "string" ) then return end
	if ( type( func ) ~= "function" ) then return end
	if Tags[tag] then return end
	
	Tags[tag] = func;
end
local function SetMeta( object, addon )
	object.ADDON 			= addon; 
	object.VERSION 			= GetAddOnMetadata( object.ADDON, "Version" ) or ""; 
	object.NAME 			= GetAddOnMetadata( object.ADDON, "Title" ) or addon; 
	object.REALM 			= ( GetRealmName() );
	object.CHARNAME 		= ( select( 1, UnitName( "player" ) ) );
	object.FACTION 			= ( UnitFactionGroup( "player" ) );
end

-- our theory here is that our function is better than the built-in
local function split( str, separator )
	if type( str ) ~= "string" or type( separator ) ~= "string" then return end
	--
	-- 	usage: 	{substr1, substr2,...} = string.split(str, separator) or str:split(separator)
	--
	local t = {}  
	local fpat = "(.-)" .. separator
	local last_end = 1
	local s, e, cap = str:find(fpat, 1)
	while s do
		if s ~= 1 or cap ~= "" then
			tinsert(t,cap)
		end
		last_end = e+1
		s, e, cap = str:find(fpat, last_end)
	end
	if last_end <= #str then
		cap = str:sub(last_end)
		tinsert(t, cap)
	end
	return t
end
local function SortByKey( tbl )
	-- create a sorted array of the keys in a given table

	if not tbl or type( tbl ) ~= "table" then return end
	
	-- create a new empty table to sort our keys in
	local ntbl = {}
	
	for i,v in pairs(tbl) do
		tinsert( ntbl, i ) 
	end
	
	-- sort our new table
	sort( ntbl )
	
	-- return our sorted array
	return ntbl
end
local function Toggle( object, state )
	-- sets the visibily of object
	if not object.Show then return end

	-- Hides/Shows any given element
	if state then
		object:Show();
	elseif not state then
		object:Hide();
	end
end
local function TrueTable( indexlist )
	-- create a table with all given values set to true

	if not indexlist then return nil end
	if not( type( indexlist ) == "table" ) then return nil end
	
	local truetable = {}
	for i,v in pairs( indexlist ) do
		-- take the values from our original table, 
		-- and turn them into indexes in our new.
		truetable[v] = true;
	end
	return truetable
end
local function Upper( str )
	-- return only the uppercase characters from a given string

	-- usual syntax checks	
	if not( str ) then return end
	if type( str ) ~= "string" then return end
	if strlen( str ) == 0 then return end
	
	-- return the string stripped of lowercase characters
	return (gsub(str, "[%l]", ""))
end

local function PixelPerfectScale()
	-- In 4.0.x+ one pixel is one pixel big wehen not using custom uiscales
	SetCVar( "useUiScale", 0 );
	ReloadUI();
end

-- the following functions depend on one or more of the above. do not change the order.
local function ArgCheck( user, defaults ) 
	-- compare the types of the user input to the default settings, 
	-- and fill in the holes with the given defaults

	if not( defaults ) or type( defaults ) ~= "table" then return end

	if ( not(user) ) or ( type(user) ~= "table" ) then 
		user = {}; 
	end

	-- this is where Recount_MainWindow crept in
	-- let's make sure badly coded addons doesn't break the game
	if user.RegisterEvent or user.SetPoint then return user end

	for i,v in pairs( defaults ) do
		user[i] = ( ( user[i] == nil ) and DeepCopy( defaults[i], nil ) ) or user[i];		-- get rid of nil values, keep false
		
		if ( type( user[i] ) ~= type( defaults[i] ) ) then
			user[i] = DeepCopy( defaults[i], nil );				-- if the type is wrong, replace with default value
		elseif ( type( user[i] ) == "table" ) then
			user[i] = ArgCheck( user[i], defaults[i] );		-- if the value is a table, verify the elements of the table
		end
	end
	
	return user
end
local function ResetDefaults( user, defaults )
	-- reset a user database to it's defaults

	if not( defaults ) or type( defaults ) ~= "table" then return end
	if ( not( user ) ) or ( type( user ) ~= "table" ) then 
		user = {}; 
	end

	wipe( user );
	user = DeepCopy( defaults, user );

	return user
end 
local function Slash( self, name, func, slashes )
	if not( name ) or type( name )~= "string" then 
		name = ( type( slashes ) == "table" ) and slashes[1] or ( type( slashes ) == "string" ) and slashes or  "";
		elseif ( type( slashes ) == "string" ) then 
	end
	func = ( ( type(func) == "function" ) and func ) or ( function() end )

	if self and self:GetName() then 
		name = self:GetName()..name;
	end
	
	if ( type( slashes ) == "table" ) then 
		for i,v in pairs( slashes ) do
			_G["SlashCmdList"][name] = function( argString ) 
				func( unpack( split( argString:lower(), " " ) ) );
			end;
			_G["SLASH_"..name..i] = "/"..v;
		end
	elseif ( type( slashes ) == "string" ) then 
		_G["SlashCmdList"][name] = function( argString ) 
			func( unpack( split( argString:lower(), " " ) ) );
		end;
		_G["SLASH_"..name.."1"] = "/"..slashes;
	end
end
local function Tag( str  )
	--[[
		usage: 	string.tag("[tagname:arg1,arg2,arg3...] some text [tagname:arg1,arg2,arg3...]") or str:tag()
			credits to cargor for the inspiration to this
	]]--
	if not( str ) or ( type( str ) ~= "string" ) then return end

	return ( str:gsub( "%[([^%]:]+):?(.-)%]", function( a, b ) return Tags[a]( unpack( split( b, "[,]+") ) ) end ) )
end


------------------------------------------------------------------------------------------------------------
-- 	Interface Options Menus
------------------------------------------------------------------------------------------------------------

local function CreateText( args )
	args = ArgCheck( args, {
		parent						= UIParent; 
		name 						= "";
		framestrata 				= "BACKGROUND"; 
		font 						= ( select( 1, GameFontNormal:GetFont() ) ); 
		fontsize 					= ( select( 2, GameFontNormal:GetFont() ) ); 
		point 						= "TOPLEFT"; 
		relpoint 					= "TOPLEFT"; 
		anchor 						= UIParent; 
		offsetX 					= 0; 
		offsetY 					= 0; 
		justifyH 					= "LEFT"; 
		justifyV 					= "TOP"; 
		text 						= ""; 
		maxwidth 					= InterfaceOptionsFramePanelContainer:GetWidth() - 40; 
	});
	
	--double check points and alignments
	if not( VALIDPOINTS[args.point] ) then args.point = "TOPLEFT"; end
	if not( VALIDPOINTS[args.relpoint] ) then args.relpoint = "TOPLEFT"; end
	if not( ( args.justifyH == "RIGHT" ) or ( args.justifyH == "LEFT" ) ) then args.justifyH = "LEFT"; end
	if not( ( args.justifyV == "TOP" ) or ( args.justifyV == "BOTTOM" ) ) then args.justifyV = "TOP"; end

	local f = args.parent:CreateFontString( args.name, args.framestrata );
	f:SetWordWrap( true );
	f:SetNonSpaceWrap( false );
	f:SetWidth( args.maxwidth );
	f:SetFont( args.font, args.fontsize );
	f:SetText( args.text );
	f:SetJustifyH( args.justifyH );
	f:SetPoint( args.point, args.anchor, args.relpoint, args.offsetX, args.offsetY );
	return f;
end
local function CreateCheckButton( args )
	args = ArgCheck( args, {
		name 						= "";
		text 						= "";
		maxwidth 					= InterfaceOptionsFramePanelContainer:GetWidth() - 40;
		parent 						= UIParent;
		relpoint 					= "TOPLEFT";
		anchor 						= UIParent;
		offsetX 					= 0;
		offsetY 					= 0;
		checked 					= false;
		justifyH 					= "LEFT";
		justifyV 					= "TOP"; 
		onclick						= function() end;
	});
	
	-- double check region points and alignments
	if not( VALIDPOINTS[args.relpoint] ) then args.relpoint = "TOPLEFT"; end
	if not( ( args.justifyH == "RIGHT" ) or ( args.justifyH == "LEFT" ) ) then args.justifyH = "LEFT"; end
	if not( ( args.justifyV == "TOP" ) or ( args.justifyV == "BOTTOM" ) ) then args.justifyV = "TOP"; end

	local b = CreateFrame("CheckButton", args.name, args.parent, "InterfaceOptionsCheckButtonTemplate");
	b:SetPoint( args.relpoint, args.anchor, args.relpoint, args.offsetX, args.offsetY );
	b:SetChecked( args.checked );
	b:SetScript( "OnClick", args.onclick );

	b.text = CreateText({
		parent 				= b; 
		name 				= args.name.."Text";
		framestrata 		= "ARTWORK";
		font 				= ( select( 1, GameFontWhite:GetFont() ) ); 
		fontsize 			= ( select( 2, GameFontWhite:GetFont() ) ); 
		point 				= "LEFT";
		relpoint 			= "RIGHT";
		anchor 				= b;
		text 				= args.text;
		maxwidth 			= args.maxwidth - b:GetWidth();
	});

	return b;
end
local function CreateDropDown( args )
	--[[
		
		args.menutable = {
			-- table of tables
			-- 
			{
				level 		= [INTEGER]; -- The menu level, 1 being bottom
				text 		= [STRING, function()]; -- (level == 1) The string or function returning a string to be displayed at the button. If a function, this will update whenever the user selects something
				value 		= [nil, STRING]; -- if this is a submenu, (level > 1) then this is the value (UIDROPDOWNMENU_MENU_VALUE) it is connected to
				menulist 	= {
					-- table of tables with the following options available;
					-- (cut and pasted from FrameXML\UIDropDownMenu.lua)
					{
						text 			= [STRING] -- The text of the button
						value 			= [ANYTHING] -- The value that UIDROPDOWNMENU_MENU_VALUE is set to when the button is clicked
						func 			= [function()] -- The function that is called when you click the button
						checked 			= [nil, true, function] -- Check the button if true or function returns true
						isTitle 			= [nil, true] -- If it's a title the button is disabled and the font color is set to yellow
						disabled 		= [nil, true] -- Disable the button and show an invisible button that still traps the mouseover event so menu doesn't time out
						hasArrow 		= [nil, true] -- Show the expand arrow for multilevel menus
						hasColorSwatch 	= [nil, true] -- Show color swatch or not, for color selection
						r 				= [1 - 255] -- Red color value of the color swatch
						g 				= [1 - 255] -- Green color value of the color swatch
						b 				= [1 - 255] -- Blue color value of the color swatch
						colorCode 		= [STRING] -- "|cAARRGGBB" embedded hex value of the button text color. Only used when button is enabled
						swatchFunc 		= [function()] -- Function called by the color picker on color change
						hasOpacity 		= [nil, 1] -- Show the opacity slider on the colorpicker frame
						opacity 			= [0.0 - 1.0] -- Percentatge of the opacity, 1.0 is fully shown, 0 is transparent
						opacityFunc 		= [function()] -- Function called by the opacity slider when you change its value
						cancelFunc 		= [function(previousValues)] -- Function called by the colorpicker when you click the cancel button (it takes the previous values as its argument)
						notClickable 		= [nil, 1] -- Disable the button and color the font white
						notCheckable 		= [nil, 1] -- Shrink the size of the buttons and don't display a check box
						owner 			= [Frame] -- Dropdown frame that "owns" the current dropdownlist
						keepShownOnClick 	= [nil, 1] -- Don't hide the dropdownlist after a button is clicked
						tooltipTitle 		= [nil, STRING] -- Title of the tooltip shown on mouseover
						tooltipText 		= [nil, STRING] -- Text of the tooltip shown on mouseover
						justifyH 		= [nil, "CENTER"] -- Justify button text
						arg1 			= [ANYTHING] -- This is the first argument used by func
						arg2 			= [ANYTHING] -- This is the second argument used by func
						fontObject 		= [FONT] -- font object replacement for Normal and Highlight
						menuTable 		= [TABLE] -- This contains an array of info tables to be displayed as a child menu
					};
				};
			};
		};
	]]--
	args = ArgCheck( args, {
		parent						= UIParent; 
		name 						= "";
		text 						= "";

		maxwidth 					= InterfaceOptionsFramePanelContainer:GetWidth() - 40;
		width 						= 100;
		padding 					= 2;
		justify 					= "LEFT";

		point 						= "TOPLEFT";
		anchor 						= UIParent; 
		relpoint 					= "TOPLEFT";
		offsetX 					= 0;
		offsetY 					= 0;

		menutable 					= {};
	});
	
	-- create our main button frame
	local button = CreateFrame( "Frame", args.name, args.parent );
	button:SetPoint( args.relpoint, args.anchor, args.relpoint, args.offsetX, args.offsetY );
	button:SetWidth( args.maxwidth );
	button:SetHeight( ( select( 2, GameFontNormal:GetFont() ) ) );
	
	-- create the description text for our dropdown
	button.text = CreateText({
		parent 				= button; 
		name 				= args.name.."Text";
		framestrata 		= "ARTWORK";
		font 				= ( select( 1, GameFontNormal:GetFont() ) ); 
		fontsize 			= ( select( 2, GameFontNormal:GetFont() ) ); 
		point 				= "TOPLEFT";
		relpoint 			= "TOPLEFT";
		anchor 				= button;
		justifyH 			= "LEFT";
		text 				= NORMAL_FONT_COLOR_CODE..args.text.."|r";
		maxwidth 			= args.maxwidth - args.width - 40;
	});
	
	-- create the dropdown frame
	button.dropdown = CreateFrame( "Frame", args.name.."DropDown", args.parent, "UIDropDownMenuTemplate" );
	button.dropdown:SetPoint( "TOPLEFT", button.text, "TOPRIGHT", 0, 8 );

	-- create a shortcut, since we're lazy
	local frame = button.dropdown;

	-- the table we store our buttons in
	local dd = {};
	
	-- our main loop to iterate over potential submenus
	for i,v in pairs(args.menutable) do
		local text 		= args.menutable[i].text;
		local value 	= args.menutable[i].value;
		local level 	= args.menutable[i].level or 1; -- if no level is given, we assume this is a single level menu
		local menulist 	= args.menutable[i].menulist;
		
		local pointer; 
		
		UIDropDownMenu_SetWidth( frame, args.width, args.padding );
		UIDropDownMenu_JustifyText( frame, args.justify );

		-- the text displayed on the button
		if level == 1 then 
			if type( text ) == "function" then
				-- remind me why I chose an OnUpdate function here....?
				-- let's see if it works without it
				
--				frame:HookScript( "OnUpdate", function() 
					UIDropDownMenu_SetText( frame, tostring( text() ) ); 
--				end );
			elseif type( text ) == "string" then
				UIDropDownMenu_SetText( frame, text );
			end
		end
		
		-- decide where in the dd table this info goes
		if level == 1 then
			-- create the top level
			-- we always assume this does not exist
			dd[level] 			= {};
			pointer 			= dd[level];
		else
			-- create the sublevel if it doesn't already exist
			dd[level] 			= dd[level] or {};
			dd[level][value] 	= {};
			pointer 			= dd[level][value];
		end
		
		-- create the menu items for this level and branch
		for j,w in pairs( args.menutable[i].menulist ) do
			pointer[j] = {};
			for x,y in pairs( args.menutable[i].menulist[j] ) do
				-- insert the values into the correct place in the dd table 
				pointer[j][x] = args.menutable[i].menulist[j][x];
			end
		end

	end
	

	local info = {}
	frame.initialize = function( self, level )
		if not level then return end
		wipe( info );
		local pointer 
		
		--  figure out which part of the dd table to get our info from
		if level == 1 then
			pointer = dd[level];
		elseif level > 1 then
			pointer = dd[level][UIDROPDOWNMENU_MENU_VALUE];
		end

		-- Create the buttons for this level and branch
		for i,v in pairs( pointer ) do
			for j,w in pairs(v) do
				info[j] = w;
			end
			
			-- add the button
			UIDropDownMenu_AddButton( info, level );
		end
	end

	frame.Title = CreateFrame( "Button", args.name.."_Button", frame );
	frame.Title:SetAllPoints( frame);
	frame.Title:SetScript("OnClick", function( self, button, down )
		if button == "RightButton" then
			ToggleDropDownMenu( 1, nil, frame, self:GetName(), 0, 0, "MENU" );
		end
	end)
	frame.Title:RegisterForClicks( "RightButtonUp" );

	local bh = max( frame.Title:GetHeight(), button.text:GetHeight(), button:GetHeight(), frame:GetHeight() );
	
	button:SetHeight( bh );
	
	return button
end
local function CreateOptionsPanel( panel, userDB, defaultDB )
	--[[

		Converts the array given in {panel} to an Interface Options menu,
			and returns the link to the new menu object.
		
		All the .name values are valid as links for the InterfaceOptionsFrame_OpenToCategory()
			function to open to.
		
		Format:

			panel.title 			= title of the addon to which the menu belongs
			panel.version 		= version number of the addon
			panel.description 	= description of the addon, the text shown on the menu's front page
			
			panel.okay 			= Function to be executed when you hit "Ok"
			panel.cancel 			= Function to be executed when you hit "Cancel"
			panel.default 		= Function to be executed when you hit "Default"
			panel.refresh 		= Hell if I know
			
			panel.option[x] = frontpage menu options array, se below for more settings
				panel.option[x].name 			= text on the menu option
				panel.option[x].type 			= what kind of menu option?
												"button" = normal checkbutton
												"dropdown" = dropdown menu										
												"editbox" = editbox											TODO
												"slider" = sliderbar											TODO
												"colorpicker" = function that provides an inputbox with			TODO
													a hexadecimal color value, as well as a colorwheel
													accessible by clicking the color thumbnail 
				panel.option[x].list 			= dropdown menus												
				panel.option[x].password 		= boolean. if true, editbox will mask the contents					TODO
				panel.option[x].min 			= minimum value assigned to sliders									TODO
				panel.option[x].max 			= maximum value assigned to sliders									TODO
				panel.option[x].maxletters 		= sets the maximum numbers of allowed letters in an editbox 			TODO
				panel.option[x].value 			= value of the menu option
				panel.option[x].panelfunction 	= function connected to the menu option
			
			panel.menu[x] = submenu
				panel.menu[x].name = title of the subtree
				panel.menu[x].option[x] = frontpage menu options array, se below for more settings
					panel.menu[x].option[x].name 			= text on the menu option
					panel.menu[x].option[x].type 			= what kind of menu option?
															"button" = normal checkbutton
															"dropdown" = dropdown menu										
															"editbox" = editbox											TODO
															"slider" = sliderbar											TODO
															"colorpicker" = function that provides an inputbox with			TODO
														a hexadecimal color value, as well as a colorwheel
														accessible by clicking the color thumbnail 
					panel.menu[x].option[x].list 			= dropdown menus
					panel.menu[x].option[x].password 		= boolean. if true, editbox will mask the contents					TODO
					panel.menu[x].option[x].min 			= minimum value assigned to sliders									TODO
					panel.menu[x].option[x].max 			= maximum value assigned to sliders									TODO
					panel.menu[x].option[x].maxletters 		= sets the maximum numbers of allowed letters in an editbox 			TODO
					panel.menu[x].option[x].value 			= value of the menu option
					panel.menu[x].option[x].panelfunction 	= function connected to the menu option
		
		Example:
			your_panel = Menu.CreateOptionsPanel(your_array)
			InterfaceOptionsFrame_OpenToCategory(your_array.menu[3].name)
			
			* 	Creates an Interface Options menu tree based on the table your_array, 
				and opens the menu at the 4th submenu.
	]]--
	if not panel then return end
	if not(panel.title) then return end
	
	local p
	
	-- is this a new menu, or does it belong to an existing one?
	if not panel.panel then
		p = CreateFrame( "Frame", "Menu_"..panel.title, UIParent );

		-- create main container frame for the menu tree
		p.head = CreateText({
			anchor 			= p; 
			framestrata 	= "ARTWORK";
			font 			= ( select( 1, GameFontNormalLarge:GetFont() ) ); 
			fontsize 		= ( select( 2, GameFontNormalLarge:GetFont() ) );
			name 			= panel.title.."OPTIONS_TITLE";
			offsetX 		= 14;
			offsetY 		= -14;
			parent 			= p;
			text 			= panel.title.." "..panel.version;
		});
		p.name = panel.title.." "..panel.version;
		if panel.description then 
			p.body = CreateText({
				parent 			= p;
				name 			= panel.title.."OPTIONS_DESCRIPTION"; 
				framestrata 	= "ARTWORK";
				font 			= ( select( 1, GameFontWhite:GetFont() ) );
				fontsize 		= ( select( 2, GameFontWhite:GetFont() ) );			
				anchor 			= p.head;
				offsetX 		= 8;
				offsetY 		= -20;
				text 			= panel.description;
			});
		end
	else
		-- hook this up to another menu?
		p = CreateFrame( "Frame", "Menu_"..panel.title, panel.panel );
		
		p.name = panel.title.." "..panel.version; 
		p.head = CreateText({
			anchor 			= p; 
			framestrata 	= "ARTWORK";
			font 			= ( select( 1, GameFontNormalLarge:GetFont() ) );
			fontsize 		= ( select( 2, GameFontNormalLarge:GetFont() ) );
			name 			= p.name.."_TITLE";
			offsetX 		= 14;
			offsetY 		= -14;
			parent 			= p;
			text 			=  panel.title.." "..panel.version;
		});
		if panel.description then 
			p.body = CreateText({
				parent 			= p;
				name 			= panel.title.."OPTIONS_DESCRIPTION"; 
				framestrata 	= "ARTWORK";
				font 			= ( select( 1, GameFontWhite:GetFont() ) );
				fontsize 		= ( select( 2, GameFontWhite:GetFont() ) );			
				anchor 			= p.head;
				offsetX 		= 8;
				offsetY 		= -20;
				text 			= panel.description;
			});
		end
		
		p.parent = panel.panel.name
	end
	
	-- if it exists, show the frontpage's menu options
	if panel.option then
		p.option = {};
		
		local previousparent = p.head;
		local xoff,yoff = 0, -28 ; 
		
		-- get a sorted array of the keys in panel.option
		local stbl = SortByKey( panel.option );
		
		-- iterate over our sorted array, retrieve the keys for panel.option in order
		for stblcount = 1, #stbl,1 do
			local maincounter = stbl[stblcount] 
			if maincounter == 0 and p.body then yoff = yoff - p.body:GetHeight(); end;
			if panel.option[maincounter].type == "button" then
				p.option[maincounter] = CreateCheckButton({
					name 				= "Menu_"..p.name.."_CB"..maincounter; 
					text 				= panel.option[maincounter].name;
					parent 				= p;
					anchor 				= previousparent; 
					offsetX 			= xoff; 
					offsetY 			= yoff;
					checked 			= panel.option[maincounter].value;
					onclick 			= panel.option[maincounter].panelfunction;
				});
			end
			if panel.option[maincounter].type == "dropdown" then
				p.option[maincounter] = CreateDropDown({
					name 				= "Menu_"..p.name.."_CB"..maincounter; 
					text 				= panel.option[maincounter].name;
					parent 				= p;
					anchor 				= previousparent; 
					offsetX 			= xoff; 
					offsetY 			= yoff;
					menutable 			= panel.option[maincounter].list;
				});
			end
			yoff = -( p.option[maincounter]:GetHeight() + 2 );
			previousparent = p.option[maincounter];
		end
		
		-- set the parent to our menu
		p.option.parent = (panel.panel and panel.panel.name) or p.name;
	end

	-- setup the control functions
	p.okay = panel.okay or function () end;
	p.cancel = panel.cancel or function () end;
	p.default = function()
		if not(userDB) or not(defaultDB) then return end
		
		-- overwrite all options with the defaults
		userDB = ResetDefaults( userDB, defaultDB );
		
		-- call the user's function, if any
		if panel.default then panel.default(); end
		
		-- This doesn't appear to reload the UI, just the menu. Perfect.
		ReloadUI();
		
	end;
	p.refresh = panel.refresh or function () end

	-- add main menu object
	InterfaceOptions_AddCategory(p);

	
	-- if we're hooking this menu into an existing one, we don't really want submenus
	if not panel.panel then
		-- create submenus
		if panel.menu then
			p.menu = {};
			
			-- get a sorted array of the keys in panel.option
			local stbl = SortByKey(panel.menu);
			
			-- iterate over our sorted array, retrieve the keys for panel.option in order
			for stblcount = 1, #stbl,1 do
				local counter = stbl[stblcount] 
				p.menu[counter] = CreateFrame("Frame", panel.menu[counter].name, p);
				p.menu[counter].name = panel.menu[counter].name; 
				p.menu[counter].head = CreateText({
					anchor 			= p.menu[counter]; 
					framestrata 	= "ARTWORK";
					font 			= ( select( 1, GameFontNormalLarge:GetFont() ) );
					fontsize 		= ( select( 2, GameFontNormalLarge:GetFont() ) );
					name 			= p.menu[counter].name.."_TITLE";
					offsetX 		= 14;
					offsetY 		= -14;
					parent 			= p.menu[counter];
					text 			= p.name.." "..panel.menu[counter].name;
				});
				if panel.menu[counter].description then 
					p.menu[counter].body = CreateText({
						parent 			= p.menu[counter];
						name 			= panel.menu[counter].name.."SUBOPTIONS_DESCRIPTION"..counter; 
						framestrata 	= "ARTWORK";
						font 			= ( select( 1, GameFontWhite:GetFont() ) );
						fontsize 		= ( select( 2, GameFontWhite:GetFont() ) );
						anchor 			= p.menu[counter].head;
						offsetX 		= 8;
						offsetY 		= -20;
						text 			= panel.menu[counter].description;
					});
				end
				p.menu[counter].option = {};
				local previousparent = p.menu[counter].head;
				local xoff,yoff = 0, -28;

				-- get a sorted array of the keys in panel.option
				local sstbl = SortByKey(panel.menu[counter].option);
				
				-- iterate over our sorted array, retrieve the keys for panel.option in order
				for sstblcount = 1, #sstbl,1 do
					local subcounter = sstbl[sstblcount] 
					if subcounter == 0 and p.menu[counter].body then yoff = yoff - p.menu[counter].body:GetHeight(); end				
					if panel.menu[counter].option[subcounter].type == "button" then
						p.menu[counter].option[subcounter] = CreateCheckButton({
							name 				= "Menu_"..p.menu[counter].name.."_CB"..subcounter; 
							text 				= panel.menu[counter].option[subcounter].name;
							parent 				= p.menu[counter];
							anchor 				= previousparent; 
							offsetX 			= xoff; 
							offsetY 			= yoff;
							checked 			= panel.menu[counter].option[subcounter].value;
							onclick 			= panel.menu[counter].option[subcounter].panelfunction;
						});
					end
					if panel.menu[counter].option[subcounter].type == "dropdown" then
						p.menu[counter].option[subcounter] = CreateDropDown({
							name 				= "Menu_"..p.menu[counter].name.."_CB"..subcounter; 
							text 				= panel.menu[counter].option[subcounter].name;
							parent 				= p.menu[counter];
							anchor 				= previousparent; 
							offsetX 			= xoff; 
							offsetY 			= yoff;
							menutable 			= panel.option[maincounter].option[subcounter].list;
						});
					end
					previousparent = p.menu[counter].option[subcounter];
					yoff = -( p.menu[counter].option[subcounter]:GetHeight() + 2 );
				end
				
				-- add our submenu				
				p.menu[counter].parent = p.name;
				InterfaceOptions_AddCategory( p.menu[counter] );
			end
		end
	end
	
	-- return the menu object
	return p
end


------------------------------------------------------------------------------------------------------------
-- 	API calls
------------------------------------------------------------------------------------------------------------
GL_Embed = Embed;
GL_Frame_GetAllPoints = GetAllPoints;
GL_GetChoice = GetChoice;
GL_GetTrueChoice = GetTrueChoice;
GL_GetScreenHeight = GetScreenHeight;
GL_GetScreenWidth = GetScreenWidth;
GL_PixelPerfectScale = PixelPerfectScale;
GL_SetCVars = Cvars;
GL_Table_Copy = DeepCopy;
GL_Table_Validate = ArgCheck;
GL_CreateInterfaceOptionsPanel = CreateOptionsPanel;

string.upperonly = function(str) return Upper(str); end
string.tag = function(tagstr) return Tag(tagstr); end


------------------------------------------------------------------------------------------------------------
-- 	gLib calls
------------------------------------------------------------------------------------------------------------

--
-- create a new addon that inherits all methods from prototypeAddon
--
function gLib:newAddon( name )
	if not( name ) or type( name ) ~= "string" or name == "" or _G[name] then return end

	Addon[name] = gLib:newClass( name, prototypeAddon );

	-- this is for backwards compability only, will most likely be removed in a future build
	Addon[name].newClass = gLib.newClass;
	Addon[name].ArgumentCheck = ArgCheck;
	Addon[name].CreateOptionsPanel = CreateOptionsPanel;
	Addon[name].Slash = Slash;

	SetMeta( Addon[name], name );

	_G[name] = Addon[name];

	tinsert( AddonList, name );

	return Addon[name]
end

--
-- create a new instance of the class parentClass
-- that inherits all of parentClass' methods
--
-- 	newClass = gLib:newClass( "myClass", yourClass )
-- 	
function gLib:newClass( name, parentClass )

	if not name or type( name ) ~= "string" then return end

    local newClass = {};
    local class_mt = { __index = newClass };

    function newClass:create()
        local newinst = {};
        setmetatable( newinst, class_mt );
        return newinst
    end

    if parentClass then
        setmetatable( newClass, { __index = parentClass } );
    end

    -- Return the class object of the instance
    function newClass:class()
        return newClass
    end

    -- Return the super class object of the instance
    function newClass:superClass()
        return parentClass
    end

    -- Return true if the caller is an instance of theClass
    function newClass:isa( theClass )
        local b_isa = false;

        local cur_class = newClass;

        while ( nil ~= cur_class ) and ( false == b_isa ) do
            if cur_class == theClass then
                b_isa = true;
            else
                cur_class = cur_class:superClass();
            end
        end

        return b_isa
    end
	
    return newClass
	
end

--
-- 	if no locale is given (nil), the content is passed to all locales
--
function gLib:AddLocale( addon, locale, content )
	if not addon or type( addon ) ~= "string" or not content or type( content ) ~= "table" then return end
	
	-- create addon's main locale table 
	L[addon] = L[addon] or {}

	-- add the content to a specific locale if given
	if locale and LOCALES[locale] then
		-- create the locale table
		L[addon][locale] = L[addon][locale] or {}
	
		-- if no content was given, we're done here
		if not content then return end

		-- add the content
		L[addon][locale] = DeepCopy( content, L[addon][locale] );
		
	-- add the content to ALL locales
	elseif not( locale ) then
		for i,v in pairs( LOCALES ) do
			-- add the locale
			L[addon][i] = L[addon][i] or {}

			-- add the content, if any
			if content then
				L[addon][i] = DeepCopy( content, L[addon][i] );
			end
		end
	end
	
end

--
-- 	if no locale is given (nil), the client's default locale is returned
--
function gLib:FetchLocale( addon, locale )
	if not ( L[addon] ) or ( locale and not( L[addon][locale] ) ) then return end

	return L[addon][locale or GetLocale()]
end

-- all-in-one function
-- allows you to create an addon, register events, setup slash commands, and validate settings
function gLib:New( db )
	db = ArgCheck( db, { name = ""; events = {}; slash = {}; userDB = {}; defaultDB = {}; statics = {}; });

	if ( db.name == "" ) or ( _G[db.name] ) then return end

	local addon = gLib:newAddon( db.name );
	
	addon:RegisterEvents(db.events);

	for i,v in pairs(db.slash) do
		addon:Slash( db.name.."_"..db.slash[i].name, db.slash[i].func, db.slash[i].slashes );
	end

	addon.defaults = db.defaultDB;
	addon.statics = db.statics;
	
	return addon, ArgCheck( db.userDB, db.defaultDB )
end

-- debugging
-- prints a list of currently handled gLib addons/objects
function gLib:list()
	for name, addon in pairs( Addon ) do
		print( ("%s - v%s - %s"):format(name, addon.VERSION, addon.NAME) );
	end
end

-- returns a list of the addon names
function gLib:GetList()
	return AddonList
end


------------------------------------------------------------------------------------------------------------
-- 	Slash commands
------------------------------------------------------------------------------------------------------------
Slash( nil, "gLib_Slash_Reload", 	function() ReloadUI(); end, 				{ "rl" } );
Slash( nil, "gLib_Slash_GM", 		function() ToggleHelpFrame(); end, 			{ "gm" } );
Slash( nil, "gLib_Slash_PPScale", 	function() PixelPerfectScale(); end, 		{ "ppscale" } );
Slash( nil, "gLib_Slash_LogFix", 	function() CombatLogClearEntries(); end, 	{ "fixlog" } );


------------------------------------------------------------------------------------------------------------
-- 	Tags
------------------------------------------------------------------------------------------------------------

Tags["money"] = function( money, full )
	money = money and tonumber(money) or GetMoney();

	local str;
	local g,s,c = floor(money/1e4), floor(money/100) % 100, money % 100;
	local pos = "0:0:0:0"

	if full then
		if ( g > 0 ) then str = ( str and str.." " or "" ) .. g .. CreateIcon( "Interface\\MoneyFrame\\UI-GoldIcon", pos ) end
		if ( s > 0 ) or ( g > 0 ) then str = ( str and str.." " or "" ) .. s .. CreateIcon( "Interface\\MoneyFrame\\UI-SilverIcon", pos ) end
		str = ( str and str.." " or "" ) .. c .. CreateIcon( "Interface\\MoneyFrame\\UI-CopperIcon", pos );
		return str
	else
		if ( g > 0 ) then str = ( str and str.." " or "" ) .. g .. CreateIcon( "Interface\\MoneyFrame\\UI-GoldIcon", pos ) end
		if ( s > 0 ) then str = ( str and str.." " or "" ) .. s .. CreateIcon( "Interface\\MoneyFrame\\UI-SilverIcon", pos ) end
		if ( c > 0 ) or ( ( g + s + c ) == 0 ) then str = ( str and str.." " or "" ) .. c .. CreateIcon( "Interface\\MoneyFrame\\UI-CopperIcon", pos ) end
		return str
	end
end
Tags["free"] = function( bags )
	if not bagstrings[bags] then return end
	return ( select( 1, GetBagSpace( bags ) ) )
end
Tags["max"] = function( bags )
	if not bagstrings[bags] then return end
	return ( select( 2, GetBagSpace( bags ) ) ) 
end
Tags["used"] = function( bags )
	if not bagstrings[bags] then return end
	return ( select( 3, GetBagSpace( bags ) ) )
end
Tags["freepercent"] = function( bags )
	if not bagstrings[bags] then return end
	return ( select( 4, GetBagSpace( bags ) ) ).."%"
end
Tags["usedpercent"] = function( bags )
	if not bagstrings[bags] then return end
	return ( 100 - ( select( 4, GetBagSpace( bags ) ) ) ).."%"
end
Tags["shortvalue"] = function( value )
	if not( value ) then return "0.0"; end
	
	value = tonumber( value );
	
	if value >= 1e6 then
		return ( "%.1fm" ):format( value / 1e6 ):gsub( "%.?0+([km])$", "%1" )
	elseif value >= 1e3 or value <= -1e3 then
		return ( "%.1fk" ):format( value / 1e3 ):gsub( "%.?0+([km])$", "%1" )
	else
		return value
	end	
end
Tags["smarttime"] = function( seconds )
	if not( seconds ) then return "0.0"; end
	
	seconds = tonumber( seconds );
	
	local day, hour, minute = 86400, 3600, 60
	if seconds >= day then
		return format( "%dd", floor( seconds / day + 0.5 ) ), seconds % day
	elseif seconds >= hour then
		return format( "%dh", floor( seconds / hour + 0.5 ) ), seconds % hour
	elseif seconds >= minute then
		if seconds <= minute * 5 then
			return format( "%d:%02d", floor( seconds / 60 ), seconds % minute ), seconds - floor( seconds )
		end
		return format( "%dm", floor( seconds / minute + 0.5 ) ), seconds % minute
	elseif seconds >= minute / 12 then
		return floor( seconds + 0.5 ), ( seconds * 100 - floor( seconds * 100 ) ) / 100
	end
	return format( "%.1f", seconds ), ( seconds * 100 - floor( seconds * 100 ) ) / 100
end
Tags["time"] = function( timezone )
	--[[
		time
			timezone can be; 
				game24/game (or nil) 	- 24 hour server/game time
				game12 		 		- 12 hour server/game time
				local24/local 		- 24 hour local time
				local12 				- 12 hour local time
				pvp 					- time until pvp wears off
				wintergrasp 			- time until next Wintergrasp
	]]--
	timezone = timezone and timezone:lower() or "game"

	if ( ( timezone == "local" ) or ( timezone == "local24" ) ) then
		return date( "%H:%M" )

	elseif ( timezone == "local12" ) then
		return date( "%I:%M %p" )

	elseif ( timezone == "game" ) or ( timezone == "game24" ) then
		local hours, minutes = GetGameTime();
		return ( format( "%02d", hours )..":"..format( "%02d", minutes ) )

	elseif ( timezone == "game12" ) then
		local hours, minutes = GetGameTime();
		return ( format( "%02d", ( ( hours > 12 ) or ( hours < 1 ) ) and ( hours - 12 ) or hours )..":"..format( "%02d", minutes )..( ( hours > 12 ) and " PM" or " AM" ) );

	elseif ( timezone == "pvp" ) then
		local sec = GetPVPTimer()
		if ( sec >= 0 ) and ( sec <= 301000 ) then
			seconds = floor( seconds / 1000 );
			return ( format( "%d", floor( seconds / 60 ) % 60 )..":"..format( "%02d", seconds % 60 ) )
		else
			return "--:--"
		end

	elseif ( timezone == "wintergrasp" ) then
		local seconds = GetWintergraspWaitTime();
		if seconds then
			return ( format( "%d", floor( seconds / 3600 ) )..":"..format( "%02d", floor( seconds / 60 ) % 60 )..":"..format( "%02d", seconds % 60 ) )
		else
			return ( "--:--:--" )
		end
	end
end
Tags["ap"] = function( unit )
	local base, posBuff, negBuff = UnitAttackPower( unit or "player" );
	return ( base + posBuff + negBuff )
end
Tags["rap"] = function( unit )
	local base, posBuff, negBuff = UnitRangedAttackPower( unit or "player" );
	return ( base + posBuff + negBuff )
end

--
-- Following ratings are for the unit "player" only
--
Tags["mcrit"] = function()
	return format( "%.2f",GetCritChance() )
end
Tags["mhaste"] = function()
	return format( "%.2f", GetCombatRatingBonus(18) )
end
Tags["rcrit"] = function()
	return format( "%.2f", GetRangedCritChance() )
end
Tags["rhaste"] = function()
	return format( "%.2f", GetCombatRatingBonus(19) )
end
Tags["sp"] = function()
	return max( GetSpellBonusDamage(2), GetSpellBonusDamage(3), GetSpellBonusDamage(4), GetSpellBonusDamage(5), GetSpellBonusDamage(6), GetSpellBonusDamage(7), GetSpellBonusHealing() )
end
Tags["scrit"] = function()
	return format( "%.2f",( max( GetSpellCritChance(2), GetSpellCritChance(3), GetSpellCritChance(4), GetSpellCritChance(5), GetSpellCritChance(6), GetSpellCritChance(7) ) ) )
end
Tags["shaste"] = function()
	return format( "%.2f", GetCombatRatingBonus(20) )
end


------------------------------------------------------------------------------------------------------------
-- 	gMods globals
------------------------------------------------------------------------------------------------------------
_GL_ACTIONBARS_HOTKEY_SHOW 			= 1;
_GL_ACTIONBARS_MACRO_SHOW 			= 1;

_GL_AURAS_SIZE 						= 27;
_GL_AURAS_ANCHOR 					= BuffFrame:GetAllPoints();
_GL_ENCHANTS_ANCHOR 				= TemporaryEnchantFrame:GetAllPoints();

_GL_COORDINATE_PRECISION 			= 1;

_GL_SHOW_TOOLTIPS_IN_COMBAT 		= true;

_GL_UI_PANEL_FONT 					= NumberFontNormalSmallGray;
_GL_UI_PANEL_FONT_SIZE 				= 12;
_GL_UI_PANEL_FONT_STYLE 			= "THINOUTLINE";

_GL_UI_PANEL_INDEXCOLOR 			= { 1.00, 1.00, 1.00 };
_GL_UI_PANEL_VALUECOLOR 			= { 1.00, 0.80, 0.20 };

_GL_UI_PANEL_BACKGROUNDCOLOR 		= { 0.00, 0.00, 0.00 };
_GL_UI_PANEL_BORDERCOLOR 			= { 0.15, 0.15, 0.15 };
_GL_UI_PANEL_OVERLAYCOLOR 			= { 0.35, 0.35, 0.35 };

_GL_MINIMAP_SIZE_DEFAULT 			= 150;
_GL_MINIMAP_SIZE 					= Minimap:GetWidth();
_GL_MINIMAP_POSITION 				= Minimap:GetAllPoints();
